"""
Lorenz factor correction for single crystal
"""

__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"


import numpy as np

class LorenzFactor:
    def __init__(self):
        """
        constructor
        """
        pass

    def LorenzCorrection(self,wavelength,hkl,powder=False):
        """
        Lorenz factor calculation for single crystals and powders
        
        @type wavelength: float
        @param wavelength: neutron wavelength
        @type hkl: numpy.ndarray
        @param hkl: HKL in AA^-1
        @type: boolean
        @param powder: switch single crystal/powder
        @return: Lorenz factor for single crystal
        @rtype: float
        """
        tmp=hkl*hkl
        dspacing=[1./np.sqrt(np.sum(tmp[i])) for i in range(np.shape(tmp)[0])]
        ttheta=2.*np.arcsin(wavelength/2./np.array(dspacing))
        if powder==False:
            return 1.0/np.sin(ttheta)
        else:
            return 1.0/np.sin(ttheta)/np.sin(ttheta/2.)

