#!/usr/bin/env python
"""
defines crystal system
N.B.:
The hexagonal family can be subdivided into two crystal systems: the hexagonal
and the trigonal.
The hexagonal system comprises 27 space groups. All of these have hexagonal Bravais lattices, labeled hP.
The trigonal system is the tricky one, because its 25 space groups (143-167) belong either to the hexagonal (hP, 18 space groups)
or the rhombohedral (hR, 7 space groups) Bravais lattice.
In addition, all trigonal crystals with rhombohedral lattices (space groups 146, 148, 155, 160, 161, 166, and 167),
can be represented as an equivalent hexagonal system: there is a choice of using a hexagonal or a rhombohedral setting.

Note that for the rhombohedral lattice the subgroup of translation is (1,0,0),(0,1,0),(0,0,1),(2/3,1/3,0)
"""
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"


from collections import OrderedDict

class CrystalSystem():
    """
    defines a dictionary that associates a crystal system to the space group number and to the point groups
    """
    def __init__(self):
        self.crystalsystemdict=OrderedDict()
        self.crystalsystemdict['Triclinic']={'sg#':[1,2],'PointGroups':['1','-1']} #from space group 1 to space group 2
        self.crystalsystemdict['Monoclinic']={'sg#':[3,15],'PointGroups':['2/m','m','2']}
        self.crystalsystemdict['Orthorhombic']={'sg#':[16,74],'PointGroups':['mmm','mm2','222']}
        self.crystalsystemdict['Tetragonal']={'sg#':[75,142],'PointGroups':['4/mmm','-42m','4mm','422','4/m','-4','4']}
        self.crystalsystemdict['Trigonal']={'sg#':[143,167],'PointGroups':['-3m','3m','32','-3','3']}
        self.crystalsystemdict['Hexagonal']={'sg#':[168,194],'PointGroups':['6/mmm','-62m','6mm','622','6/m','-6','6']}
        self.crystalsystemdict['Cubic']={'sg#':[195,230],'PointGroups':['m-3m','-43m','432','m-3','23']}
        return

if __name__ == "__main__":
    cs=CrystalSystem()
    for k, val in cs.crystalsystemdict.iteritems():
        print k, val
