#!/usr/bin/env python
"""
some utility functions to transform matrices to
symbols or vice-versa
"""
__author__ = "B. Roessli"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"

import numpy as np
import sys
from PointGroupMatrices import *

class OperationHelper():
    def __init__(self):
        pass

    def _Mat2Symbol(self,grouplist,crystalsystem,setting):
        """
        return symmetry symbol from matrix

        @param grouplist: matrices of symmetry operations
        @type grouplist: list
        @param crystalsystem: crystal system
        @type crystalsystem: string
        @param setting: Hexagonal vs. Rhombohedral
        @type setting: string
        @rtype: list
        @return: symbol of pointgroup operations
        """
        oplist = []
        if (crystalsystem=='Hexagonal'):
            pointgroupmatrices=self.PointGroupHMat
        elif ((crystalsystem=='Trigonal') and ((setting=='H') or (setting==None))):
            pointgroupmatrices=self.PointGroupHMat
        elif (crystalsystem=='Trigonal') and (setting=='R'):
            pointgroupmatrices=self.PointGroupRMat
        else:
            pointgroupmatrices=self.PointGroupRMat

        for k in range(0,len(grouplist)):
            found = False
            for key, val in pointgroupmatrices.iteritems():
                if (grouplist[k] == val).all():
                    oplist.append(key)
                    found = True
                    break
            if found == False:
                print 'symmetry element', grouplist[k], 'not found check list...'
                sys.exit(0)
        return oplist

    def PointGroupSymbol2Matrix(self,symop,crystalsystem,setting=None):
        """
        return matrix operation corresponding to point group symbol

        @param symop: point group symmetry operation
        @type symop: list
        @param crystalsystem: crystal system
        @type crystalsystem: string
        @param setting: Hexagonal vs. Rhombohedral
        @type setting: string
        @rtype: numpy.array
        @return: matrix of point group operation
        """
        if (crystalsystem=='Hexagonal'):
            R=self.PointGroupHMat[symop]
        elif ((crystalsystem=='Trigonal') and ((setting=='H') or (setting==None))):
            R=self.PointGroupHMat[symop]
        elif (crystalsystem=='Trigonal') and (setting=='R'):
            R=self.PointGroupRMat[symop]
        else:
            R=self.PointGroupRMat[symop]
        return R

    def _Seitz2Matrix(self,seitz,crystalsystem,setting):
        """
        return rotation and translation parts of a seitz symbol
        as numpy arrays

        @param seitz: Seitzsymbol
        @type seitz: string
        @param crystalsystem: crystal system
        @type crystalsystem: string
        @param setting: Hexagonal vs. Rhombohedral
        @type setting: string
        @rtype: numpy.array
        @return: matrix of space group operation
        """
        k=seitz
        W=k[k.find('{')+1:k.find('|')]
        t=k[k.find('(')+1:k.find(')')].split(',')
        mat=np.zeros((4,4))
        mat[0:3,0:3]=self.PointGroupSymbol2Matrix(W,crystalsystem,setting)
        mat[0:3,3]=map(float,t)
        mat[3][3]=1.0
        return mat


    def _SeitzSymbol(self,el,crystalsystem,setting):
        """
        @param el: symmetry operations as a 4x4 matrix
        @type el:  numpy.array
        @return:   rotation part and full Seitz symbol of symmetry operation
        @rtype:    string
        """
        if type(el) != np.ndarray:
            sys.exit('expected a numpy.array in _SeitzSymbol as input')
        for i in range(0,3):
            if np.abs(el[i,3]) >= 1.:
                el[i,3] = abs(float(el[i,3]) % int(el[i,3]))
            if el[i,3] < 0.:
                el[i,3]=el[i,3]+1.0
        rotpart=self._Mat2Symbol([el[0:3,0:3]],crystalsystem,setting)[0]

        seitzsymbol='{'+rotpart+'|(' +','.join([str(el[k,3]) for k in range(0,3)])+')}'
        return rotpart,seitzsymbol
