#!/usr/bin/env python
"""
Find symmetry elements that leave order parameter invariant

"""

__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"

try:
    import sympy as sp
except ImportError:
    sys.Exit('Sympy module could not be imported')
try:
    import numpy as np
except ImportError:
    sys.Exit('Numpy module could not be imported')


from Kvector import Kvector
from PointGroupIrreps import _PointGroupIrreps
from SpaceGroupIrreps import BetaMatrices
from SpaceGroupIrreps import SpaceGroupIrreps

import sys

class OrderParameter(SpaceGroupIrreps,Kvector,_PointGroupIrreps,BetaMatrices):
    def __init__(self,sglabel,k0):
        Kvector.__init__(self,sglabel,k0)
        _PointGroupIrreps.__init__(self,self.crystalsystem)
        BetaMatrices.__init__(self)

        #calculate irreducible representations of a space group
        gt=SpaceGroupIrreps(SpaceGroupLabel,kvector)
        #calculate allowed irreps of the group
        cosetsofg,littlegroupoperations,onBZ=gt.KVectorGroup()
        self.Irreps=gt.CalcSpaceGroupIrreps(cosetsofg,littlegroupoperations,onBZ)
        gt.PrintIrreps(self.Irreps)
            
        #order parameters
        #1dimensioanl order parameter 
        self.op1=dict()
        C = sp.Symbol('C')
        self.op1['C1']=sp.Matrix([C])
        #2d matrix
        self.op2=dict()
        C  = sp.Symbol('C')
        C1 = sp.Symbol('C1')
        C2 = sp.Symbol('C2')
        self.op2['C1']=sp.Matrix(([C,0]))
        self.op2['C2']=sp.Matrix(([C,C]))
        self.op2['C3']=sp.Matrix(([C,-C]))
        self.op2['C4']=sp.Matrix(([C1,C2]))

        #3d matrix
        self.op3=dict()
        C =  sp.Symbol('C')
        C1 = sp.Symbol('C1')
        C2 = sp.Symbol('C2')
        C3 = sp.Symbol('C3')
        self.op3['C1']=sp.Matrix(([C,0,0]))
        self.op3['C2']=sp.Matrix(([C,C,0]))
        self.op3['C3']=sp.Matrix(([C,-C,0]))
        self.op3['C4']=sp.Matrix(([C1,C2,0]))
        self.op3['C5']=sp.Matrix(([C,C,C]))
        self.op3['C6']=sp.Matrix(([C,C,-C]))
        self.op3['C7']=sp.Matrix(([C,C,C1]))
        self.op3['C8']=sp.Matrix(([C1,-C,C1]))
        self.op3['C9']=sp.Matrix(([C1,C2,C3]))


    def IsotropySubgroups(self):
        """
        Give irreducible representations 
        return symmetry elements that leave the 
        order parameter invariant 
        """
        inv_symop = dict()
        for G,val in self.Irreps.iteritems():
            opdict=dict()
            inv_symop[G] = dict()
            dim_mat = np.shape(val['{1|(0.0,0.0,0.0)}'])
            if (dim_mat == (1,1)):
                opdict=self.op1
            elif (dim_mat == (2,2)):
                opdict=self.op2
            elif (dim_mat == (3,3)):
                opdict=self.op3
            else:
                sys.exit('order parameter not defined for dimensions > 3')
            print '\nIrreducible Representation:', G
            print '*--order parameter direction:'
            for op, vec in opdict.iteritems():
                print op, vec
            for op,vec in opdict.iteritems():
                tmplistop=[]
                for symop,mat in val.iteritems():
                    res=np.around(mat,3)*vec-vec
                    try:
                        np.array(res.tolist()).astype(np.float64)
                        tmplistop.append(symop)
                    except TypeError:
                        pass
                inv_symop[G][op]=tmplistop
            
        print '\nIsotropy subgroups:'
        for G, val in inv_symop.iteritems():
            print '\n*--Irreducible Representation:', G
            for op, symop in val.iteritems():
                print op
                for symel in symop:
                    print symel
                
            
if __name__ == "__main__":
    SpaceGroupLabel='I 4/m'
    kvector=np.array([1.,0.,0.])
    op=OrderParameter(SpaceGroupLabel,kvector)
    op.IsotropySubgroups()
