#!/usr/bin/env python
"""
defines the rotation matrices for point groups
operations as dictionaries. Taken from the International Tables
for Crystallography (table 11.2.2.1 and 2)
"""
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"


import numpy as np

class PointGroupMatrices(object):
    def __init__(self):
        self.PointGroupRMat=dict()
        self.PointGroupRMat['1']          =   np.array([[ 1, 0, 0],[ 0, 1, 0],[ 0, 0, 1]])
        self.PointGroupRMat['3^+_xxx']    =   np.array([[ 0, 0, 1],[ 1, 0, 0],[ 0, 1, 0]])
        self.PointGroupRMat['3^-_xxx']    =   np.array([[ 0, 1, 0],[ 0, 0, 1],[ 1, 0, 0]])
        self.PointGroupRMat['-1']         =   np.array([[-1, 0, 0],[ 0,-1, 0],[ 0, 0,-1]])
        self.PointGroupRMat['-3^+_xxx']   =   np.array([[ 0, 0,-1],[-1, 0, 0],[ 0,-1, 0]])
        self.PointGroupRMat['-3^-_xxx']   =   np.array([[ 0,-1, 0],[ 0, 0,-1],[-1, 0, 0]])
        self.PointGroupRMat['2_00z']      =   np.array([[-1, 0, 0],[ 0,-1, 0],[ 0, 0, 1]])
        self.PointGroupRMat['2_0y0']      =   np.array([[-1, 0, 0],[ 0, 1, 0],[ 0, 0,-1]])
        self.PointGroupRMat['2_x00']      =   np.array([[ 1, 0, 0],[ 0,-1, 0],[ 0, 0,-1]])
        self.PointGroupRMat['3^+_x-x-x']  =   np.array([[ 0, 0,-1],[-1, 0, 0],[ 0, 1, 0]])
        self.PointGroupRMat['3^-_x-x-x']  =   np.array([[ 0,-1, 0],[ 0, 0, 1],[-1, 0, 0]])
        self.PointGroupRMat['2_xx0']      =   np.array([[ 0, 1, 0],[ 1, 0, 0],[ 0, 0,-1]])
        self.PointGroupRMat['2_x-x0']     =   np.array([[ 0,-1, 0],[-1, 0, 0],[ 0, 0,-1]])
        self.PointGroupRMat['4^+_00z']    =   np.array([[ 0,-1, 0],[ 1, 0, 0],[ 0, 0, 1]])
        self.PointGroupRMat['4^-_00z']    =   np.array([[ 0, 1, 0],[-1, 0, 0],[ 0, 0, 1]])
        self.PointGroupRMat['m_xy0']      =   np.array([[ 1, 0, 0],[ 0, 1, 0],[ 0, 0,-1]])
        self.PointGroupRMat['m_x0z']      =   np.array([[ 1, 0, 0],[ 0,-1, 0],[ 0, 0, 1]])
        self.PointGroupRMat['m_0yz']      =   np.array([[-1, 0, 0],[ 0, 1, 0],[ 0, 0, 1]])
        self.PointGroupRMat['-3^+_x-x-x'] =   np.array([[ 0, 0, 1],[ 1, 0, 0],[ 0,-1, 0]])
        self.PointGroupRMat['-3^-_x-x-x'] =   np.array([[ 0, 1, 0],[ 0, 0,-1],[ 1, 0, 0]])
        self.PointGroupRMat['m_x-xz']     =   np.array([[ 0,-1, 0],[-1, 0, 0],[ 0, 0, 1]])
        self.PointGroupRMat['m_xxz']      =   np.array([[ 0, 1, 0],[ 1, 0, 0],[ 0, 0, 1]])
        self.PointGroupRMat['-4^+_00z']   =   np.array([[ 0, 1, 0],[-1, 0, 0],[ 0, 0,-1]])
        self.PointGroupRMat['-4^-_00z']   =   np.array([[ 0,-1, 0],[ 1, 0, 0],[ 0, 0,-1]])
        self.PointGroupRMat['3^+_-xx-x']  =   np.array([[ 0, 0, 1],[-1, 0, 0],[ 0,-1, 0]])
        self.PointGroupRMat['3^-_-xx-x']  =   np.array([[ 0,-1, 0],[ 0, 0,-1],[ 1, 0, 0]])
        self.PointGroupRMat['2_x0x']      =   np.array([[ 0, 0, 1],[ 0,-1, 0],[ 1, 0, 0]])
        self.PointGroupRMat['2_-x0x']     =   np.array([[ 0, 0,-1],[ 0,-1, 0],[-1, 0, 0]])
        self.PointGroupRMat['4^+_0y0']    =   np.array([[ 0, 0, 1],[ 0, 1, 0],[-1, 0, 0]])
        self.PointGroupRMat['4^-_0y0']    =   np.array([[ 0, 0,-1],[ 0, 1, 0],[ 1, 0, 0]])
        self.PointGroupRMat['-3^+_-xx-x'] =   np.array([[ 0, 0,-1],[ 1, 0, 0],[ 0, 1, 0]])
        self.PointGroupRMat['-3^-_-xx-x'] =   np.array([[ 0, 1, 0],[ 0, 0, 1],[-1, 0, 0]])
        self.PointGroupRMat['m_-xyz']     =   np.array([[ 0, 0,-1],[ 0, 1, 0],[-1, 0, 0]])
        self.PointGroupRMat['m_xyx']      =   np.array([[ 0, 0, 1],[ 0, 1, 0],[ 1, 0, 0]])
        self.PointGroupRMat['-4^+_0y0']   =   np.array([[ 0, 0,-1],[ 0,-1, 0],[ 1, 0, 0]])
        self.PointGroupRMat['-4^-_0y0']   =   np.array([[ 0, 0, 1],[ 0,-1, 0],[-1, 0, 0]])
        self.PointGroupRMat['3^+_-x-xx']  =   np.array([[ 0, 0,-1],[ 1, 0, 0],[ 0,-1, 0]])
        self.PointGroupRMat['3^-_-x-xx']  =   np.array([[ 0, 1, 0],[ 0, 0,-1],[-1, 0, 0]])
        self.PointGroupRMat['2_0yy']      =   np.array([[-1, 0, 0],[ 0, 0, 1],[ 0, 1, 0]])
        self.PointGroupRMat['2_0y-y']     =   np.array([[-1, 0, 0],[ 0, 0,-1],[ 0,-1, 0]])
        self.PointGroupRMat['4^+_x00']    =   np.array([[ 1, 0, 0],[ 0, 0,-1],[ 0, 1, 0]])
        self.PointGroupRMat['4^-_x00']    =   np.array([[ 1, 0, 0],[ 0, 0, 1],[ 0,-1, 0]])
        self.PointGroupRMat['-3^+_-x-xx'] =   np.array([[ 0, 0, 1],[-1, 0, 0],[ 0, 1, 0]])
        self.PointGroupRMat['-3^-_-x-xx'] =   np.array([[ 0,-1, 0],[ 0, 0, 1],[ 1, 0, 0]])
        self.PointGroupRMat['m_xy-y']     =   np.array([[ 1, 0, 0],[ 0, 0,-1],[ 0,-1, 0]])
        self.PointGroupRMat['m_xyy']      =   np.array([[ 1, 0, 0],[ 0, 0, 1],[ 0, 1, 0]])
        self.PointGroupRMat['-4^+_x00']   =   np.array([[-1, 0, 0],[ 0, 0, 1],[ 0,-1, 0]])
        self.PointGroupRMat['-4^-_x00']   =   np.array([[-1, 0, 0],[ 0, 0,-1],[ 0, 1, 0]])

        #table 11.2.2.2 hexagonal coordinates
        self.PointGroupHMat=dict()
        self.PointGroupHMat['1']         =   np.array([[ 1, 0, 0],[ 0, 1, 0],[ 0, 0, 1]])
        self.PointGroupHMat['2_00z']     =   np.array([[-1, 0, 0],[ 0,-1, 0],[ 0, 0, 1]])
        self.PointGroupHMat['2_xx0']     =   np.array([[ 0, 1, 0],[ 1, 0, 0],[ 0, 0,-1]])
        self.PointGroupHMat['2_x-x0']    =   np.array([[ 0,-1, 0],[-1, 0, 0],[ 0, 0,-1]])
        self.PointGroupHMat['-1']        =   np.array([[-1, 0, 0],[ 0,-1, 0],[ 0, 0,-1]])
        self.PointGroupHMat['m_xy0']     =   np.array([[ 1, 0, 0],[ 0, 1, 0],[ 0, 0,-1]])
        self.PointGroupHMat['m_x-xz']    =   np.array([[ 0,-1, 0],[-1, 0, 0],[ 0, 0, 1]])
        self.PointGroupHMat['m_xxz']     =   np.array([[ 0, 1, 0],[ 1, 0, 0],[ 0, 0, 1]])
        self.PointGroupHMat['3^+_00z']   =   np.array([[ 0,-1, 0],[ 1,-1, 0],[ 0, 0, 1]])
        self.PointGroupHMat['6^+_00z']   =   np.array([[ 1,-1, 0],[ 1, 0, 0],[ 0, 0, 1]])
        self.PointGroupHMat['2_x00']     =   np.array([[ 1,-1, 0],[ 0,-1, 0],[ 0, 0,-1]])
        self.PointGroupHMat['2_x2x0']    =   np.array([[-1, 1, 0],[ 0, 1, 0],[ 0, 0,-1]])
        self.PointGroupHMat['-3^+_00z']  =   np.array([[ 0, 1, 0],[-1, 1, 0],[ 0, 0,-1]])
        self.PointGroupHMat['-6^+_00z']  =   np.array([[-1, 1, 0],[-1, 0, 0],[ 0, 0,-1]])
        self.PointGroupHMat['m_x2xz']    =   np.array([[-1, 1, 0],[ 0, 1, 0],[ 0, 0, 1]])
        self.PointGroupHMat['m_x0z']     =   np.array([[ 1,-1, 0],[ 0,-1, 0],[ 0, 0, 1]])
        self.PointGroupHMat['3^-_00z']   =   np.array([[-1, 1, 0],[-1, 0, 0],[ 0, 0, 1]])
        self.PointGroupHMat['6^-_00z']   =   np.array([[ 0, 1, 0],[-1, 1, 0],[ 0, 0, 1]])
        self.PointGroupHMat['2_0y0']     =   np.array([[-1, 0, 0],[-1, 1, 0],[ 0, 0,-1]])
        self.PointGroupHMat['m_2xx0']    =   np.array([[ 1, 0, 0],[ 1,-1, 0],[ 0, 0,-1]])
        self.PointGroupHMat['-3^-_00z']  =   np.array([[ 1,-1, 0],[ 1, 0, 0],[ 0, 0,-1]])
        self.PointGroupHMat['-6^-_00z']  =   np.array([[ 0,-1, 0],[ 1,-1, 0],[ 0, 0,-1]])
        self.PointGroupHMat['m_2xxz']    =   np.array([[ 1, 0, 0],[ 1,-1, 0],[ 0, 0, 1]])
        self.PointGroupHMat['m_0yz']     =   np.array([[-1, 0, 0],[-1, 1, 0],[ 0, 0, 1]])


class TransformedCoordinates():
    """
    coordinate transformations of symmetry operations in point groups
    """
    def __init__(self):
        self.PointGroupRcoordinates=dict()
        self.PointGroupRcoordinates['1']          =   ['+x','+y','+z']
        self.PointGroupRcoordinates['3^+_xxx']    =   ['+z','+x','+y']
        self.PointGroupRcoordinates['3^-_xxx']    =   ['+y','+z','+x']
        self.PointGroupRcoordinates['-1']         =   ['-x','-y','-z']
        self.PointGroupRcoordinates['-3^+_xxx']   =   ['-z','-x','-y']
        self.PointGroupRcoordinates['-3^-_xxx']   =   ['-y','-z','-x']
        self.PointGroupRcoordinates['2_00z']      =   ['-x','-y','+z']
        self.PointGroupRcoordinates['3^+_x-x-x']  =   ['-z','-x','+y']
        self.PointGroupRcoordinates['3^-_x-x-x']  =   ['-y','+z','-x']
        self.PointGroupRcoordinates['2_xx0']      =   ['+y','+x','-z']
        self.PointGroupRcoordinates['2_x-x0']     =   ['-y','-x','-z']
        self.PointGroupRcoordinates['4^+_00z']    =   ['-y','+x','+z']
        self.PointGroupRcoordinates['4^-_00z']    =   ['+y','-x','+z']
        self.PointGroupRcoordinates['m_xy0']      =   ['+x','+y','-z']
        self.PointGroupRcoordinates['-3^+_x-x-x'] =   ['+z','+x','-y']
        self.PointGroupRcoordinates['-3^-_x-x-x'] =   ['+y','-z','+x']
        self.PointGroupRcoordinates['m_x-xz']     =   ['-y','-x','+z']
        self.PointGroupRcoordinates['m_xxz']      =   ['+y','+x','+z']
        self.PointGroupRcoordinates['-4^+_00z']   =   ['+y','-x','-z']
        self.PointGroupRcoordinates['-4^-_00z']   =   ['-y','+x','-z']
        self.PointGroupRcoordinates['2_0y0']      =   ['-x','+y','-z']
        self.PointGroupRcoordinates['3^+_-xx-x']  =   ['+z','-x','-y']
        self.PointGroupRcoordinates['3^-_-xx-x']  =   ['-y','-z','+x']
        self.PointGroupRcoordinates['2_x0x']      =   ['+z','-y','+x']
        self.PointGroupRcoordinates['2_-x0x']     =   ['-z','-y','-x']
        self.PointGroupRcoordinates['4^+_0y0']    =   ['+z','+y','-x']
        self.PointGroupRcoordinates['4^-_0y0']    =   ['-z','+y','+x']
        self.PointGroupRcoordinates['m_x0z']      =   ['+x','-y','+z']
        self.PointGroupRcoordinates['-3^+_-xx-x'] =   ['-z','+x','+y']
        self.PointGroupRcoordinates['-3^-_-xx-x'] =   ['+y','+z','-x']
        self.PointGroupRcoordinates['m_-xyz']     =   ['-z','+y','-x']
        self.PointGroupRcoordinates['m_xyx']      =   ['+z','+y','+x']
        self.PointGroupRcoordinates['-4^+_0y0']   =   ['-z','-y','+x']
        self.PointGroupRcoordinates['-4^-_0y0']   =   ['+z','-y','-x']
        self.PointGroupRcoordinates['2_x00']      =   ['+x','-y','-z']
        self.PointGroupRcoordinates['3^+_-x-xx']  =   ['-z','+x','-y']
        self.PointGroupRcoordinates['3^-_-x-xx']  =   ['+y','-z','-x']
        self.PointGroupRcoordinates['2_0yy']      =   ['-x','+z','+y']
        self.PointGroupRcoordinates['2_0y-y']     =   ['-x','-z','-y']
        self.PointGroupRcoordinates['4^+_x00']    =   ['+x','-z','+y']
        self.PointGroupRcoordinates['4^-_x00']    =   ['+x','+z','-y']
        self.PointGroupRcoordinates['m_0yz']     =    ['-x','+y','+z']
        self.PointGroupRcoordinates['-3^+_-x-xx'] =   ['+z','-x','+y']
        self.PointGroupRcoordinates['-3^-_-x-xx'] =   ['-y','+z','+x']
        self.PointGroupRcoordinates['m_xy-y']     =   ['+x','-z','-y']
        self.PointGroupRcoordinates['m_xyy']      =   ['+x','+z','+y']
        self.PointGroupRcoordinates['-4^+_x00']   =   ['-x','+z','-y']
        self.PointGroupRcoordinates['-4^-_x00']   =   ['-x','-z','+y']

        #table 11.2.2.2 hexagonal coordinates
        self.PointGroupHcoordinates=dict()
        self.PointGroupHcoordinates['1']         =   ['+x','+y','+z']
        self.PointGroupHcoordinates['2_00z']     =   ['-x','-y','+z']
        self.PointGroupHcoordinates['2_xx0']     =   ['+y','+x','-z']
        self.PointGroupHcoordinates['2_x-x0']    =   ['-y','-x','-z']
        self.PointGroupHcoordinates['-1']        =   ['-x','-y','-z']
        self.PointGroupHcoordinates['m_xy0']     =   ['+x','+y','-z']
        self.PointGroupHcoordinates['m_x-xz']    =   ['-y','-x','+z']
        self.PointGroupHcoordinates['m_xxz']     =   ['+y','+x','+z']
        self.PointGroupHcoordinates['3^+_00z']   =   ['-y','+x-y','+z']
        self.PointGroupHcoordinates['6^+_00z']   =   ['+x-y','+x','+z']
        self.PointGroupHcoordinates['2_x00']     =   ['+x-y','-y','-z']
        self.PointGroupHcoordinates['2_x2x0']    =   ['-x+y','+y','-z']
        self.PointGroupHcoordinates['-3^+_00z']  =   ['+y','-x+y','-z']
        self.PointGroupHcoordinates['-6^+_00z']  =   ['-x+y','-x','-z']
        self.PointGroupHcoordinates['m_x2xz']    =   ['-x+y','+y','+z']
        self.PointGroupHcoordinates['m_x0z']     =   ['+x-y','-y','+z']
        self.PointGroupHcoordinates['3^-_00z']   =   ['-x+y','-x','+z']
        self.PointGroupHcoordinates['6^-_00z']   =   ['+y','-x+y','+z']
        self.PointGroupHcoordinates['2_0y0']     =   ['-x','-x+y','-z']
        self.PointGroupHcoordinates['m_2xx0']    =   ['+x','+x-y','-z']
        self.PointGroupHcoordinates['-3^-_00z']  =   ['+x-y','+x','-z']
        self.PointGroupHcoordinates['-6^-_00z']  =   ['-y','+x-y','-z']
        self.PointGroupHcoordinates['m_2xxz']    =   ['+x','-x-y','+z']
        self.PointGroupHcoordinates['m_0yz']     =   ['-x','-x+y','+z']
