#!/usr/bin/env python
"""
Irreducible representations and character table of point groups
Deprecated, do not use it
"""
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Deprecated"


from collections import OrderedDict
import copy
import cmath
import numpy as np
import os
import sys
from types import *

from CrystalSystem import CrystalSystem
from PointGroupMatrices import PointGroupMatrices

#sys.path.append(os.path.join(os.getcwd(),'..'))
from prettytable.prettytable import *

class PointGroups(CrystalSystem,PointGroupMatrices):
    """
    generators for non-cyclic point groups
    (Note: the generators are not unique)
    generator for cyclic point group are given by a single element
    and its powers G={g,g^2,g^3,...,g^n=e}
    """
    def __init__(self):
        """
        constructor
        """
        CrystalSystem.__init__(self)
        PointGroupMatrices.__init__(self)
        self.pointgroupgenerators=OrderedDict()
        #triclinic
        self.pointgroupgenerators[('1','C1')]=['1']
        self.pointgroupgenerators[('-1','Ci')]=['1','-1']

        #monoclinic (a,b,c)
        self.pointgroupgenerators[('2','C2')]=['1','2_00z']
        self.pointgroupgenerators[('m','Cs')]=['1','m_xy0']
        self.pointgroupgenerators[('2/m','C2h')]=['1','2_00z','-1']

        #orthorhombic (a,b,c)
        self.pointgroupgenerators[('222','D2')]=['1','2_00z','2_0y0']
        self.pointgroupgenerators[('mm2','C2v')]=['1','2_00z','m_x0z']
        self.pointgroupgenerators[('mmm','D2h')]=['1','2_00z','2_0y0','-1']

        #tetragonal c, (a,b), (a+b,a-b)
        self.pointgroupgenerators[('4','C4')]=['1','2_00z','4^+_00z']
        self.pointgroupgenerators[('-4','S4')]=['1','2_00z','-4^+_00z']
        self.pointgroupgenerators[('4/m','C4h')]=['1','2_00z','4^+_00z','-1']
        self.pointgroupgenerators[('422','D4')]=['1','2_00z','4^+_00z','2_0y0']
        self.pointgroupgenerators[('4mm','C4v')]=['1','2_00z','4^+_00z','m_x0z']
        self.pointgroupgenerators[('-42m','D2d')]=['1','2_00z','-4^+_00z','2_0y0']
        self.pointgroupgenerators[('4/mmm','D4h')]=['1','2_00z','4^+_00z','2_0y0','-1']
        #cubic
        self.pointgroupgenerators[('23','T')]=['1','2_00z','2_0y0','3^+_xxx']
        self.pointgroupgenerators[('m-3','Th')]=['1','2_00z','2_0y0','3^+_xxx','-1']
        self.pointgroupgenerators[('432','O')]=['1','2_00z','2_0y0','3^+_xxx','2_xx0']
        self.pointgroupgenerators[('-43m','Td')]=['1','2_00z','2_0y0','3^+_xxx','m_x-xz']
        self.pointgroupgenerators[('m-3m','Oh')]=['1','2_00z','2_0y0','3^+_xxx','2_xx0','-1']
        #trigonal c, (a,b,a+b), (2a+b, a+2b, -a+b)
        #hexagonal coordinates
        self.pointgroupgenerators[('3','C3')]=['1','3^+_00z']
        self.pointgroupgenerators[('-3','S6')]=['1','3^+_00z','-1']
        self.pointgroupgenerators[('321','D3')]=['1','3^+_00z','2_xx0']
        self.pointgroupgenerators[('312','D3')]=['1','3^+_00z','2_x-x0']
        self.pointgroupgenerators[('3m1','C3v')]=['1','3^+_00z','m_xx0']
        self.pointgroupgenerators[('31m','C3v')]=['1','3^+_00z','m_x-x0']
        self.pointgroupgenerators[('-3m1','D3d')]=['1','3^+_00z','2_xx0','-1']
        self.pointgroupgenerators[('-31m','D3d')]=['1','3^+_00z','2_x-x0','-1']
        #hexagonal
        self.pointgroupgenerators[('6','C6')]=['1','3^+_00z','2_00z']
        self.pointgroupgenerators[('-6','C3h')]=['1','3^+_00z','m_xy0']
        self.pointgroupgenerators[('6/m','C6h')]=['1','3^+_00z','2_00z','-1']
        self.pointgroupgenerators[('622','D6')]=['1','3^+_00z','2_00z','2_xx0']
        self.pointgroupgenerators[('6mm','C6v')]=['1','3^+_00z','2_00z','m_x-xz']
        self.pointgroupgenerators[('-62m','D3h')]=['1','3^+_00z','m_xy0','2_xx0']
        self.pointgroupgenerators[('6/mmm','D6h')]=['1','3^+_00z','2_00z','2_xx0','-1']

        self.pointgroupsymbol='1'
        self.pointgroupsymmetryoperations=[]
        self.pointgroupcompositionseries=[]
        self.conjugacyclasses=dict()
        self.cosetrepresentatives=set()
        self.IR=dict()
        self.IR[('1','C1')]=np.array([1])

    def _isinG(self,h,G):
        """
        return True if operation in group

        @type: numpy array
        @param: h symmetry operation
        @type: list
        @param: G (group)
        """
        isinG=False
        for j in range(0,len(G)):
            if np.array_equal(h,G[j]) == True:
                isinG=True
                break
        return isinG

    def _Mat2Symbol(self,grouplist,crystalsystem=None,setting=None):
        """
        return symmetry symbol from matrix

        @rtype: list
        @return: symbol of symmetry operations
        """
        oplist = []
        if crystalsystem==None:
            symbol=self.pointgroupsymbol[0]
            #find crystal system
            for k, val in self.crystalsystemdict.iteritems():
                for pg in val['PointGroups']:
                    if symbol==pg:
                        crystalsystem=k
                        break
        if (crystalsystem=='Hexagonal'):
            pointgroupmatrices=self.PointGroupHMat
        elif ((crystalsystem=='Trigonal') and ((setting=='H') or (setting==None))):
            pointgroupmatrices=self.PointGroupHMat
        elif (crystalsystem=='Trigonal') and (setting=='R'):
            pointgroupmatrices=self.PointGroupRMat
        else:
            pointgroupmatrices=self.PointGroupRMat

        for k in range(0,len(grouplist)):
            found = False
            for key, val in pointgroupmatrices.iteritems():
                if (grouplist[k] == val).all():
                    oplist.append(key)
                    found = True
                    break
            if found == False:
                print 'symmetry element', grouplist[k], 'not found check list...'
                sys.exit(0)
        return oplist

    def CharacterTable(self, Irreps):
        """
        print character table
        """
        labels=Irreps['G0'].keys()
        gorder=len(labels)
        labels.insert(0,'')
        charactertable=PrettyTable(labels)
        charT=dict() #to check column orthogohnality theorem
        for G, value in Irreps.iteritems():
            rowtable=[]
            rowtable.append(G)
            tmp=dict()
            for key, irrep in value.iteritems():
                if np.shape(irrep)[0] >= 2:
                    character=np.around(np.trace(irrep),3)
                else:
                    character=np.around(irrep[0][0],3)
                tmp[key]=character
            charT[G]=tmp
            for k in labels[1:]:
                rowtable.append(tmp[k])
            charactertable.add_row(rowtable)
        return charactertable,charT

    def CheckOrthogonality(self,charT):
        """
        check orthogonality relations for character table
        """
        gorder=len(self.pointgroupsymmetryoperations)
        #check row orthogonality
        passed_check=True
        for G1, irrep1 in charT.iteritems():
            for G2, irrep2 in charT.iteritems():
                res = 0.
                for symop in irrep1.iterkeys():
                    res+=np.dot(np.conj(np.array(irrep1[symop])),np.array(irrep2[symop]))
                realpart="{0:.2f}".format(res.real)
                imagpart="{0:.2f}".format(res.imag)
                res=float(realpart)+1j*float(imagpart)
                if G1 == G2:
                    if res != gorder:
                        print 'character table is not consistent with row orthogonality theorem'
                        passed_check=False
                else:
                    if res != 0:
                        print res
                        print 'character table is not consistent with row orthogonality theorem'
                        passed_check=False
        #check column for orthogonality
        classes=self.conjugacyclasses.keys()
        classel=dict()
        for key in classes:
            classel[key]=len(self.conjugacyclasses[key])
        for i in classes:
            for j in classes:
                total=0.0
                for G, value in charT.iteritems():
                    total+=value[i]*np.conj(value[j])
                realpart="{0:.2f}".format(total.real)
                imagpart="{0:.2f}".format(total.imag)
                total=float(realpart)+1j*float(imagpart)
                if i == j:
                    if total != gorder/classel[i]:
                        print 'character table is not consistent with column orthogonal theorem'
                        passed_check=False
                else:
                    if total != 0:
                        print i,j,total
                        print 'character table is not consistent with column orthogonal theorem'
                        passed_check=False
        return passed_check

    def CosetRepresentatives(self,H):
        """
        return the coset representative of group G relative to subgroup H

        @type: tuple
        @param: group label
        @rtype: list
        @return: coset representatives
        """
        hgen=self.pointgroupgenerators[H]
        try:
            ggen=self.pointgroupgenerators[self.pointgroupsymbol]
        except KeyError:
            for key in self.pointgroupgenerators.iterkeys():
                if key[0]==self.pointgroupsymbol or key[1]==self.pointgroupsymbol:
                    label=(key[0],key[1])
                    break
            ggen=self.pointgroupgenerators[label]
        self.cosetrepresentatives.clear()
        self.cosetrepresentatives=set('1')|(set(ggen)-set(hgen))
        #check that the coset representatives are complete
        #the number of cosets = G/N
        self.PointGroupOperations(hgen)
        hop=copy.copy(self.pointgroupsymmetryoperations)
        self.PointGroupOperations(ggen)
        gop=copy.copy(self.pointgroupsymmetryoperations)
        I=len(gop)/len(hop)
        if len(self.cosetrepresentatives) == I:
            pass
        else:
            #the coset representatives form a group (factor group)
            self.PointGroupOperations(list(self.cosetrepresentatives))
            self.cosetrepresentatives.clear()
            self.cosetrepresentatives=copy.copy(set(self.pointgroupsymmetryoperations))
        return

    def GetPointGroupGenerators(self,label):
        """
        return generators of point group

        @type label: point group label (HM or SChoenfliess)
        @param: string
        @rtype retval: generators
        @return: list
        """
        if (type(label) is StringType) or (type(label) is TupleType):
            self.pointgroupsymbol=label
        else:
            sys.exit('expected a string or a tuple type as input')
        retval = None

        if type(label)==TupleType:
            retval=self.pointgroupgenerators[label]
        else:
            for key in self.pointgroupgenerators.iterkeys():
                if key[0] == label or key[1] == label:
                    self.pointgroupsymbol=(key[0],key[1])
                    retval=self.pointgroupgenerators[self.pointgroupsymbol]
                    break
        if retval == None:
            print 'unrecognized point group symbol'
            self.PrintPointGroups()
        return retval

    def IndexOfSubgroup(self,H):
        """
        return index of subgroup

        @type: tuple
        @param: subgroup label
        @rtype: integer
        @return: index of subgroup
        """
        hgen=self.pointgroupgenerators[H]
        self.PointGroupOperations(hgen)
        horder=len(self.pointgroupsymmetryoperations)
        try:
            ggen=self.pointgroupgenerators[self.pointgroupsymbol]
        except:
            for key in self.pointgroupgenerators.iterkeys():
                if key[0]==self.pointgroupsymbol or key[1]==self.pointgroupsymbol:
                    label=(key[0],key[1])
                    break
            ggen=self.pointgroupgenerators[label]

        self.PointGroupOperations(ggen)
        gorder=len(self.pointgroupsymmetryoperations)
        return gorder/horder

    def InducedRepresentations(self,Irrep,k0=None,lat2cart=None):
        """
        def: induced representation with matrix elements D^{ind,j})_{mt,ns}
        H<G
        D^j(H) is an irrep of H
        D^{ind,j}(g)_{mt,ns}=D^j(g^-1_m*h*g_n)_{t,s} if g^-1_m*g*g_n=h with h in H, 0 otherwise
        for all g in G forms a representation of G
        (aroyo, p. 22)
        """
        # N=normal subgroup of pointgroup
        ngen=Irrep.keys()
        self.PointGroupOperations(ngen)
        Nop=self.pointgroupsymmetryoperations
        gop=self.SemiDirectProduct(Nop,list(self.cosetrepresentatives))
        Gop=[item for sublist in gop for item in sublist]
        IndRep=[]
        m=len(Gop)/len(Nop)
        n=len(Gop)/len(Nop)
        (t,s)=np.shape(Irrep['1'])
        dim=(m*t,n*s)
        Dind=dict()
        cosets=list(self.cosetrepresentatives)
        for g in Gop:
            g_mat=self.PointGroupSymbol2Matrix(g)
            m=1
            dinduced=np.zeros(dim,dtype='complex')
            for gm in cosets:
                try:
                    gm_mat=self.PointGroupSymbol2Matrix(gm)
                    tm=np.array([0.,0.,0.])
                except KeyError: #coset has translation
                    op=gm.split('|')
                    gm_mat=self.PointGroupSymbol2Matrix(op[0])
                    tmp=op[1]
                    tmp=tmp.replace('(','')
                    tmp=tmp.replace(')','')
                    tmp=tmp.split(',')
                    tm=np.array(tmp,dtype='float')
                n=1
                for gn in cosets:
                    try:
                        gn_mat=self.PointGroupSymbol2Matrix(gn)
                        tn=np.array([0.,0.,0.])
                    except KeyError:
                        op=gn.split('|')
                        gn_mat=self.PointGroupSymbol2Matrix(op[0])
                        tmp=op[1]
                        tmp=tmp.replace('(','')
                        tmp=tmp.replace(')','')
                        tmp=tmp.split(',')
                        tn=np.array(tmp,dtype='float')
                    gm_inv=np.linalg.inv(gm_mat)
                    O=np.dot(gm_inv,np.dot(g_mat,gn_mat))
                    #conjugation of the translation part
                    trans=np.dot(gm_inv,np.dot(g_mat,tn))-np.dot(gm_inv,tm)
                    try:
                        factor=np.exp(-1j*np.dot(k0,np.dot(lat2cart,trans)))
                        #round to 2 decimals
                        realpart="{0:.2f}".format(factor.real)
                        imagpart="{0:.2f}".format(factor.imag)
                        factor=float(realpart)+1j*float(imagpart)
                    except TypeError:
                        factor = 1.0
                    Osymbol=self._Mat2Symbol([O])
                    if Osymbol[0] in Nop:
                        D=factor*Irrep[Osymbol[0]]
                        for indx in range(1,t+1):
                            for indy in range(1,s+1):
                                dinduced[m*indx-1][n*indy-1]=D[indx-1][indy-1]
                    n=n+1
                m=m+1
            Dind[g]=dinduced
        IndRep.append(Dind)
        return IndRep

    def Irreps(self,Ds,Orbit,indexofsubgroup,k0=None,lat2cart=None):
        """
        calculate induced irreps from normal subgroup

        @param: D = irreps of normal subgroup
        @type: dictionary
        @param: orbit
        @type: dictionary
        @param: index of subgroup
        @type: integer
        """
        k=0
        Dirr=dict()
        coset=self.cosetrepresentatives
        for key,values in Orbit.iteritems():
            L=len(values)
            if L == 1:
                """
                if the length of the orbit is 1, the irreps are self-conjugate
                and the induction procedure from the subgroup
                will give a reducible representation, which
                can be reduced using
                1.- D(r^-1*n*r)=R^-1*D(n)*R
                2.- D(r^p)=R^p
                and the irreps are given by:
                1.- D(n)_q = D(n)
                2.- D(r)_q = alfa *R (alfa = exp(2*pi*q*i/p) q=0,1,...,(p-1))
                see Bradley, Cracknell p.199
                """
                irrep=Ds[list(values)[0]]
                dim_irrepG=np.shape(irrep['1'])
                #choose coset
                for c in list(coset):
                    if c == '1':
                        pass
                    else:
                        cosetmat=self.PointGroupSymbol2Matrix(c)
                        break
                hop=Ds[list(values)[0]].keys()
                CX=np.zeros((dim_irrepG[0],dim_irrepG[1]),dtype=complex)
                #calculate matrix A in Klauder and Gay
                #see also eq. 4.5.51 in Bradley and Cracknell
                #Any matrix will do as long as CX (below) is not zero
                #if the coset is the inversion then CX=1 as g commutes with all g
                if self._Mat2Symbol([cosetmat])[0] == '-1':
                    CX=np.eye(dim_irrepG[0],dim_irrepG[1],dtype=complex)
                else:
                    Xmat=np.ones((dim_irrepG[0],dim_irrepG[1]),dtype=complex)
                    #Xmat=np.eye(dim_irrepG[0],dim_irrepG[1],dtype=complex)
                    for h in hop:
                        #qrq^-1
                        r=self.PointGroupSymbol2Matrix(h)
                        qinv=np.linalg.inv(cosetmat)
                        q=cosetmat
                        el=self._Mat2Symbol([np.dot(q,np.dot(r,qinv))])
                        Dconj=irrep[el[0]]
                        rinv=np.linalg.inv(r)
                        el=self._Mat2Symbol([rinv])
                        Dinv=irrep[el[0]]
                        CX=CX+np.dot(Dconj,np.dot(Xmat,Dinv))
                if indexofsubgroup == 2:
                    gamma1=dict()
                    gamma2=dict()
                    for key in irrep.iterkeys():
                        gamma1[key]=irrep[key]
                        gamma2[key]=irrep[key]
                    #normalise CX
                    q=list(coset)[1]
                    qmat=self.PointGroupSymbol2Matrix(q)
                    q2mat=np.dot(qmat,qmat)
                    q2symbol=self._Mat2Symbol([q2mat])
                    D_r=irrep[q2symbol[0]]
                    CX2=np.dot(CX,CX)
                    if np.array_equal(CX2,D_r) == True:
                        l=1.0
                    else:
                        indx=np.nonzero(CX2)[0][0]
                        l=cmath.sqrt(D_r[0][0]/CX2[0][0])
                    gamma1[q]=l*CX
                    Dirr['G'+str(k)]=gamma1
                    gamma2[q]=-l*CX
                    k=k+1
                    Dirr['G'+str(k)]=gamma2
                if indexofsubgroup == 3:
                    gamma1=dict()
                    gamma2=dict()
                    gamma3=dict()
                    for key in irrep.iterkeys():
                        gamma1[key]=irrep[key]
                        gamma2[key]=irrep[key]
                        gamma3[key]=irrep[key]
                    q=list(coset)[1]
                    qmat=self.PointGroupSymbol2Matrix(q)
                    qmat3=np.dot(qmat,np.dot(qmat,qmat))
                    q3symbol=self._Mat2Symbol([qmat3])
                    D_r=irrep[q3symbol[0]]
                    CX3=np.dot(CX,np.dot(CX,CX))
                    if np.array_equal(CX3,D_r) == True:
                        l=1.0
                    else:
                        indx=np.nonzero(CX3)[0][0]
                        l=np.power(D_r[0][0]/CX3[0][0],1./3.)
                    gamma1[q]=CX*l
                    Dirr['G'+str(k)]=gamma1

                    k=k+1
                    gamma2[q]=np.exp(1j*2.*np.pi/3.)*CX*l
                    Dirr['G'+str(k)]=gamma2

                    k=k+1
                    gamma3[q]=np.exp(-1j*2.*np.pi/3.)*CX*l
                    Dirr['G'+str(k)]=gamma3
            else:
                #use induction theorem
                #select allowed (small) irrep from orbit
                #N.B.: all members of an orbit have conjugate little groups -> conjugate irreps
                #choose one irrep from orbit
                irr=str(key[2:-1])
                inducedIrrep=self.InducedRepresentations(Ds[irr],k0,lat2cart)
                for ind in range(0,len(inducedIrrep)):
                    Dirr['G'+str(k)]=inducedIrrep[ind]
                    k=k+1
                k=k-1
            k=k+1
        #complete set of Irreps of the group using the multiplication table
        G=self.pointgroupsymbol
        try:
            ggen=self.pointgroupgenerators[G]
        except:
            for key in self.pointgroupgenerators.iterkeys():
                if key[0]==G or key[1]==G:
                    label=(key[0],key[1])
                    break
            ggen=self.pointgroupgenerators[label]
        self.PointGroupOperations(ggen)
        oplist=self.pointgroupsymmetryoperations
        multiplicationtable=self.PointGroupMultiplicationTable(oplist)
        GIrreps=Dirr
        #group generators
        tmp=Dirr.keys()
        for indx in range(0,len(Dirr)):
            Ggen=Dirr[tmp[indx]].keys()
            if set(Ggen) == set(oplist):
                pass
            else:
                Ggen.remove('1')
                for gel in oplist:
                    if gel not in Dirr[tmp[indx]].keys():
                        #find element in multiplication table
                        for key, value in multiplicationtable.iteritems():
                            if value == gel:
                                if '1' not in key:
                                    #find two elements in the multiplication table that
                                    #are in the symmetry elements of the irreps
                                    if (key[0] in Ggen) and (key[1] in Ggen):
                                        for k in Dirr.keys():
                                            mat=np.dot(Dirr[k][key[0]],Dirr[k][key[1]])
                                            GIrreps[k][gel]=mat
        return GIrreps

    def LittleGroup(self,Orbit,N):
        """
        the Little Group, G^s, is the subset of G that conjugate D^(s)(H) onto an equivalent representation
        and forms a group
        N.B. shortcuts
        1.- the orbit has length 1 then G^s=G
        2.- the orbit has length 2 or 3 then G^s=H
        """
        for key,values in Orbit.iteritems():
            L=len(values)
            if L==1:
                print 'Little group of',list(values)[0],'relative to',self.pointgroupsymbol,'is:', self.pointgroupsymbol
            elif (L==2) or (L==3):
                print 'Little group of',','.join([k for k in sorted(list(values)[0])]),'relative to',self.pointgroupsymbol,'is:', N
            else:
                sys.exit('could not determine the little group...')
        return

    def Orbit(self,Irreps,k0=None,lat2cart=None):
        """
        return the orbit (i.e. maximal set of mutually inequivalent representations conjugated with respect to G)
        by conjugation of the elements of H
        with the coset representatives of G relative to H
        (Aroyo, Representation of Crystallographic Groups, p.26)
        N.B.: It may be shown that members of a sam eorbit induce equivalent irreps
        @type:   dictionary
        @input:  Irreducible representations of subgroup
        as a list of dictionaries with key = symmetry operation, value = irreps
        e.g. 'manual' input for group '4' would be
        Irreps=[]
        d={'1':np.array([[1]]),'4^+_00z':np.array([[1]]), '2_00z':np.array([[1]]),'4^-_00z':np.array([[1]])}
        Irreps.append(d)
        d={'1':np.array([[1]]),'4^+_00z':np.array([[-1]]),'2_00z':np.array([[1]]),'4^-_00z':np.array([[-1]])}
        Irreps.append(d)
        d={'1':np.array([[1]]),'4^+_00z':np.array([[1J]]),'2_00z':np.array([[-1]]),'4^-_00z':np.array([[-1J]])}
        Irreps.append(d)
        d={'1':np.array([[1]]),'4^+_00z':np.array([[-1J]]),'2_00z':np.array([[-1]]),'4^-_00z':np.array([1J]])}
        Irreps.append(d)

        @param: subgroup label
        @type: string
        @return: orbit relative to point group G
        @rtype:dictionary
        """
        Ds=dict()
        #symmetry operations
        Nop=Irreps['G0'].keys()
        for Gi, irrep in Irreps.iteritems():
            dcoset=dict()
            for coset in self.cosetrepresentatives:
                d=dict()
                try:
                    R=self.PointGroupSymbol2Matrix(coset)
                    for h in Nop:
                        M=self.PointGroupSymbol2Matrix(h)
                        O=self._Mat2Symbol([np.dot(np.linalg.inv(R),np.dot(M,R))])
                        d[h]=irrep[O[0]]
                except KeyError:   #if coset has translations
                    op=coset.split('|')
                    R=self.PointGroupSymbol2Matrix(op[0])
                    t=op[1]
                    t=t.replace('(','')
                    t=t.replace(')','')
                    t=t.split(',')
                    tpart=np.array(t,dtype='float')
                    for h in Nop:
                        M=self.PointGroupSymbol2Matrix(h)
                        #conjugation of the rotation part
                        O=self._Mat2Symbol([np.dot(np.linalg.inv(R),np.dot(M,R))])
                        #conjugation of the translation part
                        Rinv=np.linalg.inv(R)
                        t=np.dot(Rinv,np.dot(M,tpart))-np.dot(Rinv,tpart)
                        factor=np.exp(-1j*np.dot(k0,np.dot(lat2cart,t)))
                        #round to 2 decimals
                        realpart="{0:.2f}".format(factor.real)
                        imagpart="{0:.2f}".format(factor.imag)
                        factor=float(realpart)+1j*float(imagpart)
                        d[h]=factor*irrep[O[0]]
                #conjugate representation
                dcoset[coset]=d
            Ds[Gi]=dcoset
        #orbits
        Orbit=dict()
        cosetlist=Ds['G0'].keys()
        for Gi, irrep in Irreps.iteritems():
            orbit=set()
            for coset in cosetlist:
                for label in Ds.iterkeys(): #loop over conjugate irreps
                    inorbit=True
                    for symop_Gi in irrep.iterkeys():
                        if np.shape(Ds[label][coset][symop_Gi]) != np.shape(irrep[symop_Gi]):
                            inorbit=False
                            break
                        if np.array_equal(np.around(Ds[label][coset][symop_Gi],3),np.around(irrep[symop_Gi],3))==False:
                            inorbit=False
                            break
                    if inorbit == True:
                        #orbit.add((key,coset))
                        orbit.add(label)
            Orbit['O('+Gi+')']=orbit
        #remove duplicates
        dic=dict()
        for k,v in Orbit.iteritems():
            if v not in dic.values():
                dic[k]=v
        return dic

    def PointGroupCompositionSeries(self):
        """
        Point groups are solvable, so there exists a composition series
        1 <| Z_1 <| Z_2 <|... <| G (index 2 or 3)
        that can be obtained from the generator list (per construction),
        e.g.
        1 <| 3 <| 6 <| 622 <| 6/mmm
        1 <| 2<| 222 <| 23 <| 432 <| m-3m
        """
        self.pointgroupcompositionseries[:]=[]
        try:
            generatorlist=self.pointgroupgenerators[self.pointgroupsymbol]
        except KeyError:
            for key in self.pointgroupgenerators.iterkeys():
                if key[0]==self.pointgroupsymbol or key[1]==self.pointgroupsymbol:
                    label=(key[0],key[1])
                    break
            generatorlist=self.pointgroupgenerators[label]
        for k in range(len(generatorlist),0,-1):
            for key, value in self.pointgroupgenerators.iteritems():
                if set(generatorlist) == set(value):
                    self.pointgroupcompositionseries.append(key)
            generatorlist=generatorlist[0:-1]
        return

    def PointGroupConjugacyClasses(self,multiplicationtable):
        """
        uses theorem:
        distinct alements a and b in G are conjugate (and belong to the
        same class) if and only if there are elements u and v in G such
        that a=uv and b=vu
        (S.W. Golomb, Mathematics Magazine, 69, 373 (1996))

        @param:  multiplicationtable
        @rtpe:   dictionary
        @return: conjugacy classes
        @rtype:  list
        """
        self.conjugacyclasses.clear()
        for key in self.pointgroupsymmetryoperations:
            self.conjugacyclasses[key]=[key]
        for key0 in self.pointgroupsymmetryoperations:
            for key1 in self.pointgroupsymmetryoperations:
                u=key0
                v=key1
                el1=multiplicationtable[(u,v)]
                el2=multiplicationtable[(v,u)]
                if el1 != el2:
                    try:
                        if el2 not in self.conjugacyclasses[el1]:
                            self.conjugacyclasses[el1].append(el2)
                            del self.conjugacyclasses[el2]
                        if el1 not in self.conjugacyclasses[el2]:
                            self.conjugacyclasses[el2].append(el1)
                            del self.conjugacyclasses[el1]
                    except:
                        pass

        return

    def PointGroupIrreps(self, pgsymbol):
        if pgsymbol != 'CoGroup':
            print 'point group: ', pgsymbol
            generators=self.GetPointGroupGenerators(pgsymbol)
            print 'point group generators:', generators
            self.PointGroupOperations(generators)
            pgoperations=self.pointgroupsymmetryoperations
            print 'number of point group operations:', len(pgoperations)
            print 'point group operations:',pgoperations
            table=self.PointGroupMultiplicationTable(pgoperations)
            self.PrintTable(table,pgoperations)
            self.PointGroupConjugacyClasses(table)
            print 'conjugacy classes:'
            for key, value in self.conjugacyclasses.iteritems():
                print '{',', '.join(value),'}'
            #print 'maximum subgroup order:', max(self.SubGroupOrder())
            self.PointGroupCompositionSeries()
            #print 'compositionseries:'
            #print ' <|'.join([x[0] for x in compositionseries]), ' or ', ' <|'.join([x[1] for x in compositionseries])
            #print 'group-subgroup order and cosets of subgroups'
        else:
            pass
        #e.g. irreps of normal subgroup '1'
        D=dict()
        D['G0']={'1':np.array([[1]])}
        for k in range(len(self.pointgroupcompositionseries)-1,0,-1):
            self.pointgroupsymbol=self.pointgroupcompositionseries[k-1]
            N=self.pointgroupcompositionseries[k]
            index=self.IndexOfSubgroup(N)
            #print 'Normal subgroup of ', self.pointgroupsymbol,': ', N, ' with index', index
            self.CosetRepresentatives(N)
            print 'Coset representatives of', self.pointgroupsymbol, ' relative to ', N, ':','{',', '.join(self.cosetrepresentatives),'}'
            O=self.Orbit(D)
            print 'Orbits of',N,'relative to',self.pointgroupsymbol,':',O
            #self.LittleGroup(O,N)
            #print 'Irreducible representations of the group', self.pointgroupsymbol
            D=self.Irreps(D,O,index)
        return D

    def PrintIrreps(self,d):
        for g,irrep in d.iteritems():
            print g
            for sym,mat in irrep.iteritems():
                print sym
                print('\n'.join([''.join(['{:4}'.format(item) for item in row])
                      for row in mat]))
                #print np.around(mat,3)
                print
            print'--------------------------------------------'
            print

    def PrintCharacterTable(self,d):
        #character table
        print 'character table:'
        charactertable,charT=self.CharacterTable(d)
        if self.CheckOrthogonality(charT)==True:
            print '(passed row/column orthogonality checks)'

    def PointGroupMultiplicationTable(self,symoplist):
        """
        print and return multiplication table of point group

        @param symoplist: list of symbols of point group operations, e.g. 2_00z, 4^+_00z,...
        @type param:      list of strings
        @return: multiplication table
        @rtype:  dictionary
        """
        multiplicationtable=dict()
        g=[]
        for g1 in symoplist:
            gmat1=self.PointGroupSymbol2Matrix(g1)
            for g2 in symoplist:
                gmat2=self.PointGroupSymbol2Matrix(g2)
                g.append(np.dot(gmat1,gmat2))
        glist=self._Mat2Symbol(g)
        l=0
        for g1 in symoplist:
            for g2 in symoplist:
                multiplicationtable[(g1,g2)]=glist[l]
                l+=1
        return multiplicationtable

    def PointGroupOperations(self,generatorslist):
        """
        Create all symmetry operations from generator list
        rule: only terms _j or g_j^2 occur and the next generator is chosen as soon as
        g_j^j is in G_{j-1} (ITC 8.3.5)

        @param generatorlist: generators
        @type: list
        @rtype: list
        @return: symbols of symmetry operations in point group
        """
        genlist=copy.copy(generatorslist)
        # point groups have maximum 6 generators
        for k in range(len(generatorslist)+1,7):
            genlist.append('1')
        generators=[]
        for g in genlist:
            generators.append(self.PointGroupSymbol2Matrix(g))
        grouplist=[]
        grouplist.append(np.eye((3)))
        for k0 in range(0,3):
            mat0=np.linalg.matrix_power(generators[0],k0)
            for k1 in range(0,3):
                mat1=np.linalg.matrix_power(generators[1],k1)
                for k2 in range(0,3):
                    mat2=np.linalg.matrix_power(generators[2],k2)
                    for k3 in range(0,3):
                        mat3=np.linalg.matrix_power(generators[3],k3)
                        for k4 in range(0,3):
                            mat4=np.linalg.matrix_power(generators[4],k4)
                            for k5 in range(0,3):
                                mat5=np.linalg.matrix_power(generators[5],k5)
                                mat=np.dot(mat0,np.dot(mat1,np.dot(mat2,np.dot(mat3,np.dot(mat4,mat5)))))
                                if self._isinG(mat,grouplist) == False:
                                    grouplist.append(mat)
        self.pointgroupsymmetryoperations[:]=[]
        self.pointgroupsymmetryoperations=sorted(self._Mat2Symbol(grouplist))
        return

    def PointGroupSymbol2Matrix(self,gen):
        """
        return matrix operation corresponding to generator

        @type gen: generator e.g 2_x,m_z,-1
        @param: string
        @rtype R: symmetry operation in matrix form
        @return: numpy array
        """
        symbol=self.pointgroupsymbol[0]
        if ('3' in symbol[0]) or ('-3' in symbol[0:1]) or ('6' in symbol[0]) or ('-6' in symbol[0:2]):
            R=self.PointGroupHMat[gen]
        else:
            R=self.PointGroupRMat[gen]
        return R

    def PrintPointGroups(self):
        """
        Print crystallographic point groups and generators
        """
        table=PrettyTable(["hermann-maugin","schoenflies","generators"])
        table.padding_width=1
        print
        print 'the generators of the crystallographic point groups'
        for key,values in self.pointgroupgenerators.iteritems():
            table.add_row([key[0],key[1],', '.join(values)])
        print table
        sys.exit(0)

    def PrintTable(self,table,symoplist):
        """
        print multiplcation table
        """
        tm=table
        labels = [' ']
        for key in symoplist:
            labels.append(key)
        table=PrettyTable(labels)
        table.padding_width=1
        #table.float_format="+5.3"
        print 'multiplication table:'
        for k in labels[1:]:
            row=[k]
            for i in labels[1:]:
                row.append(tm[(str(k),str(i))])
            table.add_row(row)
        print table

    def SemiDirectProduct(self,Hop,cosets):
        """
        calculate symmetry operations of group from subgroup
        elements and coset representatives

        @type: list
        @param: symmetry operations of the normal subgroup
        @type: list
        @param: coset representative
        @rtype: list
        @return: symmetry operations of group
        """
        gop=[]
        for c in cosets:
            cmat=self.PointGroupSymbol2Matrix(c)
            for h in Hop:
                hmat=self.PointGroupSymbol2Matrix(h)
                gmat=np.dot(cmat,hmat)
                gop.append(self._Mat2Symbol([gmat]))
        return gop

    def SubGroupOrder(self):
        """
        Lagrange theorem: the order of a subgroup divides the order ot the group
        @rtype: list
        @return possible subgroup order
        """
        gorder=len(self.pointgroupsymmetryoperations)
        gorder=float(gorder)
        subgrouporder=[]
        for k in range(2,int(gorder)):
            if int(gorder/k)==gorder/k:
                subgrouporder.append(k)
        else:
            subgrouporder.append(2)
        return subgrouporder

if __name__ == "__main__":

    pg=PointGroups()
    #pgsymbol='432'
    #pgsymbol='23'
    #pgsymbol='622'
    #pgsymbol='222'
    #pgsymbol='4'
    #pgsymbol='m-3m'
    #pgsymbol='6/mmm'
    #pgsymbol='m-3'
    #pgsymbol='6/m'
    #pgsymbol='-31m'
    #pgsymbol='3m1'
    #pgsymbol='2'
    #pgsymbol='4mm'
    #pgsymbol='4/mmm'
    #pgsymbol='mm2'
    #pgsymbol='m-3m'
    #pgsymbol='-43m'
    #pgsymbol='-3m1'
    pgsymbol='4/m'
    Irreps=pg.PointGroupIrreps(pgsymbol)
    pg.PrintIrreps(Irreps)
    pg.PrintCharacterTable(Irreps)
