#!/usr/bin/env python
"""
build magnetic representation
"""
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"

"""
Magnetic representation built from result
of symmetry opreation on magnetic moment, i.e.
G = V x D with
V = Permutation representation
D = magnetic symmetry operation
"""
from PermutationRep import PermutationRep
from PermutationRep import WyckoffPositions
from SpaceGroupIrreps import SpaceGroupIrreps
from prettytable import *

import copy
import numpy as np
import sys

class Found(Exception): pass

class VectorRep(SpaceGroupIrreps,PermutationRep,WyckoffPositions):
    def __init__(self,spacegrouplabel,kvector,atomcoord,axialvector=True):
        self.kvector=kvector
        WyckoffPositions.__init__(self,spacegrouplabel=spacegrouplabel,atomicposition=atomcoord)
        self.AtomicPositions()
        SpaceGroupIrreps.__init__(self,spacegrouplabel,kvector)
        self.cosetsrepoflg, littlegroupoperations, onBZ = self.KVectorGroup()

        #we need the allowed irreps of the little group so
        #set the coset representatives to contain only the unit element;
        #in that case the induction procedure will calculate the same matrices
        #for the induced representations and those of the subgroup
        cosetsofg=['{1|(0.0,0.0,0.0)}']
        irreps=self.CalcSpaceGroupIrreps(cosetsofg,littlegroupoperations,onBZ)
        self.Gm = ''                    # decomposition of the vector representation
        self.symmetrygroup=[]
        if axialvector == True:
            print '\n! Axial vector was chosen'
        else:
            print '\n! Polar vector was chosen'
        if self.isFedorovGroup == True or self.isGreyGroup == True or axialvector == False:  #phonon case
            print '\n! Time reversal is irrelevant --> representation analysis\n'
            #irreps
            #------
            self.gkirreduciblerepresentations=irreps
            self.PrintIrreps(self.gkirreduciblerepresentations)
            charactertable,self.characterlittlegroup=self.CharacterTable(self.gkirreduciblerepresentations)
            print 'character table:'
            print charactertable
            for k in littlegroupoperations:
                tmp=dict()
                tmp['seitzsymbol']=k
                tmp['matrix']=self._Seitz2Matrix(k,self.crystalsystem[0],self.crystalsystem[1])
                tmp['jonessymbol']=self.JonesSymbol(tmp['matrix'])
                self.symmetrygroup.append(tmp)
            # test reality of irrrep for the grey group
            if self.isGreyGroup:
                realitytest = self.RealityTest(self.characterlittlegroup)
        else:
            #correps
            #-------
            print '\n! Time reversal will be considered and co-representations calculated\n'
            self.PrintIrreps(irreps)
            charactertable,self.characterlittlegroup=self.CharacterTable(irreps)
            corepcase=self.RealityTest(self.characterlittlegroup)
            self.gkirreduciblerepresentations=self.ReduceCoReps(irreps,corepcase)
            self.PrintCoreps(self.gkirreduciblerepresentations)
            charactertable,self.characterlittlegroup=self.CharacterTable(self.gkirreduciblerepresentations)
            print 'character table:'
            print charactertable

            magneticlittlegroup=self.gkirreduciblerepresentations[self.gkirreduciblerepresentations.keys()[0]].keys()
            for k in magneticlittlegroup:
                tmp=dict()
                tmp['seitzsymbol']=k
                tmp['matrix']=self._Seitz2Matrix(k,self.crystalsystem[0],self.crystalsystem[1])
                self.symmetrygroup.append(tmp)
        self.allrepresentations=[]
        return

    def CalcMagneticRep(self,orbits):
        """
        return matrices of the magnetic representation (axial vector)
        """
        if len(orbits) > 1:
            print 'Atomic positions split into',len(orbits),'orbits'
            print '--> List of atomic orbits:', orbits.keys()
        print
        print 'Magnetic Representation'
        print '------------------------'
        #magnetic representation for each orbit
        gammamag=dict()
        for orbit, positions in orbits.iteritems():
            print 'atomic positions:'
            print ('\n'.join([' '.join(['{:10}'.format(item) for item in row])
                                          for row in positions]))
            self.charactermagneticoperations=dict()
            self.charactermagneticrepresentation=dict()
            self.magneticmatrices=dict()
            self.magneticrepresentation=dict()
            group=self.symmetrygroup
            kvector=self.kvector
            PermutationRep.__init__(self,self.latticesymbol[self.bravaislattice]['c2p'],positions,group,kvector)
            self.PermutationMatrix()
            self.PrintPermutationMatrix()
            self.CharacterPermutation()
            self.PrintCharacterPermutation()
            self.MagneticOperations()
            self.PrintMagneticMatrices()
            self.CharacterMagneticOperations()
            self.PrintCharacterMagneticOperations()
            if self.isFedorovGroup == True or self.isGreyGroup == True:
                self.MagneticRepresentation()
                self.PrintMagneticRepresentation(cc=False)
            else:
                self.MagneticCoRepresentation()
                self.PrintMagneticRepresentation(cc=True)
            self.CharacterMagneticRepresentation()
            self.PrintCharacterMagneticRepresentation()
            gammamag[orbit]=self.ReductionMagneticRepresentation()
            #save magnetic representation for each atomic orbit
            self.allrepresentations.append(self.magneticrepresentation)
        return gammamag #return the decomposition coefficients of the magnetic representation

    def CalcMechanicalRep(self,orbits):
        """
        return matrices of the mechanical representation (polar vector)
        """
        self.charactermechanicalrepresentation = dict()
        if len(orbits) > 1:
            print 'Atomic positions split into',len(orbits),'orbits'
            print '--> List of atomic orbits:', orbits.keys()
        print
        print 'Mechanical Representation'
        print '------------------------'
        #mechanical representation for each orbit
        gammamech=dict()
        for orbit, positions in orbits.iteritems():
            self.mechanicalmatrices=dict()
            self.mechanicalrepresentation=dict()
            self.charactermechanicalrepresentation=dict()
            print 'atomic positions:'
            print ('\n'.join([' '.join(['{:10}'.format(item) for item in row])
                                          for row in positions]))
            group=self.symmetrygroup
            kvector=self.kvector
            PermutationRep.__init__(self,self.latticesymbol[self.bravaislattice]['c2p'],positions,group,kvector)
            self.PermutationMatrix()
            self.MechanicalOperations()
            self.MechanicalRepresentation()
            self.CharacterMechanicalRepresentation()
            gammamech[orbit]=self.ReductionMechanicalRepresentation()
            #save mechanical representation for each atomic orbit
            self.allrepresentations.append(self.mechanicalrepresentation)
        return gammamech

    def CharacterMechanicalRepresentation(self):
        """
        character of mechanical representation
        """
        for k, val in self.mechanicalrepresentation.iteritems():
            character=np.around(np.trace(val))
            self.charactermechanicalrepresentation[k]=character
        return

    def CharacterMagneticRepresentation(self):
        """
        character of magnetic representation
        """
        for k, val in self.magneticrepresentation.iteritems():
            character=np.around(np.trace(val))
            self.charactermagneticrepresentation[k]=character
        return

    def CharacterMagneticOperations(self):
        for k, val in self.magneticmatrices.iteritems():
            character=np.around(np.trace(val))
            self.charactermagneticoperations[k]=character
        return

    def PrintCharacterMagneticOperations(self):
        print 'Character magnetic operations:'
        for k, val in self.charactermagneticoperations.iteritems():
            print k, val
        return

    def PrintCharacterMechanicalOperations(self):
        print 'Character mechanical operations:'
        for k, val in self.charactermechanicaloperations.iteritems():
            print k, val
        return

    def GetCosetsRepresentativesOfLG(self):
        """
        return the coset representatives of the little group
        """
        cosetlist=[]
        for op in self.cosetsrepoflg:
            mat=self._Seitz2Matrix(op,self.crystalsystem[0],self.crystalsystem[1])
            tmp=dict()
            tmp['seitzsymbol']=op
            tmp['matrix']=mat
            cosetlist.append(tmp)
        return cosetlist
        
    def MagneticOperations(self):
        """
        matrix of symmetry operation for axial vector
        (without time-reversal)
        """
        for indx, symop in enumerate(self.symmetrygroup):
            val=symop['matrix'][0:3,0:3]
            op=symop['seitzsymbol']
            mat=np.dot(np.linalg.det(val),val)
            self.magneticmatrices[op]=mat
        return

    def MechanicalOperations(self):
        """
        matrix of symmetry operation for axial vector
        (without time-reversal)
        """
        for indx, symop in enumerate(self.symmetrygroup):
            mat=symop['matrix'][0:3,0:3]
            op=symop['seitzsymbol']
            self.mechanicalmatrices[op]=mat
        return

    def MagneticRepresentation(self):
        """
        tensor product of A x B = G:
        [A]=mxn; [B]=pxq
        G_alfa,beta=m_i,j*I_k,l
        alfa=p*(i-1)+k; beta=q*(j-1)+l
        V=magnetic representation
        P=permutation representation
        """
        V=self.magneticmatrices
        P=self.permutationrepresentation
        if set(V.keys()) != set(P.keys()):
            sys.exit('sets of symmetry operations inconsistent')
        for op in V.keys():
            self.magneticrepresentation[op]=np.kron(P[op],V[op])
        return

    def MagneticCoRepresentation(self):
        """
        magnetic corepresentation has twice the dimension
        of the magnetic representation and takes into accout the
        antlinearity of the time reversal
        see e.g.: J. SChweizer / C.R. de Physique 6 (2005) 375-394
        """
        V=self.magneticmatrices
        P=self.permutationrepresentation
        if set(V.keys()) != set(P.keys()):
            sys.exit('sets of symmetry operations inconsistent')
        blockmatrices=dict()
        dim_v=np.shape(V['{1|(0.0,0.0,0.0)}'])
        dim_p=np.shape(P['{1|(0.0,0.0,0.0)}'])
        dim=[]
        dim.append(dim_v[0]*dim_p[0])
        dim.append(dim_v[1]*dim_p[1])
        for op in V.keys():
            magnmat=np.kron(P[op],V[op])
            mat=np.zeros((2*dim[0],2*dim[1]),dtype='complex')
            x=dim[0]
            y=dim[1]
            if 'T*' in op:
                mat[x:,0:y]=-magnmat
                mat[0:x,y:]=-magnmat
                self.magneticrepresentation[op]=mat
            else:
                mat[0:x,0:y]=magnmat
                mat[x:,y:]=magnmat
                self.magneticrepresentation[op]=mat
        return

    def MechanicalRepresentation(self):
        """
        tensor product of A x B = G:
        [A]=mxn; [B]=pxq
        G_alfa,beta=m_i,j*I_k,l
        alfa=p*(i-1)+k; beta=q*(j-1)+l
        V=magnetic representation
        P=permutation representation
        """
        V=self.mechanicalmatrices
        P=self.permutationrepresentation
        if set(V.keys()) != set(P.keys()):
            sys.exit('sets of symmetry operations inconsistent')
        for op in V.keys():
            self.mechanicalrepresentation[op]=np.kron(P[op],V[op])
        return

    def PrintCharacterMagneticRepresentation(self):
        """
        print characters of magnetic representation
        """
        print 'Character of magnetic representation'
        for k, val in self.charactermagneticrepresentation.iteritems():
            print k, ':', val
        return


    def PrintMagneticMatrices(self):
        """
        print matrix of magnetic matrices
        """
        print '\nAxial Representation:'
        print '---------------------'
        for k, matrix in self.magneticmatrices.iteritems():
            print k
            if 'T*' in k: matrix = -matrix #time-reversal
            print('\n'.join([' '.join(['{:14}'.format(item) for item in row])
                for row in np.around(matrix+0.0,2)]))  #add 0.0 to avoid -0.0 in printout...
        print
        return

    def PrintMagneticRepresentation(self,cc):
        """
        print matrix of magnetic representation
        cc = True if co-representations are used
        """
        print '\nMagnetic representation:'
        print '------------------------'


        #for k, val in self.magneticrepresentation.iteritems():
        #    print k
        #    print np.around(val,3)
        #raw_input()

        dim=np.shape(self.magneticrepresentation['{1|(0.0,0.0,0.0)}'])[0]
        Svar=[]
        if cc == True: dim=dim/2
        for k in range(1,dim,3):
            S='S_'+str(k/3+1)
            for i in range(0,3):
                if i == 0: S+='x'
                if i == 1: S+='y'
                if i == 2: S+='z'
                Svar.append(S)
                S=S[:-1]
        tmp=[]
        if cc == True:
            for S in Svar:
                tmp.append(S+'*')
            Svar.extend(tmp)
        table = PrettyTable()
        col_labels=self.magneticrepresentation.keys()
        table.add_column('G_mag',Svar)
        for sym_op in col_labels:
            indices = np.nonzero(self.magneticrepresentation[sym_op])
            (cols,rows)=np.shape(self.magneticrepresentation[sym_op])
            colval=[]
            for i in range(0,cols):
                tmp='';
                for j in range(0,rows):
                    factor = self.magneticrepresentation[sym_op][j,i]
                    if factor ==0:
                        pass
                    else:
                        if factor.imag == 0:
                            factor=factor.real
                        if factor == 1.:
                            factor = '+'
                        elif factor == -1.:
                            factor = '-'
                        else:
                            factor=str(factor+0.0)
                        tmp+= factor + Svar[j]
                colval.append(tmp)
            table.align[sym_op]='l'
            table.add_column(sym_op,colval)
        print table
        return

    def ReductionMagneticRepresentation(self):
        """
        reduction of representation:
        G=\sum_j n_j*G_j
        """
        k=self.characterlittlegroup
        n=len(k[k.keys()[0]].keys())
        list= k[k.keys()[0]].keys()
        if len([s for s in list if 'T*' in s]) != 0:
            corep=True
        else:
            corep=False
        n=float(n)
        nvpermutation=dict()
        nvaxialvector=dict()
        nvmagneticrepresentation=dict()
        for g, val in self.characterlittlegroup.iteritems():
            nvpermutation[g]=0.0
            nvaxialvector[g]=0.0
            nvmagneticrepresentation[g]=0.0
            for op, chi in val.iteritems():
                nvpermutation[g]+=self.characterpermutation[op]*chi.conjugate()
                nvaxialvector[g]+=np.trace(self.magneticmatrices[op])*chi.conjugate()
                nvmagneticrepresentation[g]+=self.charactermagneticrepresentation[op]*chi.conjugate()
            nvpermutation[g]/=n
            nvaxialvector[g]/=n
            nvmagneticrepresentation[g]/=n
        if corep == False:
            #does not make sense if coreps
            print '\nDecomposition of permutation representation'
            G='G='
            for g, val in sorted(nvpermutation.iteritems()):
                if val != 0:
                    G+=str(val)+'*'+g+'+'
            print G[:-1]
        print '\nDecomposition of magnetic representation'
        self.Gm='G='
        for g, val in sorted(nvmagneticrepresentation.iteritems()):
            if val != 0:
                self.Gm+=str(val)+'*'+g+'+'
        print self.Gm[:-1]

        return nvmagneticrepresentation

    def ReductionMechanicalRepresentation(self):
        """
        reduction of the mechanical representation in irreducible components:
        G=\sum_j n_j*G_j
        """
        k=self.characterlittlegroup
        n=len(k[k.keys()[0]].keys())
        list= k[k.keys()[0]].keys()

        n=float(n)
        nvmechanicalrepresentation=dict()
        for g, val in self.characterlittlegroup.iteritems():
            nvmechanicalrepresentation[g]=0.0
            for op, chi in val.iteritems():
                nvmechanicalrepresentation[g]+=self.charactermechanicalrepresentation[op]*chi.conjugate()
            nvmechanicalrepresentation[g]/=n
        print '\nDecomposition of the mechanical representation'
        self.Gm='G='
        for g, val in sorted(nvmechanicalrepresentation.iteritems()):
            if val != 0:
                self.Gm+=str(val)+'*'+g+'+'
        print self.Gm[:-1]

        return nvmechanicalrepresentation

if __name__=="__main__":

    # tested structures
    #------------------

    #MnSi
    #spacegroup='P 21 3'
    #kvector=[0.05,0.05,0.05]
    #atomicposition=[0.137,0.137,0.137]

    #mnP
    spacegroup='P 2/b 1 1'
    kvector=[0.14,0.,0.]
    atomicposition=[0.15,0.15,0.25]

    #AgFe3(S04)2(OH)6
    #jarosite
    #spacegroup='R -3 m:H'
    #kvector=[0.,0.,1.5]
    #atomicposition=[0.5,0.,0.5] #R

    #HoAuGe
    #spacegroup = 'P 63 m c'
    #kvector = [0.5,0,0]
    #atomicposition = [0,0,0.05]

    #PrPtAl,
    #spacegroup='P n m a'
    #kvector=[0.,0.,0.45]
    #atomicposition = [0.,0.25,0.3]

    #HoBa2NiO5, Ni site at (0,0,0)
    #spacegroup = 'I m m m'
    #kvector = [0.5,0.,0.5]
    #atomicposition=[0.5,0,0.20]

    #U2Ni2In, k= (0,0,1/2)
    #spacegroup = 'P 4/m b m'
    #kvector= [0.,0,0.5]
    #atomicposition = [0.1,0.6,0.5] #R

    #CeAl2
    #spacegroup = 'F d -3 m:2'
    #kvector = np.array([0.45,0.55,0.5])
    #atomicposition = [0.125,0.125,0.125]

    #CuB2O4 A-site
    #spacegroup = 'I -4 2 d'
    #kvector = np.array([0.,0.,0.])
    #atomicposition = [0,0,0.5]
    #CuB2O4 B-site
    #spacegroup = 'I -4 2 d'
    #kvector = np.array([0.,0.,0.])
    #atomicposition = [0.08,0.250,0.125]

    #FeNdBO, k = [0,0,1.5], Nd position at (0,0,0)
    #spacegroup = 'R 32:H'
    #kvector = np.array([0.,0.,1.5])
    #atomicposition = [0.,0.,0.]
    #FeNdBO, k = [0,0,1.5], Nd position at (0,0,0)
    #spacegroup = 'R 32:H'
    #kvector = np.array([0.,0.,1.5])
    #atomicposition = [0.55,0.,0.]

    #Ce2Pd2Sn
    #spacegroup = 'P 4/m b m'
    #kvector = np.array([0.15,0,0])
    #atomicposition = [0.1,0.6,0.5]

    #R3c:R, k = [0,0,0], ion at (0,0,0)
    #spacegroup = 'R 3 c:R'
    #kvector = np.array([0,0,0])
    #atomicposition = [0,0,0]

    #R3m:R, k = [0,0,0], ion at (0,0,0)
    #spacegroup = 'R 3 m:R'
    #kvector = np.array([0,0,0])
    #atomicposition = [0,0,0]

    #R3m:R, k = [0.5,0.5,0.5], ion at (0,0,0)
    #spacegroup = 'R 3 m:R'
    #kvector = np.array([0.5,0.5,0.5])
    #atomicposition = [0,0,0]

    #Cuo
    #spacegroup='C 1 2/c 1'
    #kvector=np.array([0.5,0.,-0.5])
    #atomicposition=[0.,0.,0.5]

    #Mn2GeO4, first site at (0,0,0)
    #spacegroup='P n m a'
    #kvector=np.array([0.15,0.15,0.])
    #atomicposition=[0.,0.,0.]

    #Mn2GeO4, first site at (0.28,0.25,0.99)
    #spacegroup='P n m a'
    #kvector=np.array([0.15,0.15,0.])
    #atomicposition=[0.28,0.25,0.99]

    #LiHoF4, k = [0,0,0], ion at (0,0.25,0.125)
    #spacegroup='I 41/a:2'
    #kvector=np.array([0,0,0])
    #atomicposition=[0,0.25,0.125]

    #-P4, k = (0,0,kz) (see J. Schweizer's talk at PSI)
    #spacegroup = '-P 4'
    #kvector = [0,0,0.15]
    #atomicposition=[0,0.,0.]

    #RMn2O5
    #spacegroup='P b a m'
    #kvector=[0.5,0.,0.25]
    #atomicposition=[0.1,0.85,0.5] #Mn
    #atomicposition=[0.,0.5,0.25] #Mn4+
    #atomicposition=[0.137,0.171,0.0] #Er3+

    #LaMnO3
    #spacegroup='P b n m'
    #kvector=np.array([0.5,0.,0.])
    #kvector=np.array([0.,0.,0.])
    #atomicposition=[0.5,0.,0.]

    #TEST PHONONS
    #BaTiO3
    #spacegroup='P m -3 m'
    #kvector=np.array([0.,0.,0.])
    #atomicposition=[0.5,0.5,0.]

    axialvector = True
    
    GV=VectorRep(spacegroup,kvector,np.array(atomicposition),axialvector)
    orbits=GV.SplitOrbits()

    #copy-paste orbits into test_MagneticRepresentation.py
    #loop over orbits
    for o,val in orbits.iteritems():
        orbit=dict()
        orbit[o]=val    
        if axialvector == True:
            GV.CalcMechanicalRep(orbit)
        else:
            GV.CalcMagneticRep(orbit)
            print GV.charactermagneticrepresentation
    