#!/usr/bin/env python
"""
Test small irreducible representations of little group
"""
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "beta"

from Kvector import Kvector
from SpaceGroupIrreps import SpaceGroupIrreps

import unittest

import numpy as np

class KnownSmallIrreps(unittest.TestCase):
    '''
    Test of allowed irreducible representations of the little group.
    The test compares the calculated with the known character tables
    !: the calculations are for the irreducible representations and NOT the coreps.
    '''

    #MnSi, P213, k=(0.05,0.05,0.05)
    MnSi_CharTable=dict()
    MnSi_CharTable['G0']=dict()
    MnSi_CharTable['G1']=dict()
    MnSi_CharTable['G2']=dict()

    MnSi_CharTable['G0']['{3^+_xxx|(0.0,0.0,0.0)}']=1+0j
    MnSi_CharTable['G0']['{3^-_xxx|(0.0,0.0,0.0)}']=1+0j
    MnSi_CharTable['G0']['{1|(0.0,0.0,0.0)}']=1+0j

    MnSi_CharTable['G1']['{3^+_xxx|(0.0,0.0,0.0)}']=-0.5+0.86599999999999999j
    MnSi_CharTable['G1']['{3^-_xxx|(0.0,0.0,0.0)}']=-0.5-0.86599999999999999j
    MnSi_CharTable['G1']['{1|(0.0,0.0,0.0)}']=1+0j

    MnSi_CharTable['G2']['{3^+_xxx|(0.0,0.0,0.0)}']=-0.5-0.86599999999999999j
    MnSi_CharTable['G2']['{3^-_xxx|(0.0,0.0,0.0)}']=-0.5+0.86599999999999999j
    MnSi_CharTable['G2']['{1|(0.0,0.0,0.0)}']=1+0j

    #CeAl2,
    CeAl2_CharTable=dict()
    CeAl2_CharTable['G0']=dict()
    CeAl2_CharTable['G1']=dict()

    CeAl2_CharTable['G0']['{1|(0.0,0.0,0.0)}']= 1+0j
    CeAl2_CharTable['G0']['{2_x-x0|(0.0,0.0,0.0)}']= 1+0j

    CeAl2_CharTable['G1']['{1|(0.0,0.0,0.0)}']= 1+0j
    CeAl2_CharTable['G1']['{2_x-x0|(0.0,0.0,0.0)}']= -1+0j

    #MnP
    MnP_CharTable=dict()
    MnP_CharTable['G0']=dict()
    MnP_CharTable['G1']=dict()

    MnP_CharTable['G0']['{1|(0.0,0.0,0.0)}']=1+0j
    MnP_CharTable['G0']['{2_x00|(0.5,0.5,0.0)}']= 0.90500000000000003-0.42599999999999999j

    MnP_CharTable['G1']['{1|(0.0,0.0,0.0)}']=1+0j
    MnP_CharTable['G1']['{2_x00|(0.5,0.5,0.0)}']= -0.90500000000000003+0.42599999999999999j

    #CuO
    CuO_CharTable=dict()
    CuO_CharTable['G0']=dict()
    CuO_CharTable['G1']=dict()

    CuO_CharTable['G0']['{1|(0.0,0.0,0.0)}']=1+0j
    CuO_CharTable['G0']['{m_x0z|(0.0,0.0,0.5)}']= 1j

    CuO_CharTable['G1']['{1|(0.0,0.0,0.0)}']=1+0j
    CuO_CharTable['G1']['{m_x0z|(0.0,0.0,0.5)}']= -1j

    #CuB2O4, k = 0
    CuB2O4_CharTable=dict()
    CuB2O4_CharTable['G0']=dict()
    CuB2O4_CharTable['G1']=dict()
    CuB2O4_CharTable['G2']=dict()
    CuB2O4_CharTable['G3']=dict()
    CuB2O4_CharTable['G4']=dict()

    CuB2O4_CharTable['G0']['{-4^+_00z|(0.0,0.0,0.0)}'] = 1+0j
    CuB2O4_CharTable['G0']['{2_x00|(0.0,0.5,0.25)}'] = -1+0j
    CuB2O4_CharTable['G0']['{m_x-xz|(0.5,0.0,0.75)}'] = -1+0j
    CuB2O4_CharTable['G0']['{m_xxz|(0.5,0.0,0.75)}'] =  -1+0j
    CuB2O4_CharTable['G0']['{1|(0.0,0.0,0.0)}'] =  1+0j
    CuB2O4_CharTable['G0']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    CuB2O4_CharTable['G0']['{2_0y0|(0.0,0.5,0.25)}'] = -1+0j
    CuB2O4_CharTable['G0']['{-4^-_00z|(0.0,0.0,0.0)}'] = 1+0j

    CuB2O4_CharTable['G1']['{-4^+_00z|(0.0,0.0,0.0)}'] = -1+0j
    CuB2O4_CharTable['G1']['{2_x00|(0.0,0.5,0.25)}'] = 1+0j
    CuB2O4_CharTable['G1']['{m_x-xz|(0.5,0.0,0.75)}'] = -1+0j
    CuB2O4_CharTable['G1']['{m_xxz|(0.5,0.0,0.75)}'] =  -1+0j
    CuB2O4_CharTable['G1']['{1|(0.0,0.0,0.0)}'] = 1+0j
    CuB2O4_CharTable['G1']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    CuB2O4_CharTable['G1']['{2_0y0|(0.0,0.5,0.25)}'] = 1+0j
    CuB2O4_CharTable['G1']['{-4^-_00z|(0.0,0.0,0.0)}'] = -1+0j

    CuB2O4_CharTable['G2']['{-4^+_00z|(0.0,0.0,0.0)}'] = 1+0j
    CuB2O4_CharTable['G2']['{2_x00|(0.0,0.5,0.25)}'] =  1+0j
    CuB2O4_CharTable['G2']['{m_x-xz|(0.5,0.0,0.75)}'] = 1+0j
    CuB2O4_CharTable['G2']['{m_xxz|(0.5,0.0,0.75)}'] = 1+0j
    CuB2O4_CharTable['G2']['{1|(0.0,0.0,0.0)}'] = 1+0j
    CuB2O4_CharTable['G2']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    CuB2O4_CharTable['G2']['{2_0y0|(0.0,0.5,0.25)}'] = 1+0j
    CuB2O4_CharTable['G2']['{-4^-_00z|(0.0,0.0,0.0)}'] = 1+0j

    CuB2O4_CharTable['G3']['{-4^+_00z|(0.0,0.0,0.0)}'] = -1+0j
    CuB2O4_CharTable['G3']['{2_x00|(0.0,0.5,0.25)}'] = -1+0j
    CuB2O4_CharTable['G3']['{m_x-xz|(0.5,0.0,0.75)}'] = 1+0j
    CuB2O4_CharTable['G3']['{m_xxz|(0.5,0.0,0.75)}'] = 1+0j
    CuB2O4_CharTable['G3']['{1|(0.0,0.0,0.0)}'] = 1+0j
    CuB2O4_CharTable['G3']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    CuB2O4_CharTable['G3']['{2_0y0|(0.0,0.5,0.25)}'] = -1+0j
    CuB2O4_CharTable['G3']['{-4^-_00z|(0.0,0.0,0.0)}'] = -1+0j

    CuB2O4_CharTable['G4']['{-4^+_00z|(0.0,0.0,0.0)}'] =0j
    CuB2O4_CharTable['G4']['{2_x00|(0.0,0.5,0.25)}'] = 0j
    CuB2O4_CharTable['G4']['{m_x-xz|(0.5,0.0,0.75)}'] = 0j
    CuB2O4_CharTable['G4']['{m_xxz|(0.5,0.0,0.75)}'] = 0j
    CuB2O4_CharTable['G4']['{1|(0.0,0.0,0.0)}'] = 2+0j
    CuB2O4_CharTable['G4']['{2_00z|(0.0,0.0,0.0)}'] = -2+0j
    CuB2O4_CharTable['G4']['{2_0y0|(0.0,0.5,0.25)}'] = 0j
    CuB2O4_CharTable['G4']['{-4^-_00z|(0.0,0.0,0.0)}'] = 0j


    #Ho2BaNiO5, k = [0.5,0,0.5]
    Ho2BaNiO5_CharTable=dict()
    Ho2BaNiO5_CharTable['G0']=dict()
    Ho2BaNiO5_CharTable['G1']=dict()
    Ho2BaNiO5_CharTable['G2']=dict()
    Ho2BaNiO5_CharTable['G3']=dict()

    Ho2BaNiO5_CharTable['G0']['{m_x0z|(0.0,0.0,0.0)}'] = 1+0j
    Ho2BaNiO5_CharTable['G0']['{2_0y0|(0.0,0.0,0.0)}'] = 1+0j
    Ho2BaNiO5_CharTable['G0']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Ho2BaNiO5_CharTable['G0']['{-1|(0.0,0.0,0.0)}'] = 1+0j

    Ho2BaNiO5_CharTable['G1']['{m_x0z|(0.0,0.0,0.0)}'] = -1+0j
    Ho2BaNiO5_CharTable['G1']['{2_0y0|(0.0,0.0,0.0)}'] = -1+0j
    Ho2BaNiO5_CharTable['G1']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Ho2BaNiO5_CharTable['G1']['{-1|(0.0,0.0,0.0)}'] = 1+0j

    Ho2BaNiO5_CharTable['G2']['{m_x0z|(0.0,0.0,0.0)}'] = -1+0j
    Ho2BaNiO5_CharTable['G2']['{2_0y0|(0.0,0.0,0.0)}'] = 1+0j
    Ho2BaNiO5_CharTable['G2']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Ho2BaNiO5_CharTable['G2']['{-1|(0.0,0.0,0.0)}'] = -1+0j

    Ho2BaNiO5_CharTable['G3']['{m_x0z|(0.0,0.0,0.0)}'] = 1+0j
    Ho2BaNiO5_CharTable['G3']['{2_0y0|(0.0,0.0,0.0)}'] =  -1+0j
    Ho2BaNiO5_CharTable['G3']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Ho2BaNiO5_CharTable['G3']['{-1|(0.0,0.0,0.0)}'] = -1+0j

    #Ce2Pd2Sn, k = [0.1,0,0.]
    Ce2Pd2Sn_CharTable=dict()
    Ce2Pd2Sn_CharTable['G0']=dict()
    Ce2Pd2Sn_CharTable['G1']=dict()
    Ce2Pd2Sn_CharTable['G2']=dict()
    Ce2Pd2Sn_CharTable['G3']=dict()

    Ce2Pd2Sn_CharTable['G0']['{m_xy0|(0.0,0.0,0.0)}'] = 1+0j
    Ce2Pd2Sn_CharTable['G0']['{m_x0z|(0.5,0.5,0.0)}'] = 0.95099999999999996-0.309j
    Ce2Pd2Sn_CharTable['G0']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Ce2Pd2Sn_CharTable['G0']['{2_x00|(0.5,0.5,0.0)}'] = 0.95099999999999996-0.309j

    Ce2Pd2Sn_CharTable['G1']['{m_xy0|(0.0,0.0,0.0)}'] = -1+0j
    Ce2Pd2Sn_CharTable['G1']['{m_x0z|(0.5,0.5,0.0)}'] = -0.95099999999999996+0.309j
    Ce2Pd2Sn_CharTable['G1']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Ce2Pd2Sn_CharTable['G1']['{2_x00|(0.5,0.5,0.0)}'] = 0.95099999999999996-0.309j

    Ce2Pd2Sn_CharTable['G2']['{m_xy0|(0.0,0.0,0.0)}'] = -1+0j
    Ce2Pd2Sn_CharTable['G2']['{m_x0z|(0.5,0.5,0.0)}'] = 0.95099999999999996-0.309j
    Ce2Pd2Sn_CharTable['G2']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Ce2Pd2Sn_CharTable['G2']['{2_x00|(0.5,0.5,0.0)}'] = -0.95099999999999996+0.309j

    Ce2Pd2Sn_CharTable['G3']['{m_xy0|(0.0,0.0,0.0)}'] = 1+0j
    Ce2Pd2Sn_CharTable['G3']['{m_x0z|(0.5,0.5,0.0)}'] = -0.95099999999999996+0.309j
    Ce2Pd2Sn_CharTable['G3']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Ce2Pd2Sn_CharTable['G3']['{2_x00|(0.5,0.5,0.0)}'] = -0.95099999999999996+0.309j

    #KTb3F12, k = [1,0,0]
    KTb3F12_CharTable=dict()
    KTb3F12_CharTable['G0']=dict()
    KTb3F12_CharTable['G1']=dict()
    KTb3F12_CharTable['G2']=dict()
    KTb3F12_CharTable['G3']=dict()
    KTb3F12_CharTable['G4']=dict()
    KTb3F12_CharTable['G5']=dict()
    KTb3F12_CharTable['G6']=dict()
    KTb3F12_CharTable['G7']=dict()

    KTb3F12_CharTable['G0']['{-4^+_00z|(0.0,0.0,0.0)}'] = 1j
    KTb3F12_CharTable['G0']['{-4^-_00z|(0.0,0.0,0.0)}'] = -1j
    KTb3F12_CharTable['G0']['{1|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G0']['{4^-_00z|(0.0,0.0,0.0)}'] = -1j
    KTb3F12_CharTable['G0']['{2_00z|(0.0,0.0,0.0)}'] = -1+0j
    KTb3F12_CharTable['G0']['{4^+_00z|(0.0,0.0,0.0)}'] = 1j
    KTb3F12_CharTable['G0']['{m_xy0|(0.0,0.0,0.0)}'] = -1+0j
    KTb3F12_CharTable['G0']['{-1|(0.0,0.0,0.0)}'] = 1+0j

    KTb3F12_CharTable['G1']['{-4^+_00z|(0.0,0.0,0.0)}'] = -1j
    KTb3F12_CharTable['G1']['{-4^-_00z|(0.0,0.0,0.0)}'] = 1j
    KTb3F12_CharTable['G1']['{1|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G1']['{4^-_00z|(0.0,0.0,0.0)}'] = 1j
    KTb3F12_CharTable['G1']['{2_00z|(0.0,0.0,0.0)}'] = -1+0j
    KTb3F12_CharTable['G1']['{4^+_00z|(0.0,0.0,0.0)}'] = -1j
    KTb3F12_CharTable['G1']['{m_xy0|(0.0,0.0,0.0)}']= -1+0j
    KTb3F12_CharTable['G1']['{-1|(0.0,0.0,0.0)}']= 1+0j

    KTb3F12_CharTable['G2']['{-4^+_00z|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G2']['{-4^-_00z|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G2']['{1|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G2']['{4^-_00z|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G2']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G2']['{4^+_00z|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G2']['{m_xy0|(0.0,0.0,0.0)}']= 1+0j
    KTb3F12_CharTable['G2']['{-1|(0.0,0.0,0.0)}']= 1+0j

    KTb3F12_CharTable['G3']['{-4^+_00z|(0.0,0.0,0.0)}'] = -1+0j
    KTb3F12_CharTable['G3']['{-4^-_00z|(0.0,0.0,0.0)}'] = -1+0j
    KTb3F12_CharTable['G3']['{1|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G3']['{4^-_00z|(0.0,0.0,0.0)}'] = -1+0j
    KTb3F12_CharTable['G3']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G3']['{4^+_00z|(0.0,0.0,0.0)}'] = -1+0j
    KTb3F12_CharTable['G3']['{m_xy0|(0.0,0.0,0.0)}']= 1+0j
    KTb3F12_CharTable['G3']['{-1|(0.0,0.0,0.0)}']= 1+0j

    KTb3F12_CharTable['G4']['{-4^+_00z|(0.0,0.0,0.0)}'] =  1j
    KTb3F12_CharTable['G4']['{-4^-_00z|(0.0,0.0,0.0)}'] = -1j
    KTb3F12_CharTable['G4']['{1|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G4']['{4^-_00z|(0.0,0.0,0.0)}'] =  1j
    KTb3F12_CharTable['G4']['{2_00z|(0.0,0.0,0.0)}'] = -1+0j
    KTb3F12_CharTable['G4']['{4^+_00z|(0.0,0.0,0.0)}'] = -1j
    KTb3F12_CharTable['G4']['{m_xy0|(0.0,0.0,0.0)}']= 1+0j
    KTb3F12_CharTable['G4']['{-1|(0.0,0.0,0.0)}']= -1+0j

    KTb3F12_CharTable['G5']['{-4^+_00z|(0.0,0.0,0.0)}'] =  -1j
    KTb3F12_CharTable['G5']['{-4^-_00z|(0.0,0.0,0.0)}'] =  1j
    KTb3F12_CharTable['G5']['{1|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G5']['{4^-_00z|(0.0,0.0,0.0)}'] =  -1j
    KTb3F12_CharTable['G5']['{2_00z|(0.0,0.0,0.0)}'] = -1+0j
    KTb3F12_CharTable['G5']['{4^+_00z|(0.0,0.0,0.0)}'] = 1j
    KTb3F12_CharTable['G5']['{m_xy0|(0.0,0.0,0.0)}']= 1+0j
    KTb3F12_CharTable['G5']['{-1|(0.0,0.0,0.0)}']= -1+0j

    KTb3F12_CharTable['G6']['{-4^+_00z|(0.0,0.0,0.0)}'] =  1+0j
    KTb3F12_CharTable['G6']['{-4^-_00z|(0.0,0.0,0.0)}'] =  1+0j
    KTb3F12_CharTable['G6']['{1|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G6']['{4^-_00z|(0.0,0.0,0.0)}'] =  -1+0j
    KTb3F12_CharTable['G6']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G6']['{4^+_00z|(0.0,0.0,0.0)}'] = -1+0j
    KTb3F12_CharTable['G6']['{m_xy0|(0.0,0.0,0.0)}']= -1+0j
    KTb3F12_CharTable['G6']['{-1|(0.0,0.0,0.0)}']= -1+0j

    KTb3F12_CharTable['G7']['{-4^+_00z|(0.0,0.0,0.0)}'] =  -1+0j
    KTb3F12_CharTable['G7']['{-4^-_00z|(0.0,0.0,0.0)}'] =  -1+0j
    KTb3F12_CharTable['G7']['{1|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G7']['{4^-_00z|(0.0,0.0,0.0)}'] =  1+0j
    KTb3F12_CharTable['G7']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G7']['{4^+_00z|(0.0,0.0,0.0)}'] = 1+0j
    KTb3F12_CharTable['G7']['{m_xy0|(0.0,0.0,0.0)}']= -1+0j
    KTb3F12_CharTable['G7']['{-1|(0.0,0.0,0.0)}']= -1+0j

    #PrPtAl, Pnma, k = [0,0,kz], kz = 0.25
    PrPtAl_CharTable=dict()
    PrPtAl_CharTable['G0']=dict()
    PrPtAl_CharTable['G1']=dict()
    PrPtAl_CharTable['G2']=dict()
    PrPtAl_CharTable['G3']=dict()

    PrPtAl_CharTable['G0']['{m_0yz|(0.5,0.5,0.5)}'] = 0.70699999999999996-0.70699999999999996j
    PrPtAl_CharTable['G0']['{2_00z|(0.5,0.0,0.5)}'] = 0.70699999999999996-0.70699999999999996j
    PrPtAl_CharTable['G0']['{1|(0.0,0.0,0.0)}'] = 1+0j
    PrPtAl_CharTable['G0']['{m_x0z|(0.0,0.5,0.0)}'] = 1+0j

    PrPtAl_CharTable['G3']['{m_0yz|(0.5,0.5,0.5)}'] = -0.70699999999999996+0.70699999999999996j
    PrPtAl_CharTable['G3']['{2_00z|(0.5,0.0,0.5)}'] = -0.70699999999999996+0.70699999999999996j
    PrPtAl_CharTable['G3']['{1|(0.0,0.0,0.0)}'] = 1+0j
    PrPtAl_CharTable['G3']['{m_x0z|(0.0,0.5,0.0)}'] = 1+0j

    PrPtAl_CharTable['G2']['{m_0yz|(0.5,0.5,0.5)}'] = 0.70699999999999996-0.70699999999999996j
    PrPtAl_CharTable['G2']['{2_00z|(0.5,0.0,0.5)}'] = -0.70699999999999996+0.70699999999999996j
    PrPtAl_CharTable['G2']['{1|(0.0,0.0,0.0)}'] = 1+0j
    PrPtAl_CharTable['G2']['{m_x0z|(0.0,0.5,0.0)}'] = -1+0j

    PrPtAl_CharTable['G1']['{m_0yz|(0.5,0.5,0.5)}'] = -0.70699999999999996+0.70699999999999996j
    PrPtAl_CharTable['G1']['{2_00z|(0.5,0.0,0.5)}'] = 0.70699999999999996-0.70699999999999996j
    PrPtAl_CharTable['G1']['{1|(0.0,0.0,0.0)}'] = 1+0j
    PrPtAl_CharTable['G1']['{m_x0z|(0.0,0.5,0.0)}'] = -1+0j


    #Jarosite, k = [0,0,3/2]
    Jarosite_CharTable=dict()
    Jarosite_CharTable['G0']=dict()
    Jarosite_CharTable['G1']=dict()
    Jarosite_CharTable['G2']=dict()
    Jarosite_CharTable['G3']=dict()
    Jarosite_CharTable['G4']=dict()
    Jarosite_CharTable['G5']=dict()

    Jarosite_CharTable['G0']['{3^-_00z|(0.0,0.0,0.0)}'] =  1+0j
    Jarosite_CharTable['G0']['{-3^+_00z|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G0']['{-3^-_00z|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G0']['{m_x-xz|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G0']['{m_2xxz|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G0']['{2_xx0|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G0']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G0']['{2_0y0|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G0']['{3^+_00z|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G0']['{m_x2xz|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G0']['{2_x00|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G0']['{-1|(0.0,0.0,0.0)}'] = 1+0j

    Jarosite_CharTable['G1']['{3^-_00z|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G1']['{-3^+_00z|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G1']['{-3^-_00z|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G1']['{m_x-xz|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G1']['{m_2xxz|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G1']['{2_xx0|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G1']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G1']['{2_0y0|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G1']['{3^+_00z|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G1']['{m_x2xz|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G1']['{2_x00|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G1']['{-1|(0.0,0.0,0.0)}'] = -1+0j

    Jarosite_CharTable['G2']['{3^-_00z|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G2']['{-3^+_00z|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G2']['{-3^-_00z|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G2']['{m_x-xz|(0.0,0.0,0.0)}'] = 0j
    Jarosite_CharTable['G2']['{m_2xxz|(0.0,0.0,0.0)}'] = 0j
    Jarosite_CharTable['G2']['{2_xx0|(0.0,0.0,0.0)}'] = 0j
    Jarosite_CharTable['G2']['{1|(0.0,0.0,0.0)}'] = 2+0j
    Jarosite_CharTable['G2']['{2_0y0|(0.0,0.0,0.0)}'] = 0j
    Jarosite_CharTable['G2']['{3^+_00z|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G2']['{m_x2xz|(0.0,0.0,0.0)}'] = 0j
    Jarosite_CharTable['G2']['{2_x00|(0.0,0.0,0.0)}'] = 0j
    Jarosite_CharTable['G2']['{-1|(0.0,0.0,0.0)}'] = 2+0j

    Jarosite_CharTable['G3']['{3^-_00z|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G3']['{-3^+_00z|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G3']['{-3^-_00z|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G3']['{m_x-xz|(0.0,0.0,0.0)}'] = 0j
    Jarosite_CharTable['G3']['{m_2xxz|(0.0,0.0,0.0)}'] = 0j
    Jarosite_CharTable['G3']['{2_xx0|(0.0,0.0,0.0)}'] = 0j
    Jarosite_CharTable['G3']['{1|(0.0,0.0,0.0)}'] = 2+0j
    Jarosite_CharTable['G3']['{2_0y0|(0.0,0.0,0.0)}'] = 0j
    Jarosite_CharTable['G3']['{3^+_00z|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G3']['{m_x2xz|(0.0,0.0,0.0)}'] = 0j
    Jarosite_CharTable['G3']['{2_x00|(0.0,0.0,0.0)}'] = 0j
    Jarosite_CharTable['G3']['{-1|(0.0,0.0,0.0)}'] = -2+0j

    Jarosite_CharTable['G4']['{3^-_00z|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G4']['{-3^+_00z|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G4']['{-3^-_00z|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G4']['{m_x-xz|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G4']['{m_2xxz|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G4']['{2_xx0|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G4']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G4']['{2_0y0|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G4']['{3^+_00z|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G4']['{m_x2xz|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G4']['{2_x00|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G4']['{-1|(0.0,0.0,0.0)}'] = 1+0j

    Jarosite_CharTable['G5']['{3^-_00z|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G5']['{-3^+_00z|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G5']['{-3^-_00z|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G5']['{m_x-xz|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G5']['{m_2xxz|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G5']['{2_xx0|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G5']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G5']['{2_0y0|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G5']['{3^+_00z|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G5']['{m_x2xz|(0.0,0.0,0.0)}'] = 1+0j
    Jarosite_CharTable['G5']['{2_x00|(0.0,0.0,0.0)}'] = -1+0j
    Jarosite_CharTable['G5']['{-1|(0.0,0.0,0.0)}'] =  -1+0j


    #HoAuGe, P 63 m c, k = [0.5,0,0]
    HoAuGe_CharTable=dict()
    HoAuGe_CharTable['G0']=dict()
    HoAuGe_CharTable['G1']=dict()
    HoAuGe_CharTable['G2']=dict()
    HoAuGe_CharTable['G3']=dict()

    HoAuGe_CharTable['G3']['{2_00z|(0.0,0.0,0.5)}'] = -1-0j
    HoAuGe_CharTable['G3']['{m_2xxz|(0.0,0.0,0.0)}'] = 1+0j
    HoAuGe_CharTable['G3']['{m_0yz|(0.0,0.0,0.5)}'] = -1-0j
    HoAuGe_CharTable['G3']['{1|(0.0,0.0,0.0)}'] = 1

    HoAuGe_CharTable['G2']['{2_00z|(0.0,0.0,0.5)}'] = -1-0j
    HoAuGe_CharTable['G2']['{m_2xxz|(0.0,0.0,0.0)}'] = -1-0j
    HoAuGe_CharTable['G2']['{m_0yz|(0.0,0.0,0.5)}'] = 1+0j
    HoAuGe_CharTable['G2']['{1|(0.0,0.0,0.0)}'] = 1

    HoAuGe_CharTable['G1']['{2_00z|(0.0,0.0,0.5)}'] = 1+0j
    HoAuGe_CharTable['G1']['{m_2xxz|(0.0,0.0,0.0)}'] = -1-0j
    HoAuGe_CharTable['G1']['{m_0yz|(0.0,0.0,0.5)}'] = -1-0j
    HoAuGe_CharTable['G1']['{1|(0.0,0.0,0.0)}'] = 1

    HoAuGe_CharTable['G0']['{2_00z|(0.0,0.0,0.5)}'] = 1+0j
    HoAuGe_CharTable['G0']['{m_2xxz|(0.0,0.0,0.0)}'] = 1+0j
    HoAuGe_CharTable['G0']['{m_0yz|(0.0,0.0,0.5)}'] = 1+0j
    HoAuGe_CharTable['G0']['{1|(0.0,0.0,0.0)}'] = 1


    #NdFe3(BO3)4, R 32:H, k = [0,0,1.5], commensurate phase
    NdFeBO_CharTable=dict()
    NdFeBO_CharTable['G0']=dict()
    NdFeBO_CharTable['G1']=dict()
    NdFeBO_CharTable['G2']=dict()

    NdFeBO_CharTable['G2']['{3^-_00z|(0.0,0.0,0.0)}'] = 1+0j
    NdFeBO_CharTable['G2']['{2_x00|(0.0,0.0,0.0)}'] = -1+0j
    NdFeBO_CharTable['G2']['{2_xx0|(0.0,0.0,0.0)}'] = -1+0j
    NdFeBO_CharTable['G2']['{1|(0.0,0.0,0.0)}'] = 1+0j
    NdFeBO_CharTable['G2']['{2_0y0|(0.0,0.0,0.0)}'] = -1+0j
    NdFeBO_CharTable['G2']['{3^+_00z|(0.0,0.0,0.0)}'] = 1+0j

    NdFeBO_CharTable['G1']['{3^-_00z|(0.0,0.0,0.0)}'] = 1+0j
    NdFeBO_CharTable['G1']['{2_x00|(0.0,0.0,0.0)}'] = 1+0j
    NdFeBO_CharTable['G1']['{2_xx0|(0.0,0.0,0.0)}'] = 1+0j
    NdFeBO_CharTable['G1']['{1|(0.0,0.0,0.0)}'] = 1+0j
    NdFeBO_CharTable['G1']['{2_0y0|(0.0,0.0,0.0)}'] = 1+0j
    NdFeBO_CharTable['G1']['{3^+_00z|(0.0,0.0,0.0)}'] = 1+0j

    NdFeBO_CharTable['G0']['{3^-_00z|(0.0,0.0,0.0)}'] = -1+0j
    NdFeBO_CharTable['G0']['{2_x00|(0.0,0.0,0.0)}'] = 0j
    NdFeBO_CharTable['G0']['{2_xx0|(0.0,0.0,0.0)}'] = 0j
    NdFeBO_CharTable['G0']['{1|(0.0,0.0,0.0)}'] = 2+0j
    NdFeBO_CharTable['G0']['{2_0y0|(0.0,0.0,0.0)}'] = 0j
    NdFeBO_CharTable['G0']['{3^+_00z|(0.0,0.0,0.0)}'] = -1+0j


    #R 3c:R, k = [0,0,0], Journal of Magnetism and Magnetic Materials 24 (1981) 267-278
    R3cR_CharTable=dict()
    R3cR_CharTable['G0']=dict()
    R3cR_CharTable['G1']=dict()
    R3cR_CharTable['G2']=dict()

    R3cR_CharTable['G2']['{3^-_xxx|(0.0,0.0,0.0)}'] = 1+0j
    R3cR_CharTable['G2']['{m_xyx|(0.5,0.5,0.5)}'] =  -1+0j
    R3cR_CharTable['G2']['{1|(0.0,0.0,0.0)}'] =  1+0j
    R3cR_CharTable['G2']['{3^+_xxx|(0.0,0.0,0.0)}'] = 1+0j
    R3cR_CharTable['G2']['{m_xyy|(0.5,0.5,0.5)}'] = -1+0j
    R3cR_CharTable['G2']['{m_xxz|(0.5,0.5,0.5)}'] = -1+0j

    R3cR_CharTable['G1']['{3^-_xxx|(0.0,0.0,0.0)}'] = 1+0j
    R3cR_CharTable['G1']['{m_xyx|(0.5,0.5,0.5)}'] = 1+0j
    R3cR_CharTable['G1']['{1|(0.0,0.0,0.0)}'] = 1+0j
    R3cR_CharTable['G1']['{3^+_xxx|(0.0,0.0,0.0)}'] = 1+0j
    R3cR_CharTable['G1']['{m_xyy|(0.5,0.5,0.5)}'] = 1+0j
    R3cR_CharTable['G1']['{m_xxz|(0.5,0.5,0.5)}'] = 1+0j

    R3cR_CharTable['G0']['{3^-_xxx|(0.0,0.0,0.0)}'] = -1+0j
    R3cR_CharTable['G0']['{m_xyx|(0.5,0.5,0.5)}'] = 0j
    R3cR_CharTable['G0']['{1|(0.0,0.0,0.0)}'] = 2+0j
    R3cR_CharTable['G0']['{3^+_xxx|(0.0,0.0,0.0)}'] = -1+0j
    R3cR_CharTable['G0']['{m_xyy|(0.5,0.5,0.5)}'] = 0j
    R3cR_CharTable['G0']['{m_xxz|(0.5,0.5,0.5)}'] = 0j

    #R 3m:R, k = [0,0,0], Journal of Magnetism and Magnetic Materials 24 (1981) 267-278
    R3mR_CharTable=dict()
    R3mR_CharTable['G0']=dict()
    R3mR_CharTable['G1']=dict()
    R3mR_CharTable['G2']=dict()

    R3mR_CharTable['G2']['{3^-_xxx|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable['G2']['{m_xxz|(0.0,0.0,0.0)}'] = -1+0j
    R3mR_CharTable['G2']['{m_xyy|(0.0,0.0,0.0)}'] = -1+0j
    R3mR_CharTable['G2']['{1|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable['G2']['{3^+_xxx|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable['G2']['{m_xyx|(0.0,0.0,0.0)}'] = -1+0j

    R3mR_CharTable['G1']['{3^-_xxx|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable['G1']['{m_xxz|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable['G1']['{m_xyy|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable['G1']['{1|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable['G1']['{3^+_xxx|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable['G1']['{m_xyx|(0.0,0.0,0.0)}'] = 1+0j

    R3mR_CharTable['G0']['{3^-_xxx|(0.0,0.0,0.0)}'] = -1+0j
    R3mR_CharTable['G0']['{m_xxz|(0.0,0.0,0.0)}'] = 0j
    R3mR_CharTable['G0']['{m_xyy|(0.0,0.0,0.0)}'] = 0j
    R3mR_CharTable['G0']['{1|(0.0,0.0,0.0)}'] = 2+0j
    R3mR_CharTable['G0']['{3^+_xxx|(0.0,0.0,0.0)}'] = -1+0j
    R3mR_CharTable['G0']['{m_xyx|(0.0,0.0,0.0)}'] = 0j

    #R 3m:R, k = [0.5,0.5,0.5], Journal of Magnetism and Magnetic Materials 24 (1981) 267-278
    R3mR_CharTable2=dict()
    R3mR_CharTable2['G0']=dict()
    R3mR_CharTable2['G1']=dict()
    R3mR_CharTable2['G2']=dict()

    R3mR_CharTable2['G2']['{3^-_xxx|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable2['G2']['{m_xxz|(0.0,0.0,0.0)}'] = -1+0j
    R3mR_CharTable2['G2']['{m_xyy|(0.0,0.0,0.0)}'] = -1+0j
    R3mR_CharTable2['G2']['{1|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable2['G2']['{3^+_xxx|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable2['G2']['{m_xyx|(0.0,0.0,0.0)}'] = -1+0j

    R3mR_CharTable2['G1']['{3^-_xxx|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable2['G1']['{m_xxz|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable2['G1']['{m_xyy|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable2['G1']['{1|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable2['G1']['{3^+_xxx|(0.0,0.0,0.0)}'] = 1+0j
    R3mR_CharTable2['G1']['{m_xyx|(0.0,0.0,0.0)}'] = 1+0j

    R3mR_CharTable2['G0']['{3^-_xxx|(0.0,0.0,0.0)}'] = -1+0j
    R3mR_CharTable2['G0']['{m_xxz|(0.0,0.0,0.0)}'] = 0j
    R3mR_CharTable2['G0']['{m_xyy|(0.0,0.0,0.0)}'] = 0j
    R3mR_CharTable2['G0']['{1|(0.0,0.0,0.0)}'] = 2+0j
    R3mR_CharTable2['G0']['{3^+_xxx|(0.0,0.0,0.0)}'] = -1+0j
    R3mR_CharTable2['G0']['{m_xyx|(0.0,0.0,0.0)}'] = 0j


    #P4, k = [0.,0.,0.15] (example in J. Schweizer's talk)
    P4_CharTable=dict()
    P4_CharTable['G0']=dict()
    P4_CharTable['G1']=dict()
    P4_CharTable['G2']=dict()
    P4_CharTable['G3']=dict()

    P4_CharTable['G3']['{4^-_00z|(0.0,0.0,0.0)}'] = 1j
    P4_CharTable['G3']['{2_00z|(0.0,0.0,0.0)}'] = -1+0j
    P4_CharTable['G3']['{1|(0.0,0.0,0.0)}'] = 1+0j
    P4_CharTable['G3']['{4^+_00z|(0.0,0.0,0.0)}'] = -1j

    P4_CharTable['G2']['{4^-_00z|(0.0,0.0,0.0)}'] = -1j
    P4_CharTable['G2']['{2_00z|(0.0,0.0,0.0)}'] = -1+0j
    P4_CharTable['G2']['{1|(0.0,0.0,0.0)}'] = 1+0j
    P4_CharTable['G2']['{4^+_00z|(0.0,0.0,0.0)}'] = 1j

    P4_CharTable['G1']['{4^-_00z|(0.0,0.0,0.0)}'] = -1+0j
    P4_CharTable['G1']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    P4_CharTable['G1']['{1|(0.0,0.0,0.0)}'] = 1+0j
    P4_CharTable['G1']['{4^+_00z|(0.0,0.0,0.0)}'] = -1+0j

    P4_CharTable['G0']['{4^-_00z|(0.0,0.0,0.0)}'] = 1+0j
    P4_CharTable['G0']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    P4_CharTable['G0']['{1|(0.0,0.0,0.0)}'] = 1+0j
    P4_CharTable['G0']['{4^+_00z|(0.0,0.0,0.0)}'] = 1+0j


    #-P4, k = [0.,0.,0.15] (example in J. Schweizer's talk)
    mP4_CharTable=dict()
    mP4_CharTable['G0']=dict()
    mP4_CharTable['G1']=dict()
    mP4_CharTable['G2']=dict()
    mP4_CharTable['G3']=dict()

    mP4_CharTable['G3']['{4^-_00z|(0.0,0.0,0.0)}'] = 1j
    mP4_CharTable['G3']['{2_00z|(0.0,0.0,0.0)}'] = -1+0j
    mP4_CharTable['G3']['{1|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CharTable['G3']['{4^+_00z|(0.0,0.0,0.0)}'] = -1j

    mP4_CharTable['G2']['{4^-_00z|(0.0,0.0,0.0)}'] = -1j
    mP4_CharTable['G2']['{2_00z|(0.0,0.0,0.0)}'] = -1+0j
    mP4_CharTable['G2']['{1|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CharTable['G2']['{4^+_00z|(0.0,0.0,0.0)}'] = 1j

    mP4_CharTable['G1']['{4^-_00z|(0.0,0.0,0.0)}'] = -1+0j
    mP4_CharTable['G1']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CharTable['G1']['{1|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CharTable['G1']['{4^+_00z|(0.0,0.0,0.0)}'] = -1+0j

    mP4_CharTable['G0']['{4^-_00z|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CharTable['G0']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CharTable['G0']['{1|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CharTable['G0']['{4^+_00z|(0.0,0.0,0.0)}'] = 1+0j

    #RMn2O5, 'P b a m', k = [0.5,0,0.25]
    RMn2O5_CharTable=dict()
    RMn2O5_CharTable['G0']=dict()
    RMn2O5_CharTable['G0']['{m_0yz|(0.5,0.5,0.0)}'] = 0j
    RMn2O5_CharTable['G0']['{m_x0z|(0.5,0.5,0.0)}'] = 0j
    RMn2O5_CharTable['G0']['{1|(0.0,0.0,0.0)}'] = 2+0j
    RMn2O5_CharTable['G0']['{2_00z|(0.0,0.0,0.0)}'] = -0j

    #FeGe, 'P 21 3', k = [0.001,0,0]
    FeGe_CharTable=dict()
    FeGe_CharTable['G0']=dict()
    FeGe_CharTable['G0']['{1|(0.0,0.0,0.0)}'] = 1+0j
    FeGe_CharTable['G0']['{2_x00|(0.5,0.5,0.0)}'] = 0.95099999999999996-0.309j
    FeGe_CharTable['G1']=dict()
    FeGe_CharTable['G1']['{1|(0.0,0.0,0.0)}'] = 1+0j
    FeGe_CharTable['G1']['{2_x00|(0.5,0.5,0.0)}'] = -0.95099999999999996+0.309j

    #'P -4', k = [0,0,0.15]
    Pm4_CharTable=dict()
    Pm4_CharTable['G0']=dict()
    Pm4_CharTable['G0']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Pm4_CharTable['G0']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    Pm4_CharTable['G1']=dict()
    Pm4_CharTable['G1']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Pm4_CharTable['G1']['{2_00z|(0.0,0.0,0.0)}'] = -1+0j
    
    
    #space group label, k-vector, character table
    knownSmallIrrepsChar = (('P 21 3',  np.array([0.05,0.05,0.05]), MnSi_CharTable),
                       ('F d -3 m:2',  np.array([0.45,0.55,0.5]), CeAl2_CharTable),
                       ('P 21/b 1 1',   np.array([0.14,0,0]), MnP_CharTable),
                       ('C 1 2/c 1',    np.array([0.5,0,-0.5]), CuO_CharTable),
                       ('I -4 2 d',     np.array([0.,0,0.]), CuB2O4_CharTable),
                       ('I m m m',      np.array([0.5,0,0.5]), Ho2BaNiO5_CharTable),
                       ('P 4/m b m',    np.array([0.1,0,0]), Ce2Pd2Sn_CharTable),
                       ('I 4/m',        np.array([1,0,0]), KTb3F12_CharTable),
                       ('P n m a',      np.array([0,0,0.25]),PrPtAl_CharTable),
                       ('R -3 m:H',     np.array([0,0,1.5]),Jarosite_CharTable),
                       ('P 63 m c',     np.array([0.5,0,0]), HoAuGe_CharTable),
                       ('R 32:H',       np.array([0.,0.,1.5]), NdFeBO_CharTable),
                       ('R 3 c:R',      np.array([0.,0.,0.]), R3cR_CharTable),
                       ('R 3 m:R',      np.array([0.,0.,0.]), R3mR_CharTable),
                       ('P4',           np.array([0.0,0.0,0.15]), P4_CharTable),
                       ('-P4',          np.array([0.0,0.0,0.15]), mP4_CharTable),
                       ('P b a m',      np.array([0.5,0.0,0.25]), RMn2O5_CharTable),
                       ('P 21 3',       np.array([0.1,0,0]), FeGe_CharTable),
                       ('P -4',         np.array([0,0,0.15]), Pm4_CharTable),
                      )

    def testSmallIrreps(self):
        """*--> test character tables of irreducible representations of little groups"""
        for sg, kvector, chartable_calculated in self.knownSmallIrrepsChar:
            gt=SpaceGroupIrreps(sg,kvector)
            cosetsofg,littlegroupoperations,onBZ=gt.KVectorGroup()
            #calculate allowed irreps of the little group
            Irreps=gt.CalcSpaceGroupIrreps(cosetsofg,littlegroupoperations,onBZ)
            charactertable,charT=gt.CharacterTable(gt.allowedirrepsofgk)
            #test number of irreps is ok
            self.assertEqual(len(charT),len(chartable_calculated),
            '\n\n*--number of irreducible representations not correct: expected {0} and got{1}'.format(chartable_calculated,charT))
            #test symmetry elements in little group are ok
            self.assertEqual(set(charT['G0'].keys()),set(chartable_calculated['G0'].keys()))
            #test values in character table
            for irrep,value in charT.iteritems():
                char_test=[]
                char_calc=[]
                for op,char in value.iteritems():
                    char_test.append(chartable_calculated[irrep][op])
                    char_calc.append(char)
                result=len([i for i,j in zip(np.around(char_test,3),np.around(char_calc,3)) if i == j])
                self.assertEqual(len(char_calc),result,'\n\n*--for space group {0} and k-vector {1}:\nexpected {2} but got {3}'.format(sg,kvector,char_test,char_calc))



class KnownSmallCoreps(unittest.TestCase):
    '''
    Test of allowed irreducible co-representations of the magnetic little group.
    The test compares the calculated with the known values of the character table
    '''

    #Calculated values from Herring test
    #CuO
    CuO_RealityTest=dict()
    CuO_RealityTest['G0']='a'
    CuO_RealityTest['G1']='a'

    #Mn2GeO4
    Mn2GeO4_RealityTest=dict()
    Mn2GeO4_RealityTest['G0']='a'
    Mn2GeO4_RealityTest['G1']='a'

    #CeAl2
    CeAl2_RealityTest=dict()
    CeAl2_RealityTest['G0']='a'
    CeAl2_RealityTest['G1']='a'

    #PrPtAl
    PrPtAl_RealityTest=dict()
    PrPtAl_RealityTest['G0']='a'
    PrPtAl_RealityTest['G1']='a'
    PrPtAl_RealityTest['G2']='a'
    PrPtAl_RealityTest['G3']='a'

    #MnP
    MnP_RealityTest=dict()
    MnP_RealityTest['G0']='a'
    MnP_RealityTest['G1']='a'
    MnP_RealityTest['G2']='a'
    MnP_RealityTest['G3']='a'

    #-P4
    mP4_RealityTest=dict()
    mP4_RealityTest['G0']='a'
    mP4_RealityTest['G1']='a'
    mP4_RealityTest['G2']='c'
    mP4_RealityTest['G3']='c'

    #RMn2O5, 'P b a m', k = [0.5,0,0.25]
    RMn2O5_RealityTest=dict()
    RMn2O5_RealityTest['G0']='a'

    #FeGe, 'P 21 3',  k = [0.1,0,0] 
    FeGe_RealityTest=dict()
    FeGe_RealityTest['G0']='a'
    FeGe_RealityTest['G1']='a'
    
    #P -4, 'P -4',  k = [0,0,0.15] 
    Pm4_RealityTest=dict()
    Pm4_RealityTest['G0']='a'
    Pm4_RealityTest['G1']='b'
    
    #Calculated values of irreducible co-representations
    #mP4
    mP4_CoRepsCharTable=dict()
    mP4_CoRepsCharTable['G0m']=dict()
    mP4_CoRepsCharTable['G0p']=dict()
    mP4_CoRepsCharTable['G1m']=dict()
    mP4_CoRepsCharTable['G1p']=dict()

    mP4_CoRepsCharTable['G2']=dict()
    mP4_CoRepsCharTable['G3']=dict()

    mP4_CoRepsCharTable['G0m']['T*{-4^-_00z|(0.0,0.0,0.0)}'] = -1-0j
    mP4_CoRepsCharTable['G0m']['T*{-1|(0.0,0.0,0.0)}'] = -1-0j
    mP4_CoRepsCharTable['G0m']['{1|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G0m']['{4^-_00z|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G0m']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G0m']['T*{-4^+_00z|(0.0,0.0,0.0)}'] = -1-0j
    mP4_CoRepsCharTable['G0m']['{4^+_00z|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G0m']['T*{m_xy0|(0.0,0.0,0.0)}'] = -1-0j

    mP4_CoRepsCharTable['G1m']['T*{-4^-_00z|(0.0,0.0,0.0)}'] = 1-0j
    mP4_CoRepsCharTable['G1m']['T*{-1|(0.0,0.0,0.0)}'] = -1-0j
    mP4_CoRepsCharTable['G1m']['{1|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G1m']['{4^-_00z|(0.0,0.0,0.0)}'] = -1+0j
    mP4_CoRepsCharTable['G1m']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G1m']['T*{-4^+_00z|(0.0,0.0,0.0)}'] = 1-0j
    mP4_CoRepsCharTable['G1m']['{4^+_00z|(0.0,0.0,0.0)}'] = -1+0j
    mP4_CoRepsCharTable['G1m']['T*{m_xy0|(0.0,0.0,0.0)}'] = -1-0j

    mP4_CoRepsCharTable['G3']['T*{-4^-_00z|(0.0,0.0,0.0)}'] = 0j
    mP4_CoRepsCharTable['G3']['T*{-1|(0.0,0.0,0.0)}'] = 0j
    mP4_CoRepsCharTable['G3']['{1|(0.0,0.0,0.0)}'] = 2+0j
    mP4_CoRepsCharTable['G3']['{4^-_00z|(0.0,0.0,0.0)}'] = 0j
    mP4_CoRepsCharTable['G3']['{2_00z|(0.0,0.0,0.0)}'] = -2+0j
    mP4_CoRepsCharTable['G3']['T*{-4^+_00z|(0.0,0.0,0.0)}'] = 0j
    mP4_CoRepsCharTable['G3']['{4^+_00z|(0.0,0.0,0.0)}'] = 0j
    mP4_CoRepsCharTable['G3']['T*{m_xy0|(0.0,0.0,0.0)}'] = 0j

    mP4_CoRepsCharTable['G2']['T*{-4^-_00z|(0.0,0.0,0.0)}'] = 0j
    mP4_CoRepsCharTable['G2']['T*{-1|(0.0,0.0,0.0)}'] = 0j
    mP4_CoRepsCharTable['G2']['{1|(0.0,0.0,0.0)}'] = 2+0j
    mP4_CoRepsCharTable['G2']['{4^-_00z|(0.0,0.0,0.0)}'] = 0j
    mP4_CoRepsCharTable['G2']['{2_00z|(0.0,0.0,0.0)}'] = -2+0j
    mP4_CoRepsCharTable['G2']['T*{-4^+_00z|(0.0,0.0,0.0)}'] = 0j
    mP4_CoRepsCharTable['G2']['{4^+_00z|(0.0,0.0,0.0)}'] = 0j
    mP4_CoRepsCharTable['G2']['T*{m_xy0|(0.0,0.0,0.0)}'] = 0j

    mP4_CoRepsCharTable['G0p']['T*{-4^-_00z|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G0p']['T*{-1|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G0p']['{1|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G0p']['{4^-_00z|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G0p']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G0p']['T*{-4^+_00z|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G0p']['{4^+_00z|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G0p']['T*{m_xy0|(0.0,0.0,0.0)}'] = 1+0j

    mP4_CoRepsCharTable['G1p']['T*{-4^-_00z|(0.0,0.0,0.0)}'] = -1+0j
    mP4_CoRepsCharTable['G1p']['T*{-1|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G1p']['{1|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G1p']['{4^-_00z|(0.0,0.0,0.0)}'] = -1+0j
    mP4_CoRepsCharTable['G1p']['{2_00z|(0.0,0.0,0.0)}'] = 1+0j
    mP4_CoRepsCharTable['G1p']['T*{-4^+_00z|(0.0,0.0,0.0)}'] = -1+0j
    mP4_CoRepsCharTable['G1p']['{4^+_00z|(0.0,0.0,0.0)}'] = -1+0j
    mP4_CoRepsCharTable['G1p']['T*{m_xy0|(0.0,0.0,0.0)}'] = 1+0j

    #CuO
    CuO_CoRepsCharTable=dict()
    CuO_CoRepsCharTable['G0m']=dict()
    CuO_CoRepsCharTable['G0p']=dict()
    CuO_CoRepsCharTable['G1m']=dict()
    CuO_CoRepsCharTable['G1p']=dict()

    CuO_CoRepsCharTable['G0m']['{m_x0z|(0.0,0.0,0.5)}'] = 1j
    CuO_CoRepsCharTable['G0m']['T*{2_0y0|(0.0,0.0,-0.5)}'] = -0+1j
    CuO_CoRepsCharTable['G0m']['{1|(0.0,0.0,0.0)}'] = 1+0j
    CuO_CoRepsCharTable['G0m']['T*{-1|(0.0,0.0,0.0)}'] = -1-0j
    CuO_CoRepsCharTable['G1m']['{m_x0z|(0.0,0.0,0.5)}'] = -0-1j
    CuO_CoRepsCharTable['G1m']['T*{2_0y0|(0.0,0.0,-0.5)}'] = -1j
    CuO_CoRepsCharTable['G1m']['{1|(0.0,0.0,0.0)}'] = 1+0j
    CuO_CoRepsCharTable['G1m']['T*{-1|(0.0,0.0,0.0)}'] = -1-0j

    CuO_CoRepsCharTable['G0p']['{m_x0z|(0.0,0.0,0.5)}'] = 1j
    CuO_CoRepsCharTable['G0p']['T*{2_0y0|(0.0,0.0,-0.5)}'] = -1j
    CuO_CoRepsCharTable['G0p']['{1|(0.0,0.0,0.0)}'] = 1+0j
    CuO_CoRepsCharTable['G0p']['T*{-1|(0.0,0.0,0.0)}'] = 1+0j
    CuO_CoRepsCharTable['G1p']['{m_x0z|(0.0,0.0,0.5)}'] = -0-1j
    CuO_CoRepsCharTable['G1p']['T*{2_0y0|(0.0,0.0,-0.5)}'] = -0+1j
    CuO_CoRepsCharTable['G1p']['{1|(0.0,0.0,0.0)}'] = 1+0j
    CuO_CoRepsCharTable['G1p']['T*{-1|(0.0,0.0,0.0)}'] = 1+0j

    #Mn2GeO4
    Mn2GeO4_CoRepsCharTable=dict()
    Mn2GeO4_CoRepsCharTable['G0m']=dict()
    Mn2GeO4_CoRepsCharTable['G0p']=dict()
    Mn2GeO4_CoRepsCharTable['G1m']=dict()
    Mn2GeO4_CoRepsCharTable['G1p']=dict()

    Mn2GeO4_CoRepsCharTable['G0m']['T*{-1|(0.0,0.0,0.0)}'] = -1-0j
    Mn2GeO4_CoRepsCharTable['G0m']['T*{2_00z|(-0.5,0.0,-0.5)}'] = -0.89100000000000001-0.45400000000000001j
    Mn2GeO4_CoRepsCharTable['G0m']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Mn2GeO4_CoRepsCharTable['G0m']['{m_xy0|(0.5,0.0,0.5)}'] = 0.89100000000000001-0.45400000000000001j

    Mn2GeO4_CoRepsCharTable['G1m']['T*{-1|(0.0,0.0,0.0)}']= -1-0j
    Mn2GeO4_CoRepsCharTable['G1m']['T*{2_00z|(-0.5,0.0,-0.5)}'] = 0.89100000000000001+0.45400000000000001j
    Mn2GeO4_CoRepsCharTable['G1m']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Mn2GeO4_CoRepsCharTable['G1m']['{m_xy0|(0.5,0.0,0.5)}'] = -0.89100000000000001+0.45400000000000001j

    Mn2GeO4_CoRepsCharTable['G0p']['T*{-1|(0.0,0.0,0.0)}'] = 1+0j
    Mn2GeO4_CoRepsCharTable['G0p']['T*{2_00z|(-0.5,0.0,-0.5)}'] = 0.89100000000000001+0.45400000000000001j
    Mn2GeO4_CoRepsCharTable['G0p']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Mn2GeO4_CoRepsCharTable['G0p']['{m_xy0|(0.5,0.0,0.5)}'] = 0.89100000000000001-0.45400000000000001j

    Mn2GeO4_CoRepsCharTable['G1p']['T*{-1|(0.0,0.0,0.0)}'] = 1+0j
    Mn2GeO4_CoRepsCharTable['G1p']['T*{2_00z|(-0.5,0.0,-0.5)}'] = -0.89100000000000001-0.45400000000000001j
    Mn2GeO4_CoRepsCharTable['G1p']['{1|(0.0,0.0,0.0)}'] = 1+0j
    Mn2GeO4_CoRepsCharTable['G1p']['{m_xy0|(0.5,0.0,0.5)}'] = -0.89100000000000001+0.45400000000000001j

    #RMn2O5
    RMn2O5_CoRepsCharTable=dict()
    RMn2O5_CoRepsCharTable['G0m']=dict()
    RMn2O5_CoRepsCharTable['G0p']=dict()
    RMn2O5_CoRepsCharTable['G1m']=dict()
    RMn2O5_CoRepsCharTable['G1p']=dict()

    RMn2O5_CoRepsCharTable['G0m']['{m_0yz|(0.5,0.5,0.0)}'] = 0j
    RMn2O5_CoRepsCharTable['G0m']['T*{2_x00|(-0.5,-0.5,0.0)}'] = -0-0j
    RMn2O5_CoRepsCharTable['G0m']['{1|(0.0,0.0,0.0)}'] = 2+0j
    RMn2O5_CoRepsCharTable['G0m']['{2_00z|(0.0,0.0,0.0)}'] = -0j
    RMn2O5_CoRepsCharTable['G0m']['T*{2_0y0|(-0.5,-0.5,0.0)}'] = -0+0j
    RMn2O5_CoRepsCharTable['G0m']['T*{-1|(0.0,0.0,0.0)}'] = 0j
    RMn2O5_CoRepsCharTable['G0m']['{m_x0z|(0.5,0.5,0.0)}'] = 0j
    RMn2O5_CoRepsCharTable['G0m']['T*{m_xy0|(0.0,0.0,0.0)}'] = 2+0j

    RMn2O5_CoRepsCharTable['G0p']['{m_0yz|(0.5,0.5,0.0)}'] = 0j
    RMn2O5_CoRepsCharTable['G0p']['T*{2_x00|(-0.5,-0.5,0.0)}']= 0j
    RMn2O5_CoRepsCharTable['G0p']['{1|(0.0,0.0,0.0)}'] = 2+0j
    RMn2O5_CoRepsCharTable['G0p']['{2_00z|(0.0,0.0,0.0)}'] = -0j
    RMn2O5_CoRepsCharTable['G0p']['T*{2_0y0|(-0.5,-0.5,0.0)}'] = -0j
    RMn2O5_CoRepsCharTable['G0p']['T*{-1|(0.0,0.0,0.0)}'] = -0j
    RMn2O5_CoRepsCharTable['G0p']['{m_x0z|(0.5,0.5,0.0)}'] = 0j
    RMn2O5_CoRepsCharTable['G0p']['T*{m_xy0|(0.0,0.0,0.0)}'] = -2-0j

    FeGe_CoRepsCharTable=dict()
    FeGe_CoRepsCharTable['G0m']=dict()
    FeGe_CoRepsCharTable['G0p']=dict()
    FeGe_CoRepsCharTable['G1m']=dict()
    FeGe_CoRepsCharTable['G1p']=dict()
    FeGe_CoRepsCharTable['G0m']['{1|(0.0,0.0,0.0)}'] = 1 + 0.j
    FeGe_CoRepsCharTable['G0m']['{2_x00|(0.5,0.5,0.0)}'] =  0.95099999999999996-0.309j
    FeGe_CoRepsCharTable['G0m']['T*{2_0y0|(0.0,-0.5,0.5)}'] = -0.95099999999999996-0.309j
    FeGe_CoRepsCharTable['G0m']['T*{2_00z|(0.5,0.0,0.5)}'] = -1 + 0j
    FeGe_CoRepsCharTable['G0p']['{1|(0.0,0.0,0.0)}'] = 1 + 0.j
    FeGe_CoRepsCharTable['G0p']['{2_x00|(0.5,0.5,0.0)}'] =  0.95099999999999996-0.309j
    FeGe_CoRepsCharTable['G0p']['T*{2_0y0|(0.0,-0.5,0.5)}'] = 0.95099999999999996+0.309j
    FeGe_CoRepsCharTable['G0p']['T*{2_00z|(0.5,0.0,0.5)}'] = 1 + 0j
    FeGe_CoRepsCharTable['G1m']['{1|(0.0,0.0,0.0)}'] = 1 + 0.j
    FeGe_CoRepsCharTable['G1m']['{2_x00|(0.5,0.5,0.0)}'] = -0.95099999999999996+0.309j
    FeGe_CoRepsCharTable['G1m']['T*{2_0y0|(0.0,-0.5,0.5)}'] = 0.95099999999999996+0.309j
    FeGe_CoRepsCharTable['G1m']['T*{2_00z|(0.5,0.0,0.5)}'] = -1 + 0j
    FeGe_CoRepsCharTable['G1p']['{1|(0.0,0.0,0.0)}'] = 1 + 0.j
    FeGe_CoRepsCharTable['G1p']['{2_x00|(0.5,0.5,0.0)}'] = -0.95099999999999996+0.309j
    FeGe_CoRepsCharTable['G1p']['T*{2_0y0|(0.0,-0.5,0.5)}'] =-0.95099999999999996-0.309j
    FeGe_CoRepsCharTable['G1p']['T*{2_00z|(0.5,0.0,0.5)}'] = 1 + 0j


    Pm4_CoRepsCharTable=dict()
    Pm4_CoRepsCharTable['G0m']=dict()
    Pm4_CoRepsCharTable['G0p']=dict()
    Pm4_CoRepsCharTable['G1']=dict()
    Pm4_CoRepsCharTable['G0m']['{1|(0.0,0.0,0.0)}']=1+0j
    Pm4_CoRepsCharTable['G0m']['{2_00z|(0.0,0.0,0.0)}']=1+0j
    Pm4_CoRepsCharTable['G0m']['T*{-4^-_00z|(0.0,0.0,0.0)}']=-1+0j 
    Pm4_CoRepsCharTable['G0m']['T*{-4^+_00z|(0.0,0.0,0.0)}']=-1+0j
    Pm4_CoRepsCharTable['G0p']['{1|(0.0,0.0,0.0)}']=1+0j
    Pm4_CoRepsCharTable['G0p']['{2_00z|(0.0,0.0,0.0)}']=1+0j
    Pm4_CoRepsCharTable['G0p']['T*{-4^-_00z|(0.0,0.0,0.0)}']=1+0j 
    Pm4_CoRepsCharTable['G0p']['T*{-4^+_00z|(0.0,0.0,0.0)}']=1+0j
    Pm4_CoRepsCharTable['G1']['{1|(0.0,0.0,0.0)}']=2+0j
    Pm4_CoRepsCharTable['G1']['{2_00z|(0.0,0.0,0.0)}']=-2+0j
    Pm4_CoRepsCharTable['G1']['T*{-4^-_00z|(0.0,0.0,0.0)}']=0j
    Pm4_CoRepsCharTable['G1']['T*{-4^+_00z|(0.0,0.0,0.0)}']=0j

    #herring's test
    #space group label, k-vector, result of herring test
    knownHerring = (('C 1 2/c 1',  np.array([0.5,0.0,-0.5]), CuO_RealityTest),
                    ('P n m a',    np.array([0.15,0.15,0.0]), Mn2GeO4_RealityTest),
                    ('F d -3 m:2', np.array([0.45,0.55,0.5]),CeAl2_RealityTest),
                    ('P m n a',    np.array([0.,0.,0.45]),   PrPtAl_RealityTest),
                    ('P 21/b 1 1', np.array([0.14,0,0]), MnP_RealityTest),
                    ('-P 4',       np.array([0.0,0,0.15]), mP4_RealityTest),
                    ('P b a m',    np.array([0.5,0,0.25]), RMn2O5_RealityTest),
                    ('P 21 3',     np.array([0.1,0,0]), FeGe_RealityTest),
                    ('P -4',       np.array([0,0,0.15]), Pm4_RealityTest),
                   )

    #character table co-representations
    #space group label, k-vector, character table
    knownCoReps = (('C 1 2/c 1',  np.array([0.5,0.0,-0.5]), CuO_CoRepsCharTable),
                   ('P n m a',    np.array([0.15,0.15, 0.0]), Mn2GeO4_CoRepsCharTable),
                   ('-P 4',       np.array([0.0,0,0.15]), mP4_CoRepsCharTable),
                   ('P b a m',    np.array([0.5,0,0.25]), RMn2O5_CoRepsCharTable),
                   ('P 21 3',     np.array([0.1,0,0]), FeGe_CoRepsCharTable),
                   ('P -4',      np.array([0,0,0.15]), Pm4_CoRepsCharTable),
                  )

    def testHerring(self):
        """*--> test value returned by Herring test"""
        for sg, kvector, realitytest_calc in self.knownHerring:
            gt=SpaceGroupIrreps(sg,kvector)
            cosetsofg,littlegroupoperations,onBZ=gt.KVectorGroup()
            Irreps=gt.CalcSpaceGroupIrreps(cosetsofg,littlegroupoperations,onBZ)
            charactertable,characterlittlegroup=gt.CharacterTable(gt.allowedirrepsofgk)
            corepcase=gt.RealityTest(characterlittlegroup)
            for G, reality in corepcase.iteritems():
                self.assertEqual(reality,realitytest_calc[G],
                '\n\n*--for space group {0} and k-vector {1}:\nexpected {2} but got {3}'.format(sg,kvector,reality,realitytest_calc[G]))


    def testCoIrreps(self):
        """*--> test character tables of irreducible co-representations of the magnetic little groups"""
        for sg, kvector, chartable_calculated in self.knownCoReps:
            gt=SpaceGroupIrreps(sg,kvector)
            cosetsofg,littlegroupoperations,onBZ=gt.KVectorGroup()
            Irreps=gt.CalcSpaceGroupIrreps(cosetsofg,littlegroupoperations,onBZ)
            charactertable,characterlittlegroup=gt.CharacterTable(gt.allowedirrepsofgk)
            corepcase=gt.RealityTest(characterlittlegroup)
            rcoreps=gt.ReduceCoReps(gt.allowedirrepsofgk,corepcase)
            charactertable,charT=gt.CharacterTable(rcoreps)
            for irrep,value in charT.iteritems():
                char_test=[]
                char_calc=[]
                for op,char in value.iteritems():
                    char_test.append(chartable_calculated[irrep][op])
                    char_calc.append(char)
                result=len([i for i,j in zip(np.around(char_test,3),np.around(char_calc,3)) if i == j])
                self.assertEqual(len(char_calc),result,'\n\n*--for space group {0} and k-vector {1}:\nexpected {2} but got {3}'.format(sg,kvector,char_test,char_calc))


if __name__ == "__main__":
    import sys
    import tempfile

    print 'Test irreducible representations'
    sys.stdout = tempfile.TemporaryFile()
    #turn off output
    unittest.main()
    sys.stdout.close()
    #now the temp file is gone
    sys.stdout = sys.__stdout__
