#!/usr/bin/env python
"""
Test of little group and k-vector
"""
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "beta"

from Kvector import Kvector
import unittest

import numpy as np

class test_Kvector(unittest.TestCase):
    #test of little co-group
    knownCogroups = (('-P 2c', np.array([0.5,0,0.5]), ['1', '-1', '2_00z', 'm_xy0']),
                    ('C 2/c 1 1' ,np.array([0.5,0.0,0.5]),['1', '2_x00']),
                    ('C 1 2/c 1' ,np.array([0.5,0.0,-0.5]),['1', 'm_x0z']),
                    ('R -3 m:H', np.array([0,0,1.5]),['3^-_00z', 'm_x2xz', '2_xx0', 'm_x-xz', '1', '-3^+_00z', 'm_2xxz', '-1', '-3^-_00z', '3^+_00z', '2_0y0', '2_x00']),
                    ('P 21 21 21', np.array([0,0,0]),['1', '2_00z', '2_0y0', '2_x00']),
                    ('P 21 1 1', np.array([0.,0,0.25]), ['1']),
                    ('P 21 1 1', np.array([0.25,0,0.]), ['1','2_x00']),
                    ('P 3', np.array([1./3.,1./3.,0.]),['1', '3^-_00z', '3^+_00z']),
                    ('P 21 3', np.array([0.0020,0.0020,0.0020]),['1', '3^+_xxx', '3^-_xxx']),
                    ('F d -3 m:2',np.array([0.45,0.55,0.5]),['1', '2_x-x0']),
                    ('I m m m', np.array([0.5,0.0,0.5]),['1', '-1', '2_0y0', 'm_x0z']),
                    ('P 4/m b m', np.array([0.1,0.0,0.]),['m_xy0', '1', 'm_x0z', '2_x00']),
                    ('I 4/m',  np.array([1,0,0]),['2_00z', 'm_xy0', '1', '-1', '-4^+_00z', '4^-_00z', '-4^-_00z', '4^+_00z']),
                    ('P n m a', np.array([0,0,0.25]),['m_0yz', '2_00z', '1', 'm_x0z']),
                    ('P 63 mc', np.array([0.5,0,0]),['m_0yz', '2_00z', '1', 'm_2xxz']),
                    ('R 3 2:H', np.array([0,0,1.5]),['3^-_00z', '2_xx0', '1', '3^+_00z', '2_0y0', '2_x00']),
                    ('R 3 m:R', np.array([0.5,0.5,0.5]),['m_xyx', 'm_xyy', '3^+_xxx', 'm_xxz', '1', '3^-_xxx']),
                    ('P4',np.array([0.,0.,0.15]),['1', '2_00z', '4^+_00z', '4^-_00z']),
                    ('-P4',np.array([0.,0.,0.15]),['1', '2_00z', '4^+_00z', '4^-_00z']),
                    ('P n m a', np.array([0.15,0.15,0]),['m_xy0', '1']),
                    ('I 41/a:2',np.array([0,0,0]),['2_00z', 'm_xy0', '1', '-1', '-4^+_00z', '4^-_00z', '-4^-_00z', '4^+_00z']),
                    ('P b a m',np.array([0.5,0,0.25]),['m_0yz', '2_00z', '1', 'm_x0z']),
                    ('R -3 c:R',np.array([0.5,0.5,0]),['m_xxz', '2_x-x0', '1', '-1']),
                    ('P 21 3',np.array([0.001,0,0]),['1', '2_x00']),
                    ('P 1 2/c 1', np.array([-0.226,0.5,0.44]),['1', 'm_x0z']),
                    ('P -4', np.array([0,0,0.15]),['1', '2_00z']),
                    )

    #test if k is a special point on BZ, i.e. (\alpha^*\vec k -\vec k)\vec \tau ? 0,
    #see Eq.42 in Ballou, Ouladiaf
    knownBZboundary = (('-P 2c', np.array([0.5,0,0.5]),True),
                   ('C 1 2/c 1' ,np.array([0.5,0.0,-0.5]),False),
                   ('R -3 m:H', np.array([0,0,1.5]),False),
                   ('P 21 21 21', np.array([0,0,0]),False),
                   ('P 21 1 1',np.array([0.,0,0.25]),False),
                   ('P 21 1 1',np.array([0.25,0,0]),False),
                   ('P 3', np.array([1./3.,1./3.,0.]),False),
                   ('P 21 3',np.array([0.0020,0.0020,0.0020]),False),
                   ('F d -3 m:2',np.array([0.45,0.55,0.5]),False),
                   ('I m m m', np.array([0.5,0.0,0.5]),False),
                   ('P 4/m b m', np.array([0.1,0.0,0.]),False),
                   ('I 4/m', np.array([1,0,0]),False),
                   ('P n m a', np.array([0,0,0.25]),False),
                   ('P 63 mc', np.array([0.5,0,0]),True),
                   ('R 3 2:H', np.array([0,0,1.5]),False),
                   ('R 3 m:R', np.array([0.5,0.5,0.5]), False),
                   ('P4',np.array([0.,0.,0.15]),False),
                   ('-P4',np.array([0.,0.,0.15]),False),
                   ('P n m a', np.array([0.15,0.15,0]),False),
                   ('I 41/a:2',np.array([0,0,0]),False),
                   ('P b a m',np.array([0.5,0,0.25]),True),
                   ('P 21 3',np.array([0.001,0,0]),False),
                   ('P 1 2/c 1', np.array([-0.226,0.5,0.44]),True),
                   ('P -4', np.array([0,0,0.15]),False),
                   )

    def testCoGroup(self):
        """*--> test little co-groups of k-vector"""
        for sg, kvector, cogroup in self.knownCogroups:
            gt=Kvector(sg,kvector)
            gt.LittleCoGroup()
            result=set(np.sort(gt.cogroupsymmetryoperations))
            cogroup=set(np.sort(cogroup))
            self.assertEqual(result,cogroup,'\n\n*--for space group {0} and k-vector {1}:\nexpected {2} but got {3}'.format(sg,kvector,cogroup,result))

    def testBZboundary(self):
        """*--> tests if k-vectors are on a Brillouin zone boundary"""
        for sg, kvector, BZ in self.knownBZboundary:
            gt=Kvector(sg,kvector)
            gt.LittleCoGroup()
            result=gt.KVectorOnBZB()
            self.assertEqual(BZ,result,'\n\n*for space group {0} and k-vector {1}:\nexpected {2} but got {3}'.format(sg,kvector,BZ,result))


if __name__ == "__main__":
    import sys
    import tempfile
    #turn off output
    print 'Test little co-group of k-vector'
    sys.stdout = tempfile.TemporaryFile()
    unittest.main()
    sys.stdout.close()
    #now the temp file is gone
    sys.stdout = sys.__stdout__
