# -*- encoding: utf-8 -*-

__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"

from collections import defaultdict
import numpy as np
import sys
import wx
import wx.lib.agw.ultimatelistctrl as ulc

if wx.VERSION < ('2.8.11.0'):
    from wx.lib.pubsub import Publisher
else:
    from wx.lib.pubsub import setuparg1
    from wx.lib.pubsub import pub as Publisher

from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin
#from wx.lib.mixins.listctrl import TextEditMixin
class AutoWidthListCtrl(ulc.UltimateListCtrl, ListCtrlAutoWidthMixin):
    def __init__(self,parent):
        ulc.UltimateListCtrl.__init__(self, parent,id=wx.ID_ANY, agwStyle=wx.LC_REPORT|ulc.ULC_HAS_VARIABLE_ROW_HEIGHT)
        ListCtrlAutoWidthMixin.__init__(self)

class AtomicBonds(wx.Panel):
    def __init__(self, parent):
        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().subscribe(self.GetAtomicCoordinates, ("atomstobondspage"))
        else:
            Publisher.subscribe(self.GetAtomicCoordinates, ("atomstobondspage"))

        wx.Panel.__init__(self, parent)
        sb = wx.StaticBox(self,label="Atoms")
        vbox_sizer = wx.StaticBoxSizer(sb,wx.VERTICAL)
        hbox1 = wx.BoxSizer(wx.HORIZONTAL)
        #atom 1 and 2
        cb1_label = wx.StaticText(self, label='atom 1: ')
        self.cb1 = wx.ComboBox(self, style=wx.CB_READONLY)
        spacer1 = wx.StaticText(self, label='    ')
        cb2_label = wx.StaticText(self, label='atom 2: ')
        self.cb2 = wx.ComboBox(self, style=wx.CB_READONLY)
        spacer2 = wx.StaticText(self, label='    ')
        #min. and max. distance
        dmin_label = wx.StaticText(self, label='from ('+ u'\N{ANGSTROM SIGN}'+'):')
        self.dmin = wx.TextCtrl(self)
        self.dmin.SetToolTip(wx.ToolTip("min. distance for bond"))
        spacer3 = wx.StaticText(self, label='    ')
        dmax_label = wx.StaticText(self, label='to ('+ u'\N{ANGSTROM SIGN}'+'):')
        self.dmax = wx.TextCtrl(self)
        self.dmax.SetToolTip(wx.ToolTip("max. distance for bond"))

        hbox1.Add(cb1_label,0,wx.ALIGN_CENTER)
        hbox1.Add(self.cb1,0,wx.ALIGN_CENTER)
        hbox1.Add(spacer1,0,wx.ALIGN_CENTER)
        hbox1.Add(cb2_label,0,wx.ALIGN_CENTER)
        hbox1.Add(self.cb2,0,wx.ALIGN_CENTER)
        hbox1.Add(spacer2,0,wx.ALIGN_CENTER)
        hbox1.Add(dmin_label,0,wx.ALIGN_CENTER)
        hbox1.Add(self.dmin,0,wx.ALIGN_CENTER)
        hbox1.Add(spacer3,0,wx.ALIGN_CENTER)
        hbox1.Add(dmax_label,0,wx.ALIGN_CENTER)
        hbox1.Add(self.dmax,0,wx.ALIGN_CENTER)
        vbox_sizer.Add(hbox1, 1, wx.EXPAND | wx.ALL)  # proportion 1

        panel = wx.Panel(self, -1) 
        hbox2 = wx.BoxSizer(wx.HORIZONTAL)
        ADD_button = wx.Button(panel, label='Add')
        ADD_button.SetToolTip(wx.ToolTip("Add bond to table"))
        ADD_button.Bind(wx.EVT_BUTTON, self.OnADDClicked)
        hbox2.Add(ADD_button,flag=wx.ALIGN_RIGHT | wx.ALL, border=5)

        DELETE_button = wx.Button(panel,label="Delete")
        DELETE_button.SetToolTip(wx.ToolTip("Select bond in table and click to delete"))
        DELETE_button.Bind(wx.EVT_BUTTON, self.DeleteBondFromList)
        hbox2.Add(DELETE_button,flag=wx.ALIGN_RIGHT | wx.ALL, border=5)
        panel.SetSizer(hbox2)
        vbox_sizer.Add(panel, 1)

        self.color_picker = dict()
        self.color_index = dict()
        self.bondradius_index = dict()
        #control list
        self.list_ctrl = AutoWidthListCtrl(self)
        self.list_ctrl.Bind(wx.EVT_LIST_ITEM_SELECTED,self.get_selected_row)
        self.list_ctrl.InsertColumn(0, 'Atom1',wx.LIST_FORMAT_CENTER,width=100)
        self.list_ctrl.InsertColumn(1, 'Atom2',wx.LIST_FORMAT_CENTER,width=100)
        self.list_ctrl.InsertColumn(2, 'Min. Distance ('+u'\N{ANGSTROM SIGN}' + ')',wx.LIST_FORMAT_CENTER,width=150)
        self.list_ctrl.InsertColumn(3, 'Max. Distance (A)',wx.LIST_FORMAT_CENTER,width=150)
        self.list_ctrl.InsertColumn(4, 'color',wx.LIST_FORMAT_CENTER,width=100)
        self.list_ctrl.InsertColumn(5, 'radius (x10000)',wx.LIST_FORMAT_CENTER,width=200)

        vbox_sizer.Add(self.list_ctrl, 5, wx.EXPAND | wx.ALL)
        self.SetSizer(vbox_sizer)
        self.chemicalbondsdict={}
        self.row_index=-1
        return

    def CalcBonds(self, at1, at2, dmin, dmax):
        """
        calculate bonds lengthes and coordinates between atom with labels at1 and atom with label at2 
        and lengthes bewteen dmin and dmax
        """
        M=self.Mcrys2cart
        coord1=self.atomsinunitcelldict[at1]['xyz']
        coord2=self.atomsinunitcelldict[at2]['xyz']
        datadict=dict()
        datadict['bonds']=[]
        datadict['radius']=0.02
        datadict['color']=(0,0,0)
        for k in range(0,len(coord1)):
            xyz1=np.dot(M,np.array(coord1[k]))
            for l in range(0,len(coord2)):
                if at1==at2 and l<=k:
                    pass
                else:
                    xyz2=np.dot(M,np.array(coord2[l]))
                    v=xyz1-xyz2
                    d=np.sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2])
                    if d >= dmin and d <= dmax:
                        tmp=dict()
                        tmp['length']=d
                        tmp['from']=xyz1
                        tmp['to']=xyz2
                        datadict['bonds'].append(tmp)
        label=at1+'-'+at2
        datadict['bondlabel']=label
        return datadict

    def get_selected_row(self,event):
        """
        return line number when selected 
        """
        self.index_row = event.m_itemIndex
        return

    def DeleteBondFromList(self, event):
        """
        delete selected row when button 'del' is clicked
        """
        try:
            del self.chemicalbondsdict[self.list_ctrl.GetItemData(self.index_row)]
            self.list_ctrl.DeleteItem(self.index_row)
            self.index_row=-1
            if wx.VERSION <=  ('2.8.11.0'):
                Publisher().sendMessage(("atom.bonds"), self.chemicalbondsdict)
            else:
                Publisher.sendMessage(("atom.bonds"), self.chemicalbondsdict)
        except KeyError:
            pass
            #dlg = wx.MessageDialog(self, "Please select row first", "error", wx.OK | wx.ICON_ERROR)
            #dlg.ShowModal()
            #dlg.Destroy()
        return

    def GetAtomicCoordinates(self, msg):
        """
        get atomic coordinates and set labels into combobox
        """
        data=msg.data
        self.Mcrys2cart=data['crys2cart'] #transformation matrix to cartesian coordinates in AA
        self.atomsinunitcelldict=data['atoms']
        self.cb1.Clear()
        self.cb2.Clear()
        for o in self.atomsinunitcelldict.iterkeys():
            self.cb1.Append(o)
            self.cb2.Append(o)
        return

    def OnADDClicked(self, event):
        """
        add bond when button add is clicked
        """
        at1=self.cb1.GetStringSelection()
        at2=self.cb2.GetStringSelection()
        dmin=self.dmin.GetValue()
        dmax=self.dmax.GetValue()
        bonddict=self.CalcBonds(at1,at2,float(dmin),float(dmax))
        rgb=(0,0,0)
        #table
        idx = self.list_ctrl.InsertStringItem(sys.maxint,at1)  
        self.list_ctrl.SetStringItem(idx, 1, at2)
        self.list_ctrl.SetStringItem(idx, 2, dmin)
        self.list_ctrl.SetStringItem(idx, 3, dmax)

        bondid = id(idx)  #associate a unique index with the row
        #bond color
        self.color_picker[bondid] = wx.Button(self.list_ctrl, wx.ID_ANY)
        self.color_picker[bondid].SetBackgroundColour(rgb)
        self.color_index[self.color_picker[bondid].GetId()] = bondid
        self.list_ctrl.SetItemWindow(idx, col=4, wnd=self.color_picker[bondid], expand = True)
        self.color_picker[bondid].Bind(wx.EVT_BUTTON, self.GetBondColour)

        #bond radius
        self.bond_radius = wx.Slider(self.list_ctrl,wx.ID_ANY,50, 0, 100, style=wx.SL_HORIZONTAL | wx.SL_AUTOTICKS | wx.SL_LABELS)
        self.bondradius_index[self.bond_radius.GetId()] = bondid
        self.list_ctrl.SetItemWindow(idx, 5, self.bond_radius, expand = True) 
        self.bond_radius.Bind(wx.EVT_SCROLL, self.GetBondRadius)

        self.list_ctrl.SetItemData(idx,bondid)  #need additional key to delete bond (not only from table but also in the dictionary)

        self.chemicalbondsdict[bondid]=bonddict
        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().sendMessage(("atom.bonds"), self.chemicalbondsdict)
        else:
            Publisher.sendMessage(("atom.bonds"), self.chemicalbondsdict)
        return

    def GetBondColour(self, event):
        """
        choose bond color and save it
        """
        #open color dialog and pick a color
        colorpicker = wx.ColourDialog(None)
        colorpicker.GetColourData().SetChooseFull(True)
        colorpicker.ShowModal()
        color = colorpicker.GetColourData()
        rgb = color.GetColour().Get()
        colorpicker.Destroy()

        cb = event.GetId()
        bondid = self.color_index[cb]
        #set the color in the bond dictionary
        self.chemicalbondsdict[bondid]['color']=rgb
        #change color of color dialog widget
        self.color_picker[bondid].SetBackgroundColour(rgb)
        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().sendMessage(("atom.bondsnew_color"), self.chemicalbondsdict)
        else:
            Publisher.sendMessage(("atom.bondsnew_color"), self.chemicalbondsdict)
        return

    def GetBondRadius(self,event):
        """
        Read value from bond radius slider 
        """
        cb = event.GetId()
        bondid = self.bondradius_index[cb]

        obj = event.GetEventObject()
        val = obj.GetValue()
        #set the radius in the bond dictionary
        self.chemicalbondsdict[bondid]['radius']=val/1000.
        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().sendMessage(("atom.bondsnew_radius"), self.chemicalbondsdict)
        else:
            Publisher.sendMessage(("atom.bondsnew_radius"), self.chemicalbondsdict)
        return


