
__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"


import collections
import numpy as np
import os
import sys
import wx
import wx.grid as gridlib
import wx.lib.agw.ultimatelistctrl as ulc


if wx.VERSION < ('2.8.11.0'):
    from wx.lib.pubsub import Publisher
else:
    from wx.lib.pubsub import setuparg1
    from wx.lib.pubsub import pub as Publisher

try:
    from AtomDefaultRadius import AtomDefaultRadius
except ImportError:
    print sys.path[-1]
    sys.exit('Module AtomDefaultRadius was not found in system path')
try:
    from AtomDefaultColor import AtomDefaultColor
except ImportError:
    print sys.path[-1]
    sys.exit('Module AtomicDefaultColor was not found in system path')

try:
    from vpydraw_atomicbonds import AtomicBonds
except ImportError:
    print sys.path[-1]
    sys.exit('Module AtomicBonds was not found in system path')

#sys.path.append(os.path.join(os.getcwd(),'..'))
try:
    from symmetry.Hall import HallSymbols
except ImportError:
    print sys.path[-1]
    sys.exit('Module Hall was not found in system path')

from vpydraw_globals import  GlobalParameters

class SpaceGroupPage(wx.Panel,HallSymbols):
    """
    extract symmetry elements from space group symbol
    """
    def __init__(self, parent):
        HallSymbols.__init__(self)
        wx.Panel.__init__(self, parent)
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        vbox = wx.BoxSizer(wx.VERTICAL)
        self.LeftGrid = gridlib.Grid(self)
        self.LeftGrid.SetToolTip(wx.ToolTip("click on a row to select a space group"))
        self.LeftGrid.EnableEditing(False)
        self.LeftGrid.EnableDragCell(False)
        self.LeftGrid.EnableDragRowSize(False)
        self.LeftGrid.CreateGrid(len(self.spacegroups), 2)
        self.LeftGrid.SetColLabelValue(0, "Hall")
        self.LeftGrid.SetColLabelValue(1, "H.-M.")
        k=0
        for key in self.spacegroups.keys():
            self.LeftGrid.SetRowLabelValue(k,key)
            self.LeftGrid.SetCellValue(k,0, self.spacegroups[key]['Hall'])
            self.LeftGrid.SetCellValue(k,1, self.spacegroups[key]['HM'])
            k=k+1
        self.LeftGrid.AutoSizeColumns()
        self.LeftGrid.AutoSizeRows()
        self.LeftGrid.Bind(gridlib.EVT_GRID_SELECT_CELL, self.OnSelectSpaceGroup)
        self.LeftGrid.Bind(gridlib.EVT_GRID_LABEL_LEFT_CLICK, self.OnSelectSpaceGroup)
        self.grid_clicked_row = None
        vbox.Add(self.LeftGrid, 1, wx.TOP | wx.LEFT | wx.EXPAND)
        hbox.Add(vbox, 1, wx.EXPAND | wx.ALL, 5)  # proportion 1

        vbox2 = wx.BoxSizer(wx.VERTICAL)
        self.Grid2 = gridlib.Grid(self)
        self.Grid2.EnableEditing(False)
        self.Grid2.EnableDragCell(False)
        self.Grid2.EnableDragRowSize(False)
        self.Grid2.CreateGrid(10, 2)
        self.Grid2.SetColLabelValue(0, "Seitz")
        self.Grid2.SetColLabelValue(1, "Jones")

        vbox2.Add(self.Grid2, 1, wx.TOP | wx.LEFT | wx.EXPAND)
        hbox.Add(vbox2, 1, wx.EXPAND | wx.ALL, 5)   # proportion 5
        self.SetSizer(hbox)
        return

    def SetCellBackgroundColor(self, row):
        """
        set color of table and modify the color of a row after it has been clicked
        """
        if self.grid_clicked_row == None:
            pass
        else:
            #first reset colour to white
            self.LeftGrid.SetCellBackgroundColour(self.grid_clicked_row,0,(255,255,255))
            self.LeftGrid.SetCellBackgroundColour(self.grid_clicked_row,1,(255,255,255))
            self.LeftGrid.SetCellBackgroundColour(self.grid_clicked_row,2,(255,255,255))
            self.LeftGrid.ForceRefresh()
        self.LeftGrid.SetCellBackgroundColour(row,0,(0,255,255))
        self.LeftGrid.SetCellBackgroundColour(row,1,(0,255,255))
        self.LeftGrid.SetCellBackgroundColour(row,2,(0,255,255))
        self.grid_clicked_row = row
        self.LeftGrid.ForceRefresh()
        return

    def OnSelectSpaceGroup(self,event):
        """
        get space group symbol from table
        """ 
        #try:
        self.SetCellBackgroundColor(event.GetRow())     
        label=self.spacegroups.items()[event.GetRow()][1]['HM']
        self.SpaceGroup(label)
        #except wx._core.PyDeadObjectError: # Workaround for exit button
        #    return 
        #    pass 
   
        self.SpaceGroupOperations(True)
        k=0
        self.Grid2.DeleteRows(pos=0, numRows=self.Grid2.GetNumberRows())
        for mat in self.matrixspacegroupoperations:
            rotpart,seitzsymbol=self._SeitzSymbol(mat,self.crystalsystem[0],self.crystalsystem[1])
            self.Grid2.AppendRows(1)
            self.Grid2.SetCellValue(k,0, seitzsymbol)
            self.Grid2.SetCellValue(k,1, self.JonesSymbol(mat))
            k=k+1
        self.Grid2.AutoSizeColumns()
        self.Grid2.AutoSizeRows()

        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().sendMessage(("spacegroup.HM"), label)
        else:
            Publisher.sendMessage(("spacegroup.HM"), label)
        return
        
class UnitCellPage(wx.Panel):
    """
    set lattice parameters and angles
    add an option to change cell color, rotation and scale 
    """
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        gp=GlobalParameters()
        self.latpar=[0,0,0,0,0,0]
        
        mainbox_sizer=wx.BoxSizer(wx.VERTICAL)
        #lattice parameters
        staticbox1 = wx.StaticBox(self,label="Lattice parameters")
        vbox1 = wx.StaticBoxSizer(staticbox1,wx.VERTICAL)
        grid_sizer = wx.FlexGridSizer(2, 9, 10, 20) 
        a_lat = wx.StaticText(self, label="a ("+ u'\N{LATIN CAPITAL LETTER A WITH RING ABOVE}'+"):")
        b_lat = wx.StaticText(self,label="b ("+ u'\N{LATIN CAPITAL LETTER A WITH RING ABOVE}'+"):") 
        c_lat = wx.StaticText(self,label="c ("u'\N{LATIN CAPITAL LETTER A WITH RING ABOVE}'+"):") 
        aa_lat = wx.StaticText(self,label=u'\N{GREEK SMALL LETTER ALPHA}'+" (deg.):")
        bb_lat = wx.StaticText(self,label=u'\N{GREEK SMALL LETTER BETA}'+" (deg.):") 
        cc_lat = wx.StaticText(self,label=u'\N{GREEK SMALL LETTER GAMMA}'+" (deg.):")

        tc1 = wx.TextCtrl(self,id = 1, style = wx.TE_PROCESS_ENTER)
        tc1.SetToolTip(wx.ToolTip("enter lattice parameter a ("+ u'\N{LATIN CAPITAL LETTER A WITH RING ABOVE}'+")"))
 
        self.Bind(wx.EVT_TEXT, self.GetLatticeParameters, tc1)

        tc2 = wx.TextCtrl(self,id = 2, style = wx.TE_PROCESS_ENTER)
        tc2.SetToolTip(wx.ToolTip("enter lattice parameter b ("+ u'\N{LATIN CAPITAL LETTER A WITH RING ABOVE}'+")"))
        self.Bind(wx.EVT_TEXT, self.GetLatticeParameters, tc2)

        tc3 = wx.TextCtrl(self,id = 3, style = wx.TE_PROCESS_ENTER)
        tc3.SetToolTip(wx.ToolTip("enter lattice parameter c ("+ u'\N{LATIN CAPITAL LETTER A WITH RING ABOVE}'+")"))
        self.Bind(wx.EVT_TEXT, self.GetLatticeParameters, tc3)

        tc4 = wx.TextCtrl(self,id = 4, style = wx.TE_PROCESS_ENTER)
        tc4.SetToolTip(wx.ToolTip("enter angle "+u'\N{GREEK SMALL LETTER ALPHA}'+" (deg.)"))
        self.Bind(wx.EVT_TEXT, self.GetLatticeParameters, tc4)

        tc5 = wx.TextCtrl(self,id = 5, style = wx.TE_PROCESS_ENTER)
        tc5.SetToolTip(wx.ToolTip("enter angle "+u'\N{GREEK SMALL LETTER BETA}'+" (deg.)"))
        self.Bind(wx.EVT_TEXT, self.GetLatticeParameters, tc5)

        tc6 = wx.TextCtrl(self,id = 6, style = wx.TE_PROCESS_ENTER)
        tc6.SetToolTip(wx.ToolTip("enter angle "+u'\N{GREEK SMALL LETTER GAMMA}'+" (deg.)"))
        self.Bind(wx.EVT_TEXT, self.GetLatticeParameters, tc6)

        grid_sizer.AddMany([(a_lat,1,wx.ALIGN_CENTER),  (tc1, 1, wx.EXPAND), (wx.StaticText(self,label="    ")),
                            (b_lat,1,wx.ALIGN_CENTER),  (tc2, 1, wx.EXPAND), (wx.StaticText(self,label="    ")),
                            (c_lat,1,wx.ALIGN_CENTER),  (tc3, 1, wx.EXPAND), (wx.StaticText(self,label="    ")),
                            (aa_lat,1,wx.ALIGN_CENTER), (tc4, 1, wx.EXPAND), (wx.StaticText(self,label="    ")),
                            (bb_lat,1,wx.ALIGN_CENTER), (tc5, 1, wx.EXPAND), (wx.StaticText(self,label="    ")),
                            (cc_lat,1,wx.ALIGN_CENTER), (tc6, 1, wx.EXPAND)])
        vbox1.Add(grid_sizer,1,wx.EXPAND | wx.ALL, 20)

        border = wx.BoxSizer()
        OK_button = wx.Button(self, label='OK')
        OK_button.Bind(wx.EVT_BUTTON, self.OnOKClicked)
        border.Add(OK_button, 0, wx.ALL | wx.EXPAND, 10)
        vbox1.Add(border,0)

        mainbox_sizer.Add(vbox1,1,wx.EXPAND | wx.ALL)

        #plot options

        self.plotpar = dict() #scale and rotations values
        self.plotpar['scale'] = gp.defaultvalueswidgets['scale']
        self.plotpar['rotx'] = gp.defaultvalueswidgets['rotx']
        self.plotpar['roty'] = gp.defaultvalueswidgets['roty']
        self.plotpar['rotz'] = gp.defaultvalueswidgets['rotz']

        self.plotpar['scale_old']=str(gp.defaultvalueswidgets['scale'])
        self.plotpar['rotx_old']=gp.defaultvalueswidgets['rotx']
        self.plotpar['roty_old']=gp.defaultvalueswidgets['roty']
        self.plotpar['rotz_old']=gp.defaultvalueswidgets['rotz']

        staticbox2 = wx.StaticBox(self,label="Plot options")
        vbox2 = wx.StaticBoxSizer(staticbox2,wx.VERTICAL)
        #scale
        scale_ctrltext = wx.StaticText(self, label="Scale (x 100)")        
        scale_slider = wx.Slider(self, id = 1, value=self.plotpar['scale'], minValue=10., maxValue=200., size=(250, -1),
                                 style= wx.SL_HORIZONTAL | wx.SL_AUTOTICKS | wx.SL_LABELS,name='scale')
        scale_slider.Bind(wx.EVT_SCROLL, self.OnPlotOptionsScroll)

        #scale_slider.SetTickFreq(50, 10) #Windows only...
        #rotations
        rotx_ctrltext = wx.StaticText(self, label="Rotation X")        
        rotx_slider = wx.Slider(self, id = 2, value=self.plotpar['rotx'], minValue=0, maxValue=180, size=(250, -1),
                                 style= wx.SL_HORIZONTAL | wx.SL_AUTOTICKS | wx.SL_LABELS,name='scale')
        rotx_slider.Bind(wx.EVT_SCROLL, self.OnPlotOptionsScroll)

        roty_ctrltext = wx.StaticText(self, label="Rotation Y")        
        roty_slider = wx.Slider(self, id = 3, value=self.plotpar['roty'], minValue=0, maxValue=180, size=(250, -1),
                                 style= wx.SL_HORIZONTAL | wx.SL_AUTOTICKS | wx.SL_LABELS,name='scale')
        roty_slider.Bind(wx.EVT_SCROLL, self.OnPlotOptionsScroll)

        rotz_ctrltext = wx.StaticText(self, id = 4, label="Rotation Z")        
        rotz_slider = wx.Slider(self, id = 4, value=self.plotpar['rotz'], minValue=0, maxValue=180, size=(250, -1),
                                 style= wx.SL_HORIZONTAL | wx.SL_AUTOTICKS | wx.SL_LABELS,name='scale')
        rotz_slider.Bind(wx.EVT_SCROLL, self.OnPlotOptionsScroll)

        options_sizer = wx.FlexGridSizer(3, 4, 10, 20)
        options_sizer.AddMany([
                               (rotx_ctrltext, 1, wx.ALIGN_BOTTOM),  (rotx_slider, 1, wx.EXPAND),
                               (roty_ctrltext, 1, wx.ALIGN_BOTTOM),  (roty_slider, 1, wx.EXPAND),
                               (rotz_ctrltext, 1, wx.ALIGN_BOTTOM),  (rotz_slider, 1, wx.EXPAND),
                               (scale_ctrltext,1, wx.ALIGN_BOTTOM),  (scale_slider, 1, wx.EXPAND)
                              ])
        vbox2.Add(options_sizer,1,wx.EXPAND | wx.ALL, 20)
        self.Color_button = wx.ColourPickerCtrl(self)
        self.Color_button.SetToolTip(wx.ToolTip("click to set cell color"))
        self.Color_button.Bind(wx.EVT_COLOURPICKER_CHANGED, self.OnColorPickerClicked)          
        options_sizer.Add(self.Color_button,1)
        mainbox_sizer.Add(vbox2,0,wx.EXPAND | wx.ALL)
        self.SetSizer(mainbox_sizer)
        return

    def GetLatticeParameters(self,event):
        """
        Read lattice parameters from widget and call drawing function
        """
        id = event.GetId()-1
        value = event.GetString()
        if len(value) != 0:
           try:
              self.latpar[id]=float(value)
           except ValueError:
              dlg = wx.MessageDialog(None, "Wrong Input", "Error", wx.OK | wx.ICON_ERROR)
              dlg.ShowModal()
              dlg.Destroy()
        return

    def OnColorPickerClicked(self,event):
        """
        Retreive selected colour (r,g,b,alpha)
        """
        colour = self.Color_button.GetColour()
        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().sendMessage(("cell.colour"), colour)
        else:
            Publisher.sendMessage(("cell.colour"), colour)
        return

    def OnOKClicked(self,event):
        if 0 in self.latpar:
            pass
        else:
            if wx.VERSION <=  ('2.8.11.0'):
                Publisher().sendMessage(("cell.parameters"), self.latpar)
            else:
                Publisher.sendMessage(("cell.parameters"), self.latpar)
        return

    def OnPlotOptionsScroll(self,event):
        """
        values for scale, rotations around x,y,z (world coordinates)
        """
        obj = event.GetEventObject()
        val = obj.GetValue()
        id = event.GetId()
        self.plotpar['scale_old']=self.plotpar['scale']
        self.plotpar['rotx_old']=self.plotpar['rotx']
        self.plotpar['roty_old']=self.plotpar['roty']
        self.plotpar['rotz_old']=self.plotpar['rotz']
        if id == 1:
            self.plotpar['scale'] = val
        elif id == 2:
            self.plotpar['rotx'] = val
        elif id == 3:
            self.plotpar['roty'] = val
        elif id ==4:
            self.plotpar['rotz'] = val
        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().sendMessage(("cell.scale.rotations"), self.plotpar)
        else:
            Publisher.sendMessage(("cell.scale.rotations"), self.plotpar)
        return

from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin
class AutoWidthListCtrl(ulc.UltimateListCtrl, ListCtrlAutoWidthMixin):
    def __init__(self,parent):
        ulc.UltimateListCtrl.__init__(self, parent,id=wx.ID_ANY, agwStyle=wx.LC_REPORT|ulc.ULC_HAS_VARIABLE_ROW_HEIGHT)
        ListCtrlAutoWidthMixin.__init__(self)

class AtomicPositionsPage(wx.Panel, AtomDefaultColor, AtomDefaultRadius):
    """
    enter atomic parameters x,y,z coordinates
    option to change sphere color and radius
    """
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        AtomDefaultColor.__init__(self)
        AtomDefaultRadius.__init__(self)
        self.atoms = collections.OrderedDict()
        self.atom_name = None
        self.atom_label = None
        self.x_new_coord = None
        self.y_new_coord = None
        self.z_new_coord = None

        self.index_row = -1
        self.color_picker = dict()
        self.color_index = dict()
        self.radius_index = dict()
        self.magchoicebox_index = dict()
        self.ffion=[  'Am2', 'Am3', 'Am4', 'Am5', 'Am6', 'Am7', 
                      'Ce2', 'Co0', 'Co1', 'Co2', 'Co3', 'Co4', 'Cr0', 'Cr1', 'Cr2', 'Cr3', 'Cr4', 'Cu0', 'Cu1', 'Cu2', 'Cu3', 'Cu4',
                      'Dy2', 'Dy3', 
                      'Er2', 'Er3', 'Eu2', 'Eu3', 
                      'Fe0', 'Fe1', 'Fe2', 'Fe3', 'Fe4',
                      'Gd2', 'Gd3',
                      'Ho2', 'Ho3',
                      'Mn0', 'Mn1', 'Mn2', 'Mn3', 'Mn4', 'Mn5', 'Mo0', 'Mo1',
                      'Nb0', 'Nb1', 'Nd2', 'Nd3', 'Ni0', 'Ni1', 'Ni2', 'Ni3', 'Ni4', 'Np3', 'Np4', 'Np5', 'Np6',
                      'Pd0', 'Pd1', 'Pr3', 'Pu3', 'Pu4', 'Pu5', 'Pu6',
                      'Rh0', 'Rh1', 'Ru0', 'Ru1',
                      'Sc0', 'Sc1', 'Sc2', 'Sm2', 'Sm3',
                      'Tb2', 'Tb3', 'Tc0', 'Tc1', 'Ti0', 'Ti1', 'Ti2', 'Ti3', 'Tm2', 'Tm3',
                      'U3', 'U4', 'U5',
                      'V0', 'V1', 'V2', 'V3', 'V4',
                      'Y0', 'Yb2', 'Yb3', 'Zr0', 'Zr1']  #magnetic form factor

        sb = wx.StaticBox(self,label="Coordinates")
        vboxsizer = wx.StaticBoxSizer(sb,wx.VERTICAL)
        #control list
        self.list_ctrl = AutoWidthListCtrl(self)
        self.list_ctrl.InsertColumn(0, 'Atom',wx.LIST_FORMAT_CENTER,width=100)
        self.list_ctrl.InsertColumn(1, 'Label',wx.LIST_FORMAT_CENTER,width=100)
        self.list_ctrl.InsertColumn(2, 'x',wx.LIST_FORMAT_CENTER,width=100)
        self.list_ctrl.InsertColumn(3, 'y',wx.LIST_FORMAT_CENTER,width=100)
        self.list_ctrl.InsertColumn(4, 'z',wx.LIST_FORMAT_CENTER,width=100)
        self.list_ctrl.InsertColumn(5, 'color',wx.LIST_FORMAT_CENTER,width=100)
        self.list_ctrl.InsertColumn(6, 'radius (x100)',wx.LIST_FORMAT_CENTER,width=200)
        self.list_ctrl.InsertColumn(7, 'magnetic (yes/no)',wx.LIST_FORMAT_CENTER,width=100)
        self.list_ctrl.SetColumnToolTip(7, "select if ion is magnetic")
        self.list_ctrl.Bind(wx.EVT_LIST_ITEM_SELECTED,self.get_selected_row)
        vboxsizer.Add(self.list_ctrl, 2, wx.EXPAND)
        
        #x,y,z coordinates to create new atom
        hboxsizer = wx.BoxSizer(wx.HORIZONTAL)
        atom_name =  wx.StaticText(self, label="Atom:")
        atom_name_input = wx.TextCtrl(self,id=0)
        atom_name_input.SetToolTip(wx.ToolTip("enter atomic name"))
        atom_name_input.Bind(wx.EVT_TEXT, self.GetText)

        atom_label =  wx.StaticText(self, label="Label:")
        atom_label_input = wx.TextCtrl(self,id=1)
        atom_label_input.SetToolTip(wx.ToolTip("enter atomic label"))
        atom_label_input.Bind(wx.EVT_TEXT, self.GetText)

        x_par = wx.StaticText(self, label="x :")
        x_input = wx.TextCtrl(self,id=2)
        x_input.SetToolTip(wx.ToolTip("enter x-coordinates (rlu)"))
        x_input.Bind(wx.EVT_TEXT, self.GetText)

        y_par = wx.StaticText(self, label="y :")
        y_input = wx.TextCtrl(self,id=3)
        y_input.SetToolTip(wx.ToolTip("enter y-coordinates (rlu)"))
        y_input.Bind(wx.EVT_TEXT, self.GetText)

        z_par = wx.StaticText(self, label="z :")
        z_input = wx.TextCtrl(self,id=4)
        z_input.SetToolTip(wx.ToolTip("enter z-coordinates (rlu)"))
        z_input.Bind(wx.EVT_TEXT, self.GetText)

        sizer0 = wx.FlexGridSizer(1, 2)
        sizer0.AddMany([(atom_name,wx.ALIGN_CENTER), (atom_name_input, 1, wx.EXPAND, 5)])
        sizer1 = wx.FlexGridSizer(1, 2)
        sizer1.AddMany([(atom_label,wx.ALIGN_CENTER), (atom_label_input, 1, wx.EXPAND, 5)])
        sizer2 = wx.FlexGridSizer(1, 2)
        sizer2.AddMany([(x_par,wx.ALIGN_CENTER), (x_input, 1, wx.EXPAND, 5)])
        sizer3 = wx.FlexGridSizer(1, 2)
        sizer3.AddMany([(y_par,wx.ALIGN_CENTER), (y_input, 1, wx.EXPAND, 5)])
        sizer4 = wx.FlexGridSizer(1, 2)
        sizer4.AddMany([(z_par,wx.ALIGN_CENTER), (z_input, 1, wx.EXPAND, 5)])

        hboxsizer.Add(sizer0, 0, wx.EXPAND | wx.ALL, 10)
        hboxsizer.Add(sizer1, 0, wx.EXPAND | wx.ALL, 10)
        hboxsizer.Add(sizer2, 0, wx.EXPAND | wx.ALL, 10)
        hboxsizer.Add(sizer3, 0, wx.EXPAND | wx.ALL, 10)
        hboxsizer.Add(sizer4, 0, wx.EXPAND | wx.ALL, 10)
        vboxsizer.Add(hboxsizer,0,wx.EXPAND)

        panel2 = wx.Panel(self, -1)        
        buttonsizer = wx.BoxSizer(wx.HORIZONTAL)
        #'new' button that inserts x,y,z coordinates into contol list
        New_Button = wx.Button(panel2,label="New")
        New_Button.SetToolTip(wx.ToolTip("click to add an atom to the list"))
        self.Bind(wx.EVT_BUTTON, self.AddAtomToList, New_Button)
        buttonsizer.Add(New_Button,flag=wx.ALIGN_RIGHT | wx.ALL, border=5)
        Delete_Button = wx.Button(panel2,label="Delete")
        Delete_Button.Bind(wx.EVT_BUTTON, self.DeleteAtomFromList)
        Delete_Button.SetToolTip(wx.ToolTip("click to delete an atom from the list"))
        buttonsizer.Add(Delete_Button, flag=wx.ALIGN_RIGHT | wx.ALL, border=5)
        Clear_Button = wx.Button(panel2,label="Clear")
        Clear_Button.SetToolTip(wx.ToolTip("clear the list"))
        Clear_Button.Bind(wx.EVT_BUTTON, self.ClearList)
        buttonsizer.Add(Clear_Button, flag=wx.ALIGN_RIGHT | wx.ALL, border=5)
        panel2.SetSizer(buttonsizer)
        vboxsizer.Add(panel2, 1)
        self.SetSizer(vboxsizer)
        self.maplistctrl=dict()
        return

    def _check_input_N(self,a):
           if a != None:
               return True
           else:
               return False

    def _check_input_F(self,a):
        try:
           float(a)
        except ValueError: 
           return False
        else: return True

    def AddAtomToList(self, event):
        """
        Add an atom to list when button 'new' is clicked
        and save paramters
        """
        if self._check_input_N(self.atom_name) == False or self._check_input_N(self.atom_label) == False  or \
           self._check_input_F(self.x_new_coord) == False or \
           self._check_input_F(self.y_new_coord) == False or \
           self._check_input_F(self.z_new_coord) == False:
                dlg = wx.MessageDialog(self, "Check input\nCannot add atom to table!", "error", wx.OK | wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
        elif self.atom_label in self.atoms:
            dlg = wx.MessageDialog(self, "Atom label already exists!", "error", wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
        else:
            try:
                rgb=tuple(np.array(self.atomcolordefault[self.atom_name]))
            except KeyError:
                rgb=(0,0,0)
            try:
                radius=float(self.atomradiusdefault[self.atom_name])/2.0
            except KeyError:
                radius = 0.05
            idx = self.list_ctrl.InsertStringItem(sys.maxint,"")
            self.list_ctrl.SetStringItem(idx, 0, self.atom_name)
            self.list_ctrl.SetStringItem(idx, 1, self.atom_label)
            self.list_ctrl.SetStringItem(idx, 2, self.x_new_coord)
            self.list_ctrl.SetStringItem(idx, 3, self.y_new_coord)
            self.list_ctrl.SetStringItem(idx, 4, self.z_new_coord)

            bondid = id(idx)
            #color of atom
            self.color_picker[bondid] = wx.Button(self.list_ctrl, wx.ID_ANY)
            self.color_picker[bondid].SetBackgroundColour(rgb)
            self.color_index[self.color_picker[bondid].GetId()] = bondid
            self.list_ctrl.SetItemWindow(idx, 5, self.color_picker[bondid], expand = True)
            self.color_picker[bondid].Bind(wx.EVT_BUTTON, self.GetAtomColour)

            #atomic radius
            self.atom_radius = wx.Slider(self.list_ctrl,wx.ID_ANY, radius*100, 0, 300, style=wx.SL_HORIZONTAL | wx.SL_AUTOTICKS | wx.SL_LABELS)
            self.radius_index[self.atom_radius.GetId()] = bondid
            self.atom_radius.Bind(wx.EVT_SCROLL, self.GetAtomicRadius)
            self.list_ctrl.SetItemWindow(idx, 6, self.atom_radius, expand = True) 

            #checkbox for magnetic ion
            self.magchoicebox=wx.CheckBox(self.list_ctrl,wx.ID_ANY, '(yes/no)')
            self.magchoicebox.SetValue(False)
            self.magchoicebox_index[self.magchoicebox.GetId()] = bondid
            self.magchoicebox.Bind(wx.EVT_CHECKBOX, self.MagneticAtomOption)
            self.list_ctrl.SetItemWindow(idx, 7, self.magchoicebox, expand = True) 
   
            self.list_ctrl.SetItemData(idx,bondid)  #need additional key to delete bond (not only from table but also in the dictionary)
            self.maplistctrl[bondid]=idx

            #save info in a dictionary
            tmp=dict()
            tmp['atom']=self.atom_name
            tmp['x']=self.x_new_coord
            tmp['y']=self.y_new_coord
            tmp['z']=self.z_new_coord
            tmp['color']=rgb
            tmp['radius']=radius
            tmp['radius_default']=radius
            tmp['magnetic']=False
            tmp['FQ']=''
            self.atoms[self.atom_label] = tmp 
            if wx.VERSION <=  ('2.8.11.0'):
                Publisher().sendMessage(("atom.name_coordinates"), self.atoms)
            else:
                Publisher.sendMessage(("atom.name_coordinates"), self.atoms)
            return

    def ClearList(self, event):
        """
        delete all atoms from table
        """
        rows=self.list_ctrl.GetItemCount()
        for k in range(rows-1,-1,-1):
            self.list_ctrl.DeleteItem(k)
        self.atoms.clear()
        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().sendMessage(("atom.delete_all_atoms"), self.atoms)
        else:
            Publisher.sendMessage(("atom.delete_all_atoms"), self.atoms)
        return

    def DeleteAtomFromList(self, event):
        """
        delete selected row when button 'del' is clicked
        """
        try:
            self.list_ctrl.DeleteItem(self.index_row)
            key = self.atoms.keys()[self.index_row]
            del self.atoms[key]
            if wx.VERSION <=  ('2.8.11.0'):
                Publisher().sendMessage(("atom.delete"), self.atoms)
            else:
                Publisher.sendMessage(("atom.delete"), self.atoms)
        except:
            dlg = wx.MessageDialog(self, "Please select row first", "error", wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            self.index_row = -1
        return

    def get_selected_row(self,event):
        """
        return line number when selected with right-button
        """
        self.index_row = event.m_itemIndex
        return

    def GetAtomColour(self, event):
        """
        Get color from color picker
        """
        #pick a color
        colorpicker = wx.ColourDialog(None)
        colorpicker.GetColourData().SetChooseFull(True)
        colorpicker.ShowModal()
        color = colorpicker.GetColourData()
        rgb = color.GetColour().Get()
        colorpicker.Destroy()

        #get idx of clicked row 
        #cr = event.GetEventObject()
        #idx = self.color_index[cr.GetId()]
        #equivalent
        cb = event.GetId()
        bondid = self.color_index[cb]
        idx=self.maplistctrl[bondid]
        item = self.list_ctrl.GetItem(idx,1)
        atom_label = item.GetText()
        self.atoms[atom_label]['color'] = rgb

        self.color_picker[bondid].SetBackgroundColour(rgb)
        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().sendMessage(("atom.color"), self.atoms)
        else:
            Publisher.sendMessage(("atom.color"), self.atoms)
        return

    def GetText(self, event):
        """
        read coordinates of new atoms to insert into table
        """
        if event.GetId() == 0:
            self.atom_name = event.GetString()
        if event.GetId() == 1:
            self.atom_label = event.GetString()
        if event.GetId() == 2:
            self.x_new_coord = event.GetString()
        if event.GetId() == 3:
            self.y_new_coord = event.GetString()
        if event.GetId() == 4:
            self.z_new_coord = event.GetString()
        return

    def GetAtomicRadius(self,event):
        """
        Read value from atom radius slider 
        """
        cb = event.GetId()
        bondid = self.radius_index[cb]
        idx=self.maplistctrl[bondid]
        item = self.list_ctrl.GetItem(idx,1)
        atom_label = item.GetText()
        obj = event.GetEventObject()
        val = obj.GetValue()
        self.atoms[atom_label]['radius'] = val/100.
        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().sendMessage(("atom.radius"), self.atoms)
        else:
            Publisher.sendMessage(("atom.radius"), self.atoms)
        return

    def MagneticAtomOption(self,event):
        """
        Read checkBox to choose between magnetic/non-magnetic atom
        """
        cb = event.GetId()
        bondid = self.magchoicebox_index[cb]
        idx=self.maplistctrl[bondid]
        item = self.list_ctrl.GetItem(idx,1)
        atom_label = item.GetText()
        obj = event.GetEventObject()
        val = obj.GetValue()
        self.atoms[atom_label]['magnetic']=val
        return

class StructureParametersBook(wx.Frame):
    """
    main panel to define space group and the chemical structure
    """
    def __init__(self, parent=None):
        wx.Frame.__init__(self, parent, title="Chemical Structure",size=(1000, 600))
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        # Here we create a panel and a notebook on the panel
        panel = wx.Panel(self)
        self.book = wx.Notebook(panel)

        # create the page windows as children of the notebook
        book=self.book
        page0 = SpaceGroupPage(book)
        page1 = UnitCellPage(book)
        page2 = AtomicPositionsPage(book)
        page3 = AtomicBonds(book)

        # add the pages to the notebook with the label to show on the tab
        book.AddPage(page0, "Space Group")
        book.AddPage(page1, "Unit Cell")
        book.AddPage(page2, "Atomic Positions")
        book.AddPage(page3, "Chemical Bonds")
 
        # finally, put the notebook in a sizer for the panel to manage
        # the layout
        sizer = wx.BoxSizer()
        sizer.Add(book, 1, wx.EXPAND)
        panel.SetSizer(sizer)
        self.Centre()
        self.Show()
        return

    def OnClose(self,event):
        """
        close panel
        """
        self.Destroy()
        return

#if __name__ == "__main__":
#    app = wx.App()
#    StructureParametersBook(None).Show()
#    app.MainLoop()


