#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"


import collections
import copy
import numpy as np
import wx

if wx.VERSION < ('2.8.11.0'):
    from wx.lib.pubsub import Publisher
else:
    from wx.lib.pubsub import setuparg1
    from wx.lib.pubsub import pub as Publisher

import wx.lib.mixins.listctrl  as  listmix
import os
import sys


sys.path.append(os.path.join(os.getcwd(),'..'))
try:
    from symmetry.BasisVectors import BasisVectors
except ImportError:
    print sys.path[-1]
    sys.exit('Module BasisVectors was not found in system path')
try:
    from symmetry.prettytable import *
except ImportError:
    sys.exit('Module Pretty Table could not be imported')

class EditableListCtrl(wx.ListCtrl, listmix.TextEditMixin):
    def __init__(self, parent, ID=wx.ID_ANY, pos=wx.DefaultPosition,
                 size=wx.DefaultSize, style=0):
        wx.ListCtrl.__init__(self, parent, ID, pos, size, style)
        listmix.TextEditMixin.__init__(self)

class DrawMagneticModesPage(wx.Panel):
    def __init__(self, parent,mcrys2cart, mrec2cart):
        wx.Panel.__init__(self, parent)
        self.mcrys2cart=mcrys2cart                                   #transformation matrix to cartesian coordinates
        self.mrec2cart=mrec2cart
        mainbox_sizer=wx.BoxSizer(wx.VERTICAL)
        staticbox1  = wx.StaticBox(self,0,label="magnetic structure",style=wx.EXPAND)
        vbox1 = wx.StaticBoxSizer(staticbox1,wx.VERTICAL)
        grid_sizer = wx.FlexGridSizer(1, 6, 10, 20)
        #orbit
        cb_orbitlabel = wx.StaticText(self, label='orbit:')
        self.cb_orbits = wx.Choice(self, choices="",size=(200, -1))
        self.cb_orbits.Bind(wx.EVT_CHOICE, self.SelectOrbit)
        #irrep
        cb_irreplabel = wx.StaticText(self, label='Co/Irrep.:')
        self.cb_irreps = wx.Choice(self, choices="",size=(100, -1))
        self.cb_irreps.Bind(wx.EVT_CHOICE, self.SelectIrrep)
        #basis functions
        cb_modelabel = wx.StaticText(self, label='Mode:')
        self.cb_modes = wx.Choice(self, choices="",size=(100, -1))
        self.cb_modes.Bind(wx.EVT_CHOICE, self.SelectMagMode)

        grid_sizer.Add(cb_orbitlabel,0,wx.ALIGN_CENTER, 10)
        grid_sizer.Add(self.cb_orbits,0,wx.EXPAND | wx.ALL, 10)
        grid_sizer.Add(cb_irreplabel,0,wx.ALIGN_CENTER, 10)
        grid_sizer.Add(self.cb_irreps,0,wx.EXPAND | wx.ALL, 10)
        grid_sizer.Add(cb_modelabel,0,wx.ALIGN_CENTER, 10)
        grid_sizer.Add(self.cb_modes,0,wx.EXPAND | wx.ALL, 10)
        vbox1.Add(grid_sizer, flag=wx.EXPAND|wx.LEFT|wx.RIGHT, border=10)

        #basisfunction table
        self.basisfunctionstable=EditableListCtrl(self, wx.ID_ANY, wx.DefaultPosition,
                 size=(-1,150), style=wx.LC_REPORT)
        self.basisfunctionstable.InsertColumn(0, "V_{ij}", wx.LIST_FORMAT_CENTER, width=200)
        self.basisfunctionstable.InsertColumn(1, "Coefficient", wx.LIST_FORMAT_CENTER, width=200)
        self.basisfunctionstable.InsertColumn(2, "Orbit", wx.LIST_FORMAT_CENTER, width=200)
        self.basisfunctionstable.InsertColumn(3, "Irrep.", wx.LIST_FORMAT_CENTER, width=200)
        self.basisfunctionstable.Bind(wx.EVT_LIST_BEGIN_LABEL_EDIT, self.OnBasisfunctionEdit)
        self.basisfunctionstable.Bind(wx.EVT_LIST_END_LABEL_EDIT, self.OnBasisfunctionChangeText)
        vbox1.Add(self.basisfunctionstable, flag=wx.EXPAND|wx.LEFT|wx.RIGHT, border=10)
        CALC_button = wx.Button(self, label='Calc.')
        CALC_button.Bind(wx.EVT_BUTTON, self.OnCalcClicked)
        border1 = wx.BoxSizer()
        border1.Add(CALC_button, 0, wx.ALL | wx.EXPAND, 10)
        vbox1.Add(border1,0)
        vbox1.Add((-1, 10))
        #magnetic moment table
        self.magstrutable=wx.ListCtrl(self, wx.ID_ANY, wx.DefaultPosition,
                 size=(-1,250), style=wx.LC_REPORT)
        self.magstrutable.InsertColumn(0, "Label", wx.LIST_FORMAT_CENTER, width=150)
        self.magstrutable.InsertColumn(1, "Position", wx.LIST_FORMAT_CENTER, width=200)
        self.magstrutable.InsertColumn(2, "Mx", wx.LIST_FORMAT_CENTER, width=100)
        self.magstrutable.InsertColumn(3, "My", wx.LIST_FORMAT_CENTER, width=100)
        self.magstrutable.InsertColumn(4, "Mz", wx.LIST_FORMAT_CENTER, width=100)
        self.magstrutable.InsertColumn(5, "|M|", wx.LIST_FORMAT_CENTER, width=100)
        vbox1.Add(self.magstrutable, flag=wx.EXPAND|wx.LEFT|wx.RIGHT, border=10)
        vbox1.Add((-1, 10))

        #magnetic domains
        hbox1=wx.BoxSizer(wx.HORIZONTAL)    #ouptut of result calculations
        self.magdomains = wx.TextCtrl(self,wx.ID_ANY, size=(-1,150),style=wx.TE_MULTILINE|wx.TE_READONLY|wx.HSCROLL)
        old_font = self.magdomains.GetFont()
        new_font = wx.Font(old_font.GetPointSize(), wx.FONTFAMILY_MODERN, old_font.GetStyle(), old_font.GetWeight())
        self.magdomains.SetFont(new_font)
        hbox1.Add(self.magdomains,1,wx.EXPAND| wx.ALL)
        vbox1.Add(hbox1,1,wx.EXPAND| wx.ALL)

        DRAW_button = wx.Button(self, label='Draw')
        DRAW_button.Bind(wx.EVT_BUTTON, self.OnDRAWClicked)
        RESET_button = wx.Button(self, label='Reset')
        RESET_button.Bind(wx.EVT_BUTTON, self.OnResetClicked)
        SAVE_button = wx.Button(self, label='Save')
        SAVE_button.Bind(wx.EVT_BUTTON, self.OnSaveClicked)

        border2 = wx.BoxSizer()
        border2.Add(DRAW_button, 0, wx.ALL | wx.EXPAND, 10)
        border2.Add(RESET_button, 0, wx.ALL | wx.EXPAND, 10)
        border2.Add(SAVE_button, 0, wx.ALL | wx.EXPAND, 10)
        vbox1.Add(border2,0)
        mainbox_sizer.Add(vbox1,0,wx.EXPAND | wx.ALL)

        self.SetSizer(mainbox_sizer)
        mainbox_sizer.Fit(self)

        self.crystalorbits=None
        self.selected_crystalorbit=None
        self.decompirredcomp=None
        self.selected_irrep=None
        self.chosenbasisfunctions=dict() #contains orbit as key and list of selected basis functions as values
        self.atomsandmoments=dict() #contains calculated moment directions from modes
        self.littlegroup=dict()
        self.magneticrepresentation=dict()

        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().subscribe(self.GetOrbits, ("orbits"))
            Publisher().subscribe(self.GetIrrepsCoefs, ("irrepscoefs"))
            Publisher().subscribe(self.GetMagneticRepresentation, ("magneticrepresentation"))
            Publisher().subscribe(self.GetModes, ("modes"))
            Publisher().subscribe(self.GetKVector, ("kvector"))
            Publisher().subscribe(self.GetLittleGroup, ("littlegroup"))
            #lattice centering
            Publisher().subscribe(self.GetLatticeTranslations, ("latticetranslations"))
        else:
            Publisher.subscribe(self.GetOrbits, ("orbits"))
            Publisher.subscribe(self.GetIrrepsCoefs, ("irrepscoefs"))
            Publisher.subscribe(self.GetMagneticRepresentation, ("magneticrepresentation"))
            Publisher.subscribe(self.GetModes, ("modes"))
            Publisher.subscribe(self.GetKVector, ("kvector"))
            Publisher.subscribe(self.GetLittleGroup, ("littlegroup"))
            #lattice centering
            Publisher.subscribe(self.GetLatticeTranslations, ("latticetranslations"))
        return

    def GetKVector(self,msg):
        """
        Get propagation vector
        """
        self.kvector=msg.data
        return

    def GetLittleGroup(self,msg):
        """
        Symmetry elements in little group
        """
        self.littlegroup=msg.data #dictionary keys: 'seitzsymbol', 'matrix'
        return

    def GetMagneticRepresentation(self, msg):
        self.magneticrepresentation=msg.data
        return

    def GetModes(self,msg):
        """
        Get basis functions and rename keys so that they can be directly inserted in combobox
        """
        bvdict=msg.data
        self.basisfunctions=dict()
        for orbit, irrepdict in bvdict.iteritems():
            tmpGdict=dict()
            for G, modesdict in irrepdict.iteritems():
                if len(modesdict) == 0:
                    pass
                else:
                    tmpVdict=dict()
                    k=1
                    for Vdict, basislist in modesdict.iteritems():
                        for function in basislist:
                            #new key
                            key='V('+G[1:]+str(k)+')'
                            tmpVdict[key]=dict()
                            tmpVdict[key]['orderparameter']=function
                            tmpVdict[key]['coefficient']=1.0
                            k=k+1
                    tmpGdict[G]=tmpVdict
            self.basisfunctions[orbit]=tmpGdict
        return

    def GetIrrepsCoefs(self,msg):
        """
        Get coefficients of the irreps appearing in the decomposition of the magnetic representation in the combobx
        """
        self.decompirredcomp=msg.data
        return

    def GetLatticeTranslations(self,msg):
        """
        Get generators of the translations
        """
        self.translations=msg.data      
        return

    def OnBasisfunctionChangeText(self,event):
        """
        Read mixing coefficient from table
        """
        item0=event.GetItem()
        coefficient=item0.GetText()
        row=event.m_itemIndex
        item1 = self.basisfunctionstable.GetItem(itemId=row, col=0)
        bvlabel=item1.GetText() 
        item2 = self.basisfunctionstable.GetItem(itemId=row, col=2)
        orbit = item2.GetText()
        item3 = self.basisfunctionstable.GetItem(itemId=row, col=3)
        irrep = item3.GetText()

        try:
            self.basisfunctions[orbit][irrep][bvlabel]['coefficient']=complex(coefficient)
        except ValueError:
            self.basisfunctions[orbit][irrep][bvlabel]['coefficient']=1.0
            dlg = wx.MessageDialog(None, "Not a complex number", "Error", wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
        return

    def OnBasisfunctionEdit(self,event):
        """
        Make first column not editable 
        """
        if (event.m_col == 0) or (event.m_col == 2):
            #do not allow editing column 0 and 2
            event.Veto()
        else:
            pass
        return

    def OnCalcClicked(self,event):
        """
        calculate magnetic structure when ok button pressed
        """
        self.magdomains.Clear()
        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().sendMessage(("kvector"), self.kvector)
        else:
            Publisher.sendMessage(("kvector"), self.kvector)
        self.MakeModesTable()
        return

    def OnDRAWClicked(self,event):
        """
        send information to draw magnetic structure when button is pressed
        """
        retval=[self.littlegroup,self.atomsandmoments,self.kvector]
        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().sendMessage(("spins"), retval)
        else:
            Publisher.sendMessage(("spins"), retval)
        return

    def OnSaveClicked(self,event):
        """
        save magnetic structure to file, together with symmetry elements that 
        leave magnetic structure invariants
        """
        dlg = wx.FileDialog(self, message="Save magneti structure as...",defaultDir= os.getcwd(),defaultFile="" "",wildcard= "*.pym", \
                    style=wx.SAVE|wx.OVERWRITE_PROMPT)
        result = dlg.ShowModal()
        path = dlg.GetPath()
        filename = dlg.GetFilename()
        dlg.Destroy()
        #print magnetic structure
        grid=self.magstrutable
        if result == wx.ID_OK:
            table=PrettyTable(['Label','Position','Mx','My','Mz'])
            textfile = open(path, 'w')
            cols = grid.GetColumnCount() 
            for idx in range(grid.GetItemCount()): 
                table.add_row([str(grid.GetItem(idx, col).GetText()) for col in range(cols-1)]) 
            txt = table.get_string()
            textfile.write(txt)
            textfile.write("\n")
            titleline="List of symmetry elements that leave the magnetic structure invariant\n"
            textfile.write(titleline)
            textfile.write("-"*len(titleline)+"\n")
            for k in range(len(self.subgroupopsym)):
                textfile.write(self.subgroupopsym[k])
            textfile.close()
            return
        elif result == wx.ID_CANCEL:    #Either the cancel button was pressed or the window was closed
            return
        return

    def OnResetClicked(self,event):
        """
        empty table
        """
        self.basisfunctionstable.DeleteAllItems()
        self.magstrutable.DeleteAllItems()
        self.chosenbasisfunctions.clear()
        self.atomsandmoments.clear()
        self.cb_orbits.Clear()
        self.cb_irreps.Clear()
        self.cb_modes.Clear()
        self.SetOrbits()
        self.magdomains.Clear()
        return

    def SetIrreps(self):
        """
        Set irreps appearing in the decomposition in combox
        """
        #remove irreps that do not appear in the decomposition
        empty_keys = [k for k,v in self.decompirredcomp[self.selected_crystalorbit].iteritems() if not v]
        for k in empty_keys:
            del self.decompirredcomp[self.selected_crystalorbit][k]
        labelslist=self.decompirredcomp[self.selected_crystalorbit].keys()
        labelslist.sort()
        for l in labelslist:
            self.cb_irreps.Append(l)
        return

    def GetOrbits(self,msg):
        """
        Get positions in each crystallographic orbit and set orbit labels in the combobx
        """
        self.crystalorbits = msg.data
        self.cb_irreps.Clear()
        self.cb_orbits.Clear()
        self.cb_modes.Clear()
        self.basisfunctionstable.DeleteAllItems()
        #self.magstrutable.DeleteAllItems()
        self.SetOrbits()

    def SetOrbits(self):
        for o in self.crystalorbits.iterkeys():
            self.cb_orbits.Append(o)
        return

    def SetSelectedBasisFunction(self):
        """
        Set the labels of the selected basisfunction in table to define mixing coefficients
        """
        index=self.basisfunctionstable.InsertStringItem(sys.maxint,self.selected_bv)
        txt='1.0'
        self.basisfunctionstable.SetStringItem(index,1,txt)
        self.basisfunctionstable.SetStringItem(index,2,self.selected_crystalorbit)
        self.basisfunctionstable.SetStringItem(index,3,self.selected_irrep)
        #save selected bv's
        try:
            val=self.chosenbasisfunctions[self.selected_crystalorbit]
            val.append((self.selected_irrep,self.selected_bv))
            self.chosenbasisfunctions[self.selected_crystalorbit]=val
        except:
            self.chosenbasisfunctions[self.selected_crystalorbit]=[(self.selected_irrep,self.selected_bv)]
        return
 
    def SetMagModes(self):
        """
        Set magnetic mode labels in combobox
        """
        bvdict=self.basisfunctions[self.selected_crystalorbit][self.selected_irrep]
        keylist = bvdict.keys()
        keylist.sort()
        for k in keylist:
            self.cb_modes.Append(k)
        return

    def SelectOrbit(self, event):
        """
        get selected orbit
        """
        self.selected_irrep=None
        self.cb_irreps.Clear()
        self.cb_modes.Clear()
        self.selected_crystalorbit = self.cb_orbits.GetString(event.GetSelection())
        self.SetIrreps()
        return

    def SelectIrrep(self, event):
        """
        get selected irrep
        """
        self.selected_irrep = self.cb_irreps.GetString(event.GetSelection())
        self.cb_modes.Clear()
        self.SetMagModes()
        return

    def SelectMagMode(self, event):
        """
        get selected basis function
        """
        self.selected_bv = self.cb_modes.GetString(event.GetSelection())
        self.SetSelectedBasisFunction()
        return

    def MakeModesTable(self):
        """
        calculate magnetic moment directions for each atom in the orbit 
        """
        #add atoms on the border of the cell
        translations=[]
        for T in self.translations:
            for tx in range(-1,2):
                for ty in range(-1,2):
                    for tz in range(-1,2):
                        translations.append(np.array([tx,ty,tz])+T)
        for orbit, bv in self.chosenbasisfunctions.iteritems():
            try:
                #atomic positions
                atpos=self.crystalorbits[orbit]['xyz']   #atomic coordinates
                label=self.crystalorbits[orbit]['label'] #atom label
                self.atomsandmoments[label]=dict()
                tmpdict=dict()
                for GV in bv:
                    G=GV[0] #irrep label
                    V=GV[1] #basis function label
                    op=self.basisfunctions[orbit][G][V]['orderparameter']
                    coef=self.basisfunctions[orbit][G][V]['coefficient']
                    tmpdict=self.Moments(orbit, op, atpos, translations, label, coef)
                    for key, val in tmpdict.iteritems():
                        try:
                            self.atomsandmoments[label][key]+=val
                        except KeyError:
                            self.atomsandmoments[label][key]=val
            except KeyError:   #if more than one magnetic ion skip previously selected atoms
                pass  
        self.ModesTable()
        return 

    def ModesTable(self):
        """
        Set magnetic moment directions in table
        """
        for atlabel, magstrudict in self.atomsandmoments.iteritems():
            index=0
            for xyz, moment in magstrudict.iteritems():
                 self.magstrutable.InsertStringItem(index,atlabel+str(index))
                 x=str(xyz[0])
                 y=str(xyz[1])
                 z=str(xyz[2])
                 coord='('+x+','+y+','+z+')'
                 mx=moment[0]
                 my=moment[1]
                 mz=moment[2]
                 self.magstrutable.SetStringItem(index,1,coord)
                 self.magstrutable.SetStringItem(index,2,str(mx))
                 self.magstrutable.SetStringItem(index,3,str(my))
                 self.magstrutable.SetStringItem(index,4,str(mz))
                 m=np.sqrt(mx*mx+my*my+mz*mz) 
                 self.magstrutable.SetStringItem(index,5,str(round(m,2)))
                 index+=1
        self.MagneticDomains()
        return

    def MagneticDomains(self):
        """
        orientation domains are calculated by applying the magnetic representation 
        matrix on the basis vectors and the result is compared with the 
        original vector to check of it is invariant
        """
        self.subgroupopsym=[]
        self.magdomains.WriteText("List of symmetry elements that leave the magnetic structure invariant:\n"+
                                                        "(the operations with -1 are associated with time-reversal)\n")
        for orbit, bv in self.chosenbasisfunctions.iteritems():
            try:
                #atomic positions
                atpos=self.crystalorbits[orbit]['xyz']   #atomic coordinates
                label=self.crystalorbits[orbit]['label'] #atom label
                tmpdict=dict()
                basisfunction=np.zeros((3*len(self.crystalorbits[orbit]['xyz']),1), dtype='complex')
                for GV in bv:
                    G=GV[0] #irrep label
                    V=GV[1] #basis function label
                    op=self.basisfunctions[orbit][G][V]['orderparameter']
                    coef=self.basisfunctions[orbit][G][V]['coefficient']
                    try:
                        basisfunction += coef*op
                    except ValueError: #it is a corep
                        basisfunction=np.zeros((2*3*len(self.crystalorbits[orbit]['xyz']),1), dtype='complex')
                        basisfunction += coef*op
                #for symop, mat in self.magneticrepresentation[orbit].iteritems():
                for dic in self.littlegroup:
                    mat=self.magneticrepresentation[orbit][dic['seitzsymbol']]  #matrix of the magnetic representation for the symmetry element in the list 
                    if np.allclose(basisfunction+np.conj(basisfunction),np.dot(mat,basisfunction)+np.dot(mat,np.conj(basisfunction))) == True:
                        #self.magdomains.WriteText(symop+",+1\n")
                        #self.subgroupopsym.append(symop+",+1\n") #save a copy to print to file
                        det = np.linalg.det(dic["matrix"][0:3,0:3])
                        s=dic["jonessymbol"]
                        s=s.replace('+1/2','')
                        s=s.replace('+1/4','')
                        s=s.replace('+3/4','')
                        s=s.replace('+1/3','')
                        s=s.replace('+2/3','')
                        s=s.replace('+1/6','')
                        if det == -1:
                            s=s.replace('+x','-mx')
                            s=s.replace('+y','-my')
                            s=s.replace('+z','-mz')
                            s=s.replace('-x','+mx')
                            s=s.replace('-y','+my')
                            s=s.replace('-z','+mz')
                        else:
                            s=s.replace('+x','+mx')
                            s=s.replace('+y','+my')
                            s=s.replace('+z','+mz')
                            s=s.replace('-x','-mx')
                            s=s.replace('-y','-my')
                            s=s.replace('-z','-mz')

                        self.magdomains.WriteText(dic["jonessymbol"]+",+1,  "+s+"\n")
                        self.subgroupopsym.append(dic["jonessymbol"]+",+1,  "+s+"\n")

                    elif np.allclose(basisfunction+np.conj(basisfunction),-np.dot(mat,basisfunction)-np.dot(mat,np.conj(basisfunction))) == True:
                        #self.magdomains.WriteText(symop+",-1\n")
                        #self.subgroupopsym.append(symop+",-1\n")
                        det = np.linalg.det(dic["matrix"][0:3,0:3])
                        s=dic["jonessymbol"]
                        s=s.replace('+1/2','')
                        s=s.replace('+1/4','')
                        s=s.replace('+3/4','')
                        s=s.replace('+1/3','')
                        s=s.replace('+2/3','')
                        s=s.replace('+1/6','')
                        if det == -1:
                            s=s.replace('+x','+mx')
                            s=s.replace('+y','+my')
                            s=s.replace('+z','+mz')
                            s=s.replace('-x','-mx')
                            s=s.replace('-y','-my')
                            s=s.replace('-z','-mz')
                        else:
                            s=s.replace('+x','-mx')
                            s=s.replace('+y','-my')
                            s=s.replace('+z','-mz')
                            s=s.replace('-x','mx')
                            s=s.replace('-y','my')
                            s=s.replace('-z','mz')
                        self.magdomains.WriteText(dic["jonessymbol"]+",-1,  "+s+"\n")
                        self.subgroupopsym.append(dic["jonessymbol"]+",-1,  "+s+"\n")
            except KeyError:   #if more than one magnetic ion skip previously selected atoms
                pass 
        return

    def Moments(self, orbit, op, atpos, translations, label, coef):
        """
        calculate magnetic moment directions and populate table
        """
        self.magstrutable.DeleteAllItems()
        try:                            #irreps
            modes = np.reshape(op,(len(atpos),3))
        except ValueError:              #coreps
            modes = np.reshape(op,(2*len(atpos),3))
            #reverse moment because of time-reversal
            modes[len(atpos):,:]*=-1.
        modes=coef*modes
        retval=dict()
        for k in range(0,len(atpos)):
            for T in translations:
                xyz=atpos[k]+T
                if ((xyz>1).sum() > 0) or ((xyz<0).sum() > 0):  #skip atoms outside cell
                    pass
                else:
                    phase = np.exp(-1j*2.0*np.pi*np.dot(self.kvector,T))
                    mx=modes[k][0]*phase
                    mx=mx+np.conj(mx)
                    my=modes[k][1]*phase
                    my=my+np.conj(my)
                    mz=modes[k][2]*phase
                    mz=mz+np.conj(mz)
                    mx=mx.real
                    mx=round(mx,2)
                    my=my.real
                    my=round(my,2)
                    mz=mz.real
                    mz=round(mz,2)
                    retval[tuple(xyz)]=np.array([mx,my,mz])
        return retval

import wx.lib.agw.ultimatelistctrl as ulc
from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin

class AutoWidthListCtrl(ulc.UltimateListCtrl, ListCtrlAutoWidthMixin):
    def __init__(self,parent):
        ulc.UltimateListCtrl.__init__(self, parent,id=wx.ID_ANY, size=(-1,300), agwStyle=wx.LC_REPORT|ulc.ULC_HAS_VARIABLE_ROW_HEIGHT|ulc.ULC_EDIT_LABELS)
        ListCtrlAutoWidthMixin.__init__(self)

class MagneticModesPage(wx.Panel):
    def __init__(self,parent,spacegroup, atomicpositions):
        wx.Panel.__init__(self, parent)

        self.ffion=[  'Am2', 'Am3', 'Am4', 'Am5', 'Am6', 'Am7', 
                      'Ce2', 'Co0', 'Co1', 'Co2', 'Co3', 'Co4', 'Cr0', 'Cr1', 'Cr2', 'Cr3', 'Cr4', 'Cu0', 'Cu1', 'Cu2', 'Cu3', 'Cu4',
                      'Dy2', 'Dy3', 
                      'Er2', 'Er3', 'Eu2', 'Eu3', 
                      'Fe0', 'Fe1', 'Fe2', 'Fe3', 'Fe4',
                      'Gd2', 'Gd3',
                      'Ho2', 'Ho3',
                      'Mn0', 'Mn1', 'Mn2', 'Mn3', 'Mn4', 'Mn5', 'Mo0', 'Mo1',
                      'Nb0', 'Nb1', 'Nd2', 'Nd3', 'Ni0', 'Ni1', 'Ni2', 'Ni3', 'Ni4', 'Np3', 'Np4', 'Np5', 'Np6',
                      'Pd0', 'Pd1', 'Pr3', 'Pu3', 'Pu4', 'Pu5', 'Pu6',
                      'Rh0', 'Rh1', 'Ru0', 'Ru1',
                      'Sc0', 'Sc1', 'Sc2', 'Sm2', 'Sm3',
                      'Tb2', 'Tb3', 'Tc0', 'Tc1', 'Ti0', 'Ti1', 'Ti2', 'Ti3', 'Tm2', 'Tm3',
                      'U3', 'U4', 'U5',
                      'V0', 'V1', 'V2', 'V3', 'V4',
                      'Y0', 'Yb2', 'Yb3', 'Zr0', 'Zr1']  #magnetic form factor


        mainbox_sizer=wx.BoxSizer(wx.VERTICAL)
        staticbox1 = wx.StaticBox(self,-1,label="Input")
        vbox = wx.StaticBoxSizer(staticbox1,wx.VERTICAL)
        hbox1 = wx.BoxSizer(wx.HORIZONTAL)
        txt='space group: ' + spacegroup
        self.spacegroup=spacegroup
        spacegroup_info = wx.StaticText(self, label=txt)
        hbox1.Add(spacegroup_info,1,wx.EXPAND| wx.ALL)
        vbox.Add(hbox1, flag=wx.EXPAND|wx.LEFT|wx.RIGHT|wx.TOP, border=10)
        vbox.Add((-1, 10))
        hbox2 = wx.BoxSizer(wx.HORIZONTAL)
        self.magneticatoms = collections.OrderedDict()
        for label, value in atomicpositions.iteritems():
            if value['magnetic'] == True:
                tmp = dict()
                tmp['atom']=value['atom']
                tmp['x']=value['x']
                tmp['y']=value['y']
                tmp['z']=value['z']
                tmp['color']=value['color']
                tmp['radius']=value['radius']
                tmp['FQ']=''
                self.magneticatoms[label]=tmp
        self.list_ctrl = AutoWidthListCtrl(self)
        self.list_ctrl.InsertColumn(0, 'Label', wx.LIST_FORMAT_CENTER,width=200)
        self.list_ctrl.InsertColumn(1, 'x', wx.LIST_FORMAT_CENTER,width=200)
        self.list_ctrl.InsertColumn(2, 'y', wx.LIST_FORMAT_CENTER,width=200)
        self.list_ctrl.InsertColumn(3, 'z', wx.LIST_FORMAT_CENTER,width=200)
        self.list_ctrl.InsertColumn(4, 'formfactor', wx.LIST_FORMAT_CENTER,width=150)

        self.ffchoice_index=dict()
        self.ffchoice=dict()
        self.list_ctrl_idx=0
        for label, val in self.magneticatoms.iteritems():
             self.list_ctrl.InsertStringItem(self.list_ctrl_idx,label)
             x=val['x']
             self.list_ctrl.SetStringItem(self.list_ctrl_idx,1,x)
             y=val['y']
             self.list_ctrl.SetStringItem(self.list_ctrl_idx,2,y)
             z=val['z']
             self.list_ctrl.SetStringItem(self.list_ctrl_idx,3,z)
             self.ffchoice[label] = wx.ComboBox(self.list_ctrl, choices=self.ffion, style=wx.TE_PROCESS_ENTER)
             self.ffchoice_index[self.ffchoice[label].GetId()] = self.list_ctrl_idx
             self.list_ctrl.SetItemWindow(self.list_ctrl_idx, 4, self.ffchoice[label], expand = True)
             self.ffchoice[label].Bind(wx.EVT_TEXT, self.SelectMagIon)
             self.list_ctrl_idx+=1

        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnChooseAtom, self.list_ctrl)
        hbox2.Add(self.list_ctrl,2,wx.ALL)
        vbox.Add(hbox2, flag=wx.EXPAND|wx.LEFT|wx.RIGHT, border=10)
        vbox.Add((-1, 10))
        vbox3 = wx.BoxSizer(wx.VERTICAL)   #propagation vector
        klabel = wx.StaticText(self, label='propagation vector (rlu):')
        vbox3.Add(klabel,1,wx.EXPAND| wx.ALL)
        grid_sizer = wx.FlexGridSizer(1,7,10,20) 
        k_x = wx.StaticText(self,label="kx =",)
        k_y = wx.StaticText(self,label="ky =") 
        k_z = wx.StaticText(self,label="kz =") 
        tc1 = wx.TextCtrl(self,id = 1, style = wx.TE_PROCESS_ENTER)
        self.Bind(wx.EVT_TEXT, self.GetKVector, tc1)
        tc2 = wx.TextCtrl(self,id = 2, style = wx.TE_PROCESS_ENTER)
        self.Bind(wx.EVT_TEXT, self.GetKVector, tc2)
        tc3 = wx.TextCtrl(self,id = 3, style = wx.TE_PROCESS_ENTER)
        self.Bind(wx.EVT_TEXT, self.GetKVector, tc3)
        grid_sizer.AddMany([(k_x,1,wx.ALIGN_CENTER), (tc1, 1, wx.EXPAND),
                           (k_y,1,wx.ALIGN_CENTER), (tc2, 1, wx.EXPAND),
                           (k_z,1,wx.ALIGN_CENTER), (tc3, 1, wx.EXPAND),(wx.StaticText(self,label="(rlu)"),1,wx.ALIGN_CENTER)])
        vbox3.Add(grid_sizer,1,wx.EXPAND| wx.ALL)
        vbox.Add(vbox3, flag=wx.EXPAND|wx.LEFT|wx.RIGHT, border=10)
        vbox.Add((-1, 10))

        OK_button = wx.Button(self, label='OK',size=(70, 30))
        OK_button.Bind(wx.EVT_BUTTON, self.OnOKClicked)
        OK_button.SetToolTip(wx.ToolTip("click to calculate basis vectors"))
        border1 = wx.BoxSizer()
        border1.Add(OK_button,0,wx.ALL | wx.EXPAND, border=10)
        vbox.Add(border1,0)
        vbox.Add((-1, 10))

        hbox5=wx.BoxSizer(wx.HORIZONTAL)    #ouptut of result calculations
        self.tcout = wx.TextCtrl(self,wx.ID_ANY, size=(-1,300),style=wx.TE_MULTILINE|wx.TE_READONLY|wx.HSCROLL)
        old_font = self.tcout.GetFont()
        new_font = wx.Font(old_font.GetPointSize(), wx.FONTFAMILY_MODERN, old_font.GetStyle(), old_font.GetWeight())
        self.tcout.SetFont(new_font)

        hbox5.Add(self.tcout,1,wx.EXPAND| wx.ALL)
        vbox.Add(hbox5, flag=wx.EXPAND|wx.LEFT|wx.RIGHT, border=10)
        vbox.Add((-1, 10))

        hbox6=wx.BoxSizer(wx.HORIZONTAL)    #k-domains
        self.kdomain_list_ctrl =  wx.ListCtrl(self, size=(-1,400),
                          style=wx.LC_REPORT
                         |wx.BORDER_SUNKEN
                         )
        self.kdomain_list_ctrl.InsertColumn(0, 'Seitz Symbol', width = 400)
        self.kdomain_list_ctrl.InsertColumn(1, 'K-domain', width = 400)
        hbox6.Add(self.kdomain_list_ctrl,2,wx.ALL)
        vbox.Add(hbox6, flag=wx.EXPAND|wx.LEFT|wx.RIGHT, border=10)

        mainbox_sizer.Add(vbox,0,wx.EXPAND | wx.ALL)
        self.SetSizer(mainbox_sizer)
        mainbox_sizer.Fit(self)
        self.kvector = np.array([])
        self.selectedmagion = None

    def GetKVector(self, event):
        id = event.GetId()-1
        string = event.GetString()
        try:
            self.kvector=np.resize(self.kvector,(1,3))
            self.kvector[0][id] = float(string) 
        except ValueError:
            pass
        return

    def KDomains(self, cosetsrepoflglist):
        """
        calculate the K-domains created by the cosets of the little group
        relative to the space group
        """
        kdomainslist=[]
        kvector=self.kvector.tolist()[0]
        for symop in cosetsrepoflglist:
            tmp=dict()
            tmp['seitzsymbol']=symop['seitzsymbol']
            tmp['kvector']=np.dot(symop['matrix'][0:3,0:3],kvector)
            kdomainslist.append(tmp)
        return kdomainslist

    def SelectMagIon(self, event):
        idx = event.GetId()
        row = self.ffchoice_index[idx]
        item = self.list_ctrl.GetItem(row,0)
        label=item.GetText()
        item = self.list_ctrl.GetItem(row,4)
        self.FQ=self.ffchoice[label].GetValue()
        self.magneticatoms[label]['FQ']=self.FQ
        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().sendMessage(("selectedmagneticions"), self.magneticatoms)
        else:
            Publisher.sendMessage(("selectedmagneticions"), self.magneticatoms)
        return

    def OnChooseAtom(self,event):
        """
        select magnetic ion
        """
        self.selectedmagion=event.GetText()
        return

    def OnOKClicked(self,event):
        """
        start symmetry calculations
        """
        #in case no atom has been selected
        if self.selectedmagion == None:
              dlg = wx.MessageDialog(None, "Please select a magnetic atom", "Error", wx.OK | wx.ICON_ERROR)
              dlg.ShowModal()
              dlg.Destroy()
              return

        matches = [s for s in self.ffion if self.magneticatoms[self.selectedmagion]['FQ']==s]
        if len(matches)==0:
              dlg = wx.MessageDialog(None, "Please select a form factor", "Error", wx.OK | wx.ICON_ERROR)
              dlg.ShowModal()
              dlg.Destroy()
              return

        if not self.kvector.size:
              dlg = wx.MessageDialog(None, "Please define a propagation vector", "Error", wx.OK | wx.ICON_ERROR)
              dlg.ShowModal()
              dlg.Destroy()
              return
        sys.stdout=self.tcout
        title = self.spacegroup
        spacegroup=self.spacegroup
        x=float(self.magneticatoms[self.selectedmagion]['x'])
        y=float(self.magneticatoms[self.selectedmagion]['y'])
        z=float(self.magneticatoms[self.selectedmagion]['z'])
        atomicposition=[x,y,z]
        kvector=self.kvector.tolist()[0]

        axialvector = True

        #DO NOT MODIFY THE FOLLOWING CODE
        #--------------------------------
        title = title + ' and atomic position ' + str(atomicposition)
        print title 
        print '-'*len(title)

        BV=BasisVectors(spacegroup,kvector,np.array(atomicposition),axialvector)
        orbits=BV.SplitOrbits()
        #return coefficients of the decomposition of the magnetic/mechanical representation
        if axialvector == False:
            coefrep=BV.CalcMechanicalRep(orbits)
        else:
            coefrep=BV.CalcMagneticRep(orbits)
        orbitnumber=0
        #loop over crystallographic orbits
        basisvectors=dict()
        magneticrepresentation=dict()
        for orbit, positions in orbits.iteritems():
            print '\nOrbit:',orbitnumber+1
            print 'Atomic positions:\n', ('\n'.join([' '.join(['{:10}'.format(item) for item in row])
                              for row in positions]))
            #projection operator
            BV.ProjectionOperator(BV.allrepresentations[orbitnumber])
            #basis functions
            BV.CalcBasis(coefrep[orbit])
            BV.PrintBasisVectors()
            #normalise the basis vectors with the sum of the squares of the coefficients (see izyumov and naish)
            basisvectors[orbit]=BV.Vectors
            retval=copy.deepcopy(BV.Vectors)
            for G, bvdict in retval.iteritems():
                for bvkey, bvlist in bvdict.iteritems():
                    l=0
                    for bv in bvlist:
                        normsq=0.
                        for k in range(0,len(bv)):
                            normsq+=bv[k]*np.conj(bv[k]) #+bv[k+1]*np.conj(bv[k+1])+bv[k+2]*np.conj(bv[k+2])
                        basisvectors[orbit][G][bvkey][l]/=normsq
                        l=l+1
            magneticrepresentation[orbit]=BV.allrepresentations[orbitnumber]
            BV.Vectors={}
            orbitnumber+=1

        sys.stdout=sys.__stdout__
        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().sendMessage(("kvector"), kvector)
        else:
            Publisher.sendMessage(("kvector"), kvector)
        orbitdict=dict()
        k=1
        for orbit, positions in orbits.iteritems():
            #add a different label to each orbit
            tmp=dict()
            tmp['xyz']=positions
            tmp['label']=self.selectedmagion+'_'+str(k)
            k=k+1
            orbitdict[orbit]=tmp
        if wx.VERSION <=  ('2.8.11.0'):
            Publisher().sendMessage(("orbits"), orbitdict)
            Publisher().sendMessage(("irrepscoefs"), coefrep)
            Publisher().sendMessage(("magneticrepresentation"), magneticrepresentation)
            Publisher().sendMessage(("modes"), basisvectors)
            Publisher().sendMessage(("littlegroup"), BV.symmetrygroup)
            Publisher().sendMessage(("cosetrepresentatives"), BV.GetCosetsRepresentativesOfLG())
            bravais=BV.GetBravais()
            Publisher().sendMessage(("latticetranslations"), BV.GetLatticeTranslations(bravais)['translations'])
        else:
            Publisher.sendMessage(("orbits"), orbitdict)
            Publisher.sendMessage(("irrepscoefs"), coefrep)
            Publisher.sendMessage(("magneticrepresentation"), magneticrepresentation)
            Publisher.sendMessage(("modes"), basisvectors)
            Publisher.sendMessage(("littlegroup"), BV.symmetrygroup)
            Publisher.sendMessage(("cosetrepresentatives"), BV.GetCosetsRepresentativesOfLG())
            bravais=BV.GetBravais()
            Publisher.sendMessage(("latticetranslations"), BV.GetLatticeTranslations(bravais)['translations'])
        
        cosetsrepoflglist=BV.GetCosetsRepresentativesOfLG()
        kdomainslist=self.KDomains(cosetsrepoflglist)
        k=0
        for kdomain in kdomainslist:
            self.kdomain_list_ctrl.InsertStringItem(k,kdomain['seitzsymbol'])
            self.kdomain_list_ctrl.SetStringItem(k,1, str(kdomain['kvector']))            
            k=k+1
        return
        
class MagneticParametersBook(wx.Frame):
    def __init__(self, mcrys2cart, mrec2cart, parent=None, spacegroup = None, atomicpositions = None):
        wx.Frame.__init__(self, parent, title="Magnetic Structure",size=(1000, 1000))
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        errormsg=['No spacegroup defined','No magnetic atom defined']
        if spacegroup is None:
            dlg = wx.MessageDialog(None, errormsg[0], "Error", wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
        elif atomicpositions is None:
            dlg = wx.MessageDialog(None, errormsg[1], "Error", wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
        elif spacegroup is None or atomicpositions is None:
            self.Destroy()
        else:
            # Here we create a panel and a notebook on the panel
            panel = wx.Panel(self)
            book = wx.Notebook(panel)

            # create the page windows as children of the notebook
            page1 = MagneticModesPage(book,spacegroup,atomicpositions)
            help='select a magnetic atom in the table\n'
            help+='choose a form factor in the list\n'
            help+='enter the coordinates of the propagation vector\n'
            help+='and click OK to obtain the symmetry allowed basis functions'
            page1.SetToolTip(wx.ToolTip(help))
            page2 = DrawMagneticModesPage(book,mcrys2cart,mrec2cart)
            # add the pages to the notebook with the label to show on the tab
            book.AddPage(page1, "Symmetry Analysis")
            book.AddPage(page2, "Draw Magnetic Modes")
            # finally, put the notebook in a sizer for the panel to manage
            # the layout
            sizer = wx.BoxSizer()
            sizer.Add(book, 1, wx.EXPAND)
            panel.SetSizer(sizer)
            self.Centre()
            self.Show()

    def OnClose(self,event):
        self.Destroy()
        
#if __name__ == "__main__":
#    app = wx.App()
#    MagneticParametersBook(None).Show()
#    app.MainLoop()


