#!/usr/bin/python

__author__ = "B. Roessl"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "B. Roessli"
__email__ = "roessli@psi.ch"
__status__ = "Testing"



import matplotlib
matplotlib.use('WXAgg')

from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.backends.backend_wx import NavigationToolbar2Wx
from matplotlib.figure import Figure


import copy
import numpy as np
import wx
from Lattice import Lattice
from RadialFormFactor import RadialFormFactor

import sys
#sys.path.append('../..')
try:
    from symmetry.prettytable import *
except ImportError:
    sys.exit('Module Pretty Table could not be imported')

class NeutronPowderPatternPage(wx.Panel, Lattice, RadialFormFactor):
    def __init__(self,parent,lattice,atomicpositions,kvector,magmomdict):
        wx.Panel.__init__(self, parent)
        Lattice.__init__(self,lattice)
        RadialFormFactor.__init__(self)
 
        self.atomicpositions=atomicpositions
        self.kvector=kvector
        self.wl=None
        self.ttheta_min=None
        self.ttheta_max=None
        self.magmomdict=magmomdict

        mainbox_sizer=wx.BoxSizer(wx.VERTICAL)
        staticbox1 = wx.StaticBox(self,label="Parameters")
        vbox1 = wx.StaticBoxSizer(staticbox1,wx.VERTICAL)
        #wavelength
        lamda = wx.StaticText(self, label=u'\N{GREEK SMALL LETTER LAMDA}'+":")
        #resolution paramters
        Upar = wx.StaticText(self, label="U:")
        Vpar = wx.StaticText(self, label="V:") 
        Wpar = wx.StaticText(self, label="W:") 

        tc1 = wx.TextCtrl(self,id = 1, style = wx.TE_PROCESS_ENTER)
        self.Bind(wx.EVT_TEXT, self.GetParameters, tc1)
        tc2 = wx.TextCtrl(self,id = 2, style = wx.TE_PROCESS_ENTER)
        self.Bind(wx.EVT_TEXT, self.GetParameters, tc2)
        tc3 = wx.TextCtrl(self,id = 3, style = wx.TE_PROCESS_ENTER)
        self.Bind(wx.EVT_TEXT, self.GetParameters, tc3)
        tc4 = wx.TextCtrl(self,id = 4, style = wx.TE_PROCESS_ENTER)
        self.Bind(wx.EVT_TEXT, self.GetParameters, tc4)

        grid_sizer1 = wx.FlexGridSizer(1, 11, 10, 20) 
        grid_sizer1.AddMany([(lamda,1,wx.ALIGN_CENTER),(tc1, 1, wx.EXPAND), (wx.StaticText(self,label="   ")),
                             (Upar,1,wx.ALIGN_CENTER), (tc2, 1, wx.EXPAND), (wx.StaticText(self,label="   ")),
                             (Vpar,1,wx.ALIGN_CENTER), (tc3, 1, wx.EXPAND), (wx.StaticText(self,label="   ")),
                             (Wpar,1,wx.ALIGN_CENTER), (tc4, 1, wx.EXPAND)])
        vbox1.Add(grid_sizer1,0,wx.EXPAND | wx.ALL, 20)
        mainbox_sizer.Add(vbox1,0,wx.EXPAND | wx.ALL)

        #plot options
        staticbox2 = wx.StaticBox(self,label="Plot options")
        vbox2 = wx.StaticBoxSizer(staticbox2,wx.VERTICAL)
        ttheta_min = wx.StaticText(self, label="2"+u'\N{GREEK SMALL LETTER THETA}'+"_min:")
        ttheta_max = wx.StaticText(self, label="2"+u'\N{GREEK SMALL LETTER THETA}'+"_max:") 
        tc5 = wx.TextCtrl(self,id = 5, style = wx.TE_PROCESS_ENTER)
        self.Bind(wx.EVT_TEXT, self.GetParameters, tc5)
        tc6 = wx.TextCtrl(self,id = 6, style = wx.TE_PROCESS_ENTER)
        self.Bind(wx.EVT_TEXT, self.GetParameters, tc6)
        OK_button = wx.Button(self, label='OK')
        OK_button.Bind(wx.EVT_BUTTON, self.OnOKClicked)

        grid_sizer2 = wx.FlexGridSizer(1, 7, 10, 20) 
        grid_sizer2.AddMany([(ttheta_min,1,wx.ALIGN_CENTER),(tc5, 1, wx.EXPAND),(wx.StaticText(self,label="   ")),
                             (ttheta_max,1,wx.ALIGN_CENTER),(tc6, 1, wx.EXPAND), (wx.StaticText(self,label="   ")),
                             (OK_button,1,wx.ALIGN_CENTER)])
        vbox2.Add(grid_sizer2,0,wx.EXPAND | wx.ALL, 20)
        mainbox_sizer.Add(vbox2,0,wx.EXPAND | wx.ALL)

        #matplot
        staticbox3 = wx.StaticBox(self)
        vbox3 = wx.StaticBoxSizer(staticbox3,wx.VERTICAL)
        self.figure = Figure()
        self.axes = self.figure.add_subplot(111)

        self.canvas = FigureCanvas(self, -1, self.figure)
        self.zoom_factory()
        vbox3.Add(self.canvas, 1, wx.LEFT | wx.TOP | wx.GROW)
        CLEAR_button = wx.Button(self, label='Clear Plot')
        CLEAR_button.Bind(wx.EVT_BUTTON, self.OnCLEARClicked)
        vbox3.Add(CLEAR_button, 0, wx.ALL | wx.EXPAND, 10)
        mainbox_sizer.Add(vbox3,0,wx.EXPAND | wx.ALL)

        staticbox4 = wx.StaticBox(self)
        hbox4=wx.StaticBoxSizer(staticbox4, wx.HORIZONTAL)
        self.tcout = wx.TextCtrl(self,wx.ID_ANY, size=(-1,200),style=wx.TE_MULTILINE|wx.TE_READONLY|wx.HSCROLL)
        old_font = self.tcout.GetFont()
        new_font = wx.Font(old_font.GetPointSize(), wx.FONTFAMILY_MODERN, old_font.GetStyle(), old_font.GetWeight())
        self.tcout.SetFont(new_font)
        hbox4.Add(self.tcout,1,wx.EXPAND| wx.ALL)
        mainbox_sizer.Add(hbox4,0,wx.EXPAND | wx.ALL)

        self.Fit()
        self.SetSizer(mainbox_sizer)

    def GetParameters(self, event):
        """
        get wavelength, resolution parameters and 2theta min. and max. from widgets
        """
        id = event.GetId()
        value = event.GetString()
        if len(value) != 0 and value !='-':
           try:
              if id == 1:
                  self.wl=float(value)
              elif id ==2:
                  self.Ures=float(value)
              elif id==3:
                  self.Vres=float(value)
              elif id==4:
                  self.Wres=float(value)
              elif id==5:
                  self.ttheta_min=float(value)
              elif id==6:
                  self.ttheta_max=float(value) 
           except ValueError:
              dlg = wx.MessageDialog(None, "Wrong Input", "Error", wx.OK | wx.ICON_ERROR)
              dlg.ShowModal()
              dlg.Destroy()
        return

    def OnCLEARClicked(self,event): 
        """
        clear plot
        """
        self.axes.clear()
        self.canvas.draw()
        self.tcout.Clear()

    def OnOKClicked(self, event):
        """
        calculate structure factors in 2theta range
        """
        old_stdout = sys.stdout
        #redirect output
        sys.stdout=self.tcout
        HKLlist,dspacingslist=self.HKLlist()
        if len(HKLlist) == 0:
            dlg = wx.MessageDialog(None, "No peak in selected range", "Error", wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return  
        lorenzcorlist=self.LorenzFactor(dspacingslist)
        tthetabragglist=self.Bragg(dspacingslist)
        fwhmlist=self.Resolution(tthetabragglist)
        Qarray=2.*np.pi/np.array(dspacingslist)
        QQarray=Qarray*Qarray
        QQ=QQarray/np.power(4.*np.pi,2)
        SQdict=self.FourierTransformDensity(HKLlist)
        Mc=self.lat2cart()
        Mc[:,0]=Mc[:,0]/np.linalg.norm(Mc[:,0])
        Mc[:,1]=Mc[:,1]/np.linalg.norm(Mc[:,1])
        Mc[:,2]=Mc[:,2]/np.linalg.norm(Mc[:,2])
        Mcblock=np.kron(np.eye(len(HKLlist)),Mc)
        rl=self.reciprocallattice()
        Mcr=self.recilat2cart()
        Mcrblock=np.kron(np.eye(len(HKLlist)),Mcr)
        HKL=np.array(HKLlist)
        Qarray=np.reshape(HKLlist,(3*len(HKLlist),1))
        Qarray=np.dot(Mcrblock,Qarray)
        Qarray=np.reshape(Qarray,(len(HKLlist),3))
        Qnorm=np.linalg.norm(Qarray,axis=1)
        Qarray=np.divide(Qarray,Qnorm[:,np.newaxis])

        #form factor
        FQ=dict()
        for label, propdict in self.atomicpositions.iteritems():
            ion=propdict['FQ']
            self.GetRadialCoefficients(ion)
            F0list=[]
            for Q in QQ:
                F0list.append(self.J0q(Q))
            FQ[label]=np.array(F0list)

        #magnetic interaction vector
        MQperp=np.zeros((len(HKLlist),3),dtype='complex')
        for label, MQ in SQdict.iteritems():
            idx = label.find('_')
            label_at=label[:idx]
            fq=np.array(FQ[label_at])
            tmp=np.dot(Mcblock,np.reshape(MQ.T,(3*len(HKLlist),1)))
            MQ=np.reshape(tmp,(len(HKLlist),3))
            MQ=np.multiply(fq,MQ.T).T
            MQperp+=0.2695*np.cross(Qarray,np.cross(MQ,Qarray))
        IQmag=np.einsum('ij,ij->i', MQperp, np.conj(MQperp))
        IQmag=IQmag.real
        IQmag=np.multiply(IQmag.T,np.array(lorenzcorlist)).T

        #powder diffraction pattern        
        step=0.05
        xaxes=np.arange(self.ttheta_min-step,self.ttheta_max+step,step)
        Imag=np.zeros(len(xaxes)+1)
        for k in range(0,len(IQmag)):
            if np.array_equal(self.kvector,np.array([0.,0.,0.])):
                IQmag[k]*=2.0
            tthetas,Ipowder=self.Gauss(tthetabragglist[k],fwhmlist[k],IQmag[k],step)
            inds = np.digitize(tthetas,xaxes)
            tmp=np.zeros(len(xaxes)+1)
            tmp[inds]=Ipowder
            Imag=Imag+tmp
        self.DrawPowderPattern(xaxes[1:],Imag[1:-1])

        #list results
        ttheta_sorted=sorted(tthetabragglist)
        idx=sorted(range(len(tthetabragglist)),key=lambda x:tthetabragglist[x])
        fwhm_sorted=[fwhmlist[i] for i in idx]
        HKL_sorted=[HKLlist[i] for i in idx]
        IQmag_sorted=[IQmag[i] for i in idx]
        Lorcor_sorted=[lorenzcorlist[i] for i in idx]
        MQperpx=MQperp[:,0]
        MQperp_sortedx=[MQperpx[i] for i in idx]
        MQperpy=MQperp[:,1]
        MQperp_sortedy=[MQperpy[i] for i in idx]
        MQperpz=MQperp[:,2]
        MQperp_sortedz=[MQperpz[i] for i in idx]
        table=PrettyTable(['(H,K,L)','2Theta','MQperp','I(Q)','fwhm','Lorenz Factor'])
        table.float_format["2Theta"] = "+3.2"
        table.float_format["I(Q)"] = "+3.4"
        table.float_format["fwhm"] = "+2.2"
        table.float_format["Lorenz Factor"] = "+3.2"
        for k in range(0,len(HKLlist)):
            MQ='['+"{:.3f}".format(MQperp_sortedx[k])+", "+"{:.3f}".format(MQperp_sortedy[k])+"," "{:.3f}".format(MQperp_sortedz[k])+"]"
            table.add_row([HKL_sorted[k],ttheta_sorted[k],MQ,IQmag_sorted[k],fwhm_sorted[k],Lorcor_sorted[k]])
        print table
        #restore stdout
        sys.stdout=old_stdout
        return

    def Bragg(self,dspacingslist):
        """
        return scattering angle (deg)
        """
        sintheta=self.wl/2.0/np.array(dspacingslist)
        ttheta=2.*np.arcsin(sintheta)/np.pi*180.
        return list(ttheta)

    def DrawPowderPattern(self,xaxis,yaxis):
        """
        plot powder spectrum
        """
        self.figure.gca().set_xlabel('2'+r'$\Theta$'+'(deg.)')
        self.axes.plot(xaxis, yaxis)
        self.canvas.draw()
        return

    def zoom_factory(self, base_scale = 1.2):
        """
        zoom/unzoom the plot by scrolling the center button of the mouse
        """
        ax=self.axes
        def zoom(event):
            cur_xlim = ax.get_xlim()
            cur_ylim = ax.get_ylim()

            xdata = event.xdata # get event x location
            ydata = event.ydata # get event y location

            if event.button == 'down':
                # deal with zoom in
                scale_factor = 1 / base_scale
            elif event.button == 'up':
                # deal with zoom out
                scale_factor = base_scale
            else:
                # deal with something that should never happen
                scale_factor = 1

            new_width = (cur_xlim[1] - cur_xlim[0]) * scale_factor
            new_height = (cur_ylim[1] - cur_ylim[0]) * scale_factor
            try:
                relx = (cur_xlim[1] - xdata)/(cur_xlim[1] - cur_xlim[0])
                rely = (cur_ylim[1] - ydata)/(cur_ylim[1] - cur_ylim[0])
                ax.set_xlim([xdata - new_width * (1-relx), xdata + new_width * (relx)])
                #ax.set_ylim([ydata - new_height * (1-rely), ydata + new_height * (rely)])
                ax.set_ylim([0., ydata + new_height * (rely)])
                ax.figure.canvas.draw()
            except TypeError:
                pass
        fig = ax.get_figure() # get the figure of interest
        fig.canvas.mpl_connect('scroll_event', zoom)
        return zoom

    def FourierTransformDensity(self,hkllist):
        """
        return fourier transform of magnetization
        """
        unitcelldict=copy.deepcopy(self.magmomdict)
        sqdict=dict()
        for label, xyzdict in self.magmomdict.iteritems():
            for xyz, momentdirection in xyzdict.iteritems():
                atpos=np.array(xyz)
                if ((atpos==1.).sum() > 0):
                    #delete ions at the border of the cell
                    del unitcelldict[label][xyz]
            xyzmat=np.matrix(unitcelldict[label].keys())
            hklmat=np.matrix(hkllist)
            tf=np.exp(-1j*2.*np.pi*np.dot(hklmat,xyzmat.T))
            spinmat=np.matrix(unitcelldict[label].values())
            MQvecx=np.sum(np.multiply(tf,spinmat[:,0].T),axis=1)
            MQvecy=np.sum(np.multiply(tf,spinmat[:,1].T),axis=1)
            MQvecz=np.sum(np.multiply(tf,spinmat[:,2].T),axis=1)
            MQvec=np.append(MQvecx,MQvecy,axis=1)
            MQvec=np.append(MQvec,MQvecz,axis=1)
            sqdict[label]=MQvec.T
        return sqdict

    def HKLlist(self):
        """
        return list of hkl between 2th_min and 2th_max
        """
        k0=self.kvector
        self.metrictensor()
        self.contmetrictensor()
        d_max=self.wl/2./np.sin(np.deg2rad(self.ttheta_min/2.))
        d_min=self.wl/2./np.sin(np.deg2rad(self.ttheta_max/2.))
        HKLlist=[]
        dspacinglist=[]
        Hmax=0
        for hmax in range(1,100):
            h=np.array([hmax-np.abs(k0[0]),0,0])
            dsp=self.dspacing(h)
            if dsp > d_min:
                pass
            else:
                Hmax=hmax
                break
        Kmax=0
        for kmax in range(1,100):
            k=np.array([0,kmax-np.abs(k0[1]),0])
            dsp=self.dspacing(k)
            if dsp > d_min:
                pass
            else:
                Kmax=kmax
                break
        Lmax=0
        for lmax in range(1,100):
            l=np.array([0,0,lmax-np.abs(k0[2])])
            dsp=self.dspacing(l)
            if dsp > d_min:
                pass
            else:
                Lmax=lmax
                break

        HKLMm=[]
        HKLMp=[]
        dspacinglistm=[]
        dspacinglistp=[] 
        HKLlist.append(np.array([0.,0.,0.]))
        for h in range(-Hmax,Hmax+1):
            for k in range(-Kmax,Kmax+1):
                for l in range(-Lmax,Lmax+1):
                     hkl=np.array([h,k,l])
                     Q=hkl-k0
                     dspacing=self.dspacing(Q)
                     if dspacing!=np.NINF and dspacing > d_min and dspacing < d_max:
                         HKLMm.append(Q)
                         dspacinglistm.append(self.dspacing(Q))
                     Q=hkl+k0
                     dspacing=self.dspacing(Q)
                     if dspacing!=np.NINF and dspacing > d_min and dspacing < d_max:
                         HKLMp.append(Q)
                         dspacinglistp.append(self.dspacing(Q))
        if np.array_equal(k0,np.array([0.,0.,0.])):
            return HKLMm,dspacinglistm
        else:
            return HKLMm+HKLMp, dspacinglistm+dspacinglistp

    def Gauss(self,two_th,fwhm,scale,step):
        """
        multiply structure factor with gaussian function
        """
        scat_angles=np.arange(two_th-3.*fwhm,two_th+3.*fwhm,step)
        two_th_ar=two_th*np.ones(len(scat_angles))
        scat_int=1.665109/fwhm/np.sqrt(np.pi)*np.exp(-((scat_angles-two_th_ar)/fwhm*1.665109)**2)*step
        return scat_angles,scale*scat_int

    def LorenzFactor(self,dspacinglist):
        """
        return Lorenz factor for powder
        """     
        dsp=np.array(dspacinglist)
        theta=np.arcsin(self.wl/2./dsp)
        return list(1./np.sin(theta)/np.sin(2.*theta))

    def Resolution(self,tthetalist=None):
        """
        return fwhm of gaussian as a function of theta
        """
        u=self.Ures
        v=self.Vres
        w=self.Wres
        th_angle=np.array(tthetalist)/180.*np.pi/2.
        fwhm2=u*np.tan(th_angle)**2.+v*np.tan(th_angle)+w
        #set zero/negative values to 0.05
        fwhm2[fwhm2 <= 0] = 0.05
        return list(np.sqrt(fwhm2))

class PowderDiffractionBook(wx.Frame):
    def __init__(self, parent=None, lattice = None, atomicpositions = None, kvector = None, magmomdict = None):
        wx.Frame.__init__(self, parent, title="Simulations",size=(700, 1000))
        self.Bind(wx.EVT_CLOSE, self.OnPowderClose)
        errormsg=['No lattice defined','No magnetic atom defined']

        if lattice == None:
            dlg = wx.MessageDialog(None, errormsg[0], "Error", wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
        elif atomicpositions == None:
            dlg = wx.MessageDialog(None, errormsg[1], "Error", wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
        elif lattice == None or atomicpositions == None:
            self.Destroy()
        else:
            # Here we create a panel and a notebook on the panel
            panel = wx.Panel(self)
            book = wx.Notebook(panel)

            # create the page windows as children of the notebook
            page1 = NeutronPowderPatternPage(book,lattice,atomicpositions, kvector,magmomdict)
            # add the pages to the notebook with the label to show on the tab
            book.AddPage(page1, "Powder Pattern")
            # finally, put the notebook in a sizer for the panel to manage
            # the layout
            sizer = wx.BoxSizer()
            sizer.Add(book, 1, wx.EXPAND)
            panel.SetSizer(sizer)
            self.Centre()
            self.Show()

    def OnPowderClose(self,event):
        self.Destroy()
        return


